#include <stdio.h>  
#include <malloc.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "linprog.h"
#include "alldist.h"

#define epsran 0.00001          // accuracy of randomization

#define LIMIT 0.000000001   // when pdf of CF-1 is under LIMIT compute only hyperexp.

int NUMBEROFROUNDS;        // Number of rounds
int NUMBEROFITERATIONS;  // Max. number of modifications in 1 round
int intpr;                // number of intervals in main part fitting to integrate

double **a;
int *izrov, *iposv;

int lastcdfindex=0;
int samples;

int cdfnum;
double *cdfpoints,*cdfvalues;

struct measures
{
  double m1,m2,m3,ent,ad,rad;
};

int method;
int  **NUMI, **ROWI, **COLI;    
double  **INTENSITY, ***PROB;
double  ***Q;		       // matrices for integral computation 
int ACT,ACTSTART;	       // index of matrix under computation 
double mtail1,mtail2;	       // limits to compute hyperexp. part
double mtail3;		       // limit to compute measures
double tail;		       // limit to compute the main part

double sptailstart,sptailparam;
int sptailtype;

/* constants for Gaussian integral */
double C[12]={-.49078031712335963,-.45205862818523743,-.38495133709715234,
	-.29365897714330872,-.18391574949909010,-.06261670425573445,
	.06261670425573445, .18391574949909010, .29365897714330872,
	.38495133709715234, .45205862818523743, .49078031712335963};
double A[12]={ .02358766819325591, .05346966299765921, .08003916427167311,
	.10158371336153296, .11674626826917740, .12457352290670139,
	.12457352290670139, .11674626826917740, .10158371336153296,	      
	.08003916427167311, .05346966299765921, .02358766819325591};	      

int numofstates, nonzeros;

double *derp, *derb;    /* derivatives with respect to b and beta */
double *obs;            /* set of observations */
double *weight;         /* weights of observations*/
double *dist;           /* continouos distribution */
double *mass;           /* pdf of the distribution */
double *p, *beta, *bestp, *bestbeta;
double *dp, *dbeta;
double *actp, *actbeta, *stp, *stbeta;
double *prb,*prb2;
int l, ol;                      /* length of chain and observations*/
int od;                         /* length of distribution */
double Deltap,Deltabeta;        /* maximum change in the variables */


// variables describing the mixture of exp. part
double *h,*lambda;
int lh;			// number of exponentials mixed
double adj;		// to define distance of adjacent points
double ph;		// sum of initial pr.-s of exponentials mixed

// functions to describe the distribution
double (* pdf)(double,double,double);	
double (* distmean)(double,double);
double (* cdf)(double,double,double);
double (* invcdf)(double,double,double);
double (* ccdf)(double,double,double);
double (* invccdf)(double,double,double);
double param1, param2;


/* write parameters to screen */
void printp(double *p, double *b)
{
  int i;
  printf("\n");
  for(i=1;i<=l;i++)
    printf("p[%d]: %f, beta[%d]: %f\n", i, p[i], i, b[i]);
  printf("\n");
}

/* density of The Mixture */
double mixture(double t)
{
  double ft=0;
  int i;
  
  for(i=0;i<lh;i++) ft+=h[i]*lambda[i]*exp(-lambda[i]*t);
  return ft;
}

/* build part of mixed exponentials given by ccdf*/
void build_exp(void)
{
  int i,j;
  double x1,x2,t,m,b,sum=0,tailconstant;
  
  if(sptailtype!=0)
    {
      if(sptailtype==1)
	tailconstant=ccdf(sptailstart,param1,param2)/
	  pow(sptailstart,-sptailparam);
      else
	tailconstant=ccdf(sptailstart,param1,param2)/
	  exp(-pow(sptailstart,sptailparam));
    }
  
  m=pow(mtail2/mtail1,1/((double)lh-1+adj));	// distance of quantiles
  b=pow(m,adj);				// distance of adjecent points 
  //printf("%e %e m: %e\n",mtail2/mtail1,1/((double)lh-1),m);
  t=mtail1;
  for(i=0;i<lh-1;i++) t*=m;
  //printf("t: %e\n",t);
  sum=0;
  
  printf("  Points used during tail fitting:\n");
  for(i=0;i<lh;i++)
    {
      //printf(" t: %1.4e, b*t=%1.4e\n",t,b*t);
      if(t>sptailstart&&sptailtype!=0)
	if(sptailtype==1)
	  x1=tailconstant*pow(t,-sptailparam);
	else
	  x1=tailconstant*exp(-pow(t,sptailparam));
      else
	x1=ccdf(t,param1,param2);
      if(b*t>sptailstart&&sptailtype!=0)
	if(sptailtype==1)
	  x2=tailconstant*pow(b*t,-sptailparam);
	else
	  x2=tailconstant*exp(-pow(b*t,sptailparam));
      else
	x2=ccdf(b*t,param1,param2);
      printf("   ccdf(%1.4e)=%1.4e\n   ccdf(%1.4e)=%1.4e\n",b*t,x2,t,x1);
      //printf("ccdf(t)=%1.4e, ccdf(b*t)=%1.4e\n",x1,x2);
      for(j=0;j<i;j++)
	{
	  x1-=h[j]*exp(-lambda[j]*t);
	  x2-=h[j]*exp(-lambda[j]*t*b);
	}
      lambda[i]=1/(b-1)/t*log(x1/x2);
      h[i]=x1*exp(lambda[i]*t);
      sum+=h[i];	
      t=t/m;
      //printf("t: %e\n",t);
    }
  
  //printf("Mixture: \n\th\t\tlambda\n");
  //for(i=0;i<lh;i++) printf("\t%e\t%e\n",h[i],lambda[i]); 
  //printf("Mixture: %e\n", mixture(0));
}

// ----------------------
// read samples from file
// ----------------------
void loadsamples(char *fn)
{
  FILE *f;
  int i1=0,success,bytes;
  double tmp1,tmp2;

  cdfpoints=NULL;
  cdfvalues=NULL;
  f=fopen(fn,"r");
  while(!feof(f))
    {
      success=1;
      if((bytes=fscanf(f, "%le", &tmp1))==0 || bytes==-1)
	success=0;
      //printf("bytes: %d", bytes);
      if(success)
	{
	  i1++;
	  cdfpoints=(double*)realloc(cdfpoints,i1*sizeof(double));
	  cdfvalues=(double*)realloc(cdfvalues,i1*sizeof(double));
	  cdfpoints[i1-1]=tmp1;
	  cdfvalues[i1-1]=1.0;
	  //printf(" %le %le\n", cdfpoints[i1-1], cdfvalues[i1-1]);
	}
    }
  cdfnum=i1;
}


// ------------------
// read cdf from file
// ------------------
void loadcdf(char *fn)
{
  FILE *f;
  int i1=0,success,bytes;
  double tmp1,tmp2;

  //printf("hehe\n");
  cdfpoints=NULL;
  cdfvalues=NULL;
  f=fopen(fn,"r");
  while(!feof(f))
    {
      success=1;
      if((bytes=fscanf(f, "%le", &tmp1))==0 || bytes==-1)
	success=0;
      //printf("bytes: %d", bytes);
      if((bytes=fscanf(f, "%le", &tmp2))==0 || bytes==-1)
	success=0;
      //printf("bytes: %d", bytes);
      if(success)
	{
	  i1++;
	  cdfpoints=(double*)realloc(cdfpoints,i1*sizeof(double));
	  cdfvalues=(double*)realloc(cdfvalues,i1*sizeof(double));
	  cdfpoints[i1-1]=tmp1;
	  cdfvalues[i1-1]=tmp2;
	  //printf(" %le %le\n", cdfpoints[i1-1], cdfvalues[i1-1]);
	}
    }
  cdfnum=i1;
}

// ------------------
// read ccdf from file
// ------------------
void loadccdf(char *fn)
{
  FILE *f;
  int i1=0,success,bytes;
  double tmp1,tmp2;

  cdfpoints=NULL;
  cdfvalues=NULL;
  f=fopen(fn,"r");
  while(!feof(f))
    {
      success=1;
      if((bytes=fscanf(f, "%le", &tmp1))==0 || bytes==-1)
	success=0;
      //printf("bytes: %d", bytes);
      if((bytes=fscanf(f, "%le", &tmp2))==0 || bytes==-1)
	success=0;
      //printf("bytes: %d", bytes);
      if(success)
	{
	  i1++;
	  cdfpoints=(double*)realloc(cdfpoints,i1*sizeof(double));
	  cdfvalues=(double*)realloc(cdfvalues,i1*sizeof(double));
	  cdfpoints[i1-1]=tmp1;
	  cdfvalues[i1-1]=1-tmp2;
	  //printf(" %le %le\n", cdfpoints[i1-1], cdfvalues[i1-1]);
	}
    }
  cdfnum=i1;
}


// ------------------------
// mean based on loaded cdf
// ------------------------
double mean_loaded(double y, double z)
{
  int i1;
  double s;
  
  //printf(" s: %1.4e\n", s);
  s=cdfpoints[0]*cdfvalues[0];
  for(i1=1;i1<cdfnum;i1++)
    {
      //printf(" s: %1.4e\n", s);
      s+=cdfpoints[i1]*(cdfvalues[i1]-cdfvalues[i1-1]);
    }
  
  return s;
}

double pdf_loaded(double x, double p1, double p2)
{
  int i1=0;
  double tmp1;

  while(cdfpoints[i1]<x) 
    {
      i1++;
      if(i1==cdfnum)
	{
	  return 0;
	}
    }
  if(i1==0)
    {
      return cdfvalues[0]/2/cdfpoints[0];
    }
  if(i1==1)
    {
      tmp1=cdfvalues[0]/2-(cdfvalues[2]+cdfvalues[1])/2;
      return tmp1/cdfpoints[0];
    }
  
  tmp1=-(cdfvalues[i1-1]+cdfvalues[i1-2])/2+(cdfvalues[i1]+cdfvalues[i1-1])/2;
  //printf( "%1.4e %1.4e\n", tmp1,cdfpoints[i1]-cdfpoints[i1-1]);
  return tmp1/(cdfpoints[i1]-cdfpoints[i1-1]);
}

	
double cdf_loaded(double x, double p1, double p2)
{
  int i1=0;
  
  if(x<cdfpoints[0]) 
    {
      //printf("%1.8e\n", 0.0);
      return 0.0;
    }
  if(lastcdfindex>0)
    if(x>cdfpoints[lastcdfindex-1])
      i1=lastcdfindex;
  //printf(" cdf_loaded(%1.8e)=", x);
  while(x>cdfpoints[i1]) 
    {
      i1++;
      if(i1==cdfnum) 
	{
	  //printf("%1.8e\n", 1.0);
	  return 1.0;
	}
    }
  lastcdfindex=i1-1;
  //printf("%1.8e\n", cdfvalues[i1-1]);
  return cdfvalues[i1-1];
}

double invcdf_loaded(double x, double p1, double p2)
{
  int i1=0;
  
  if(x<cdfvalues[0]) return cdfpoints[0];
  while(cdfvalues[i1]<x)
    {
      i1++;
      if(i1==cdfnum)
	{
	  return cdfpoints[cdfnum-1];
	}
    }
  return cdfpoints[i1];
}

double ccdf_loaded(double x, double p1, double p2)
{
  return 1-cdf_loaded(x,0,0);
} 

double invccdf_loaded(double x, double p1, double p2)
{
  return invcdf_loaded(1-x,0,0);
} 

void createcdf(void)
{
  int i1;
  double sum=0;

  ol=cdfnum;
  weight=(double *)malloc(sizeof(double)*(ol+1));
  obs=(double *)malloc(sizeof(double)*(ol+1));
  for(i1=0;i1<cdfnum;i1++)
    {
      obs[i1+1]=cdfpoints[i1];
      weight[i1+1]=cdfvalues[i1];
      sum+=cdfvalues[i1];
    }
  cdfvalues[0]=cdfvalues[0]/sum;
  for(i1=1;i1<cdfnum;i1++)
    {
      cdfvalues[i1]=cdfvalues[i1]/sum+cdfvalues[i1-1];
    }
}

void createsamples()
{
  int i1;

  ol=cdfnum;
  weight=(double *)malloc(sizeof(double)*(ol+1));
  obs=(double *)malloc(sizeof(double)*(ol+1));

  obs[1]=cdfpoints[0];
  weight[1]=cdfvalues[0];
  for(i1=1;i1<cdfnum;i1++)
    {
      obs[i1+1]=cdfpoints[i1];
      weight[i1+1]=cdfvalues[i1]-cdfvalues[i1-1];
    }
  //for(i1=1;i1<=cdfnum;i1++)
  //printf(" %1.4e, %1.4e\n", obs[i1], weight[i1]);
}  


/* read observations and associated weights */
void readobs(char *fn)
{
	int i;
	FILE *f;
	f=fopen(fn,"r");
	fscanf(f, "%d", &ol);
	
	weight=(double *)malloc(sizeof(double)*(ol+1));
	obs=(double *)malloc(sizeof(double)*(ol+1));
	
	printf("ol: %d\n", ol);
	for(i=1;i<=ol;i++)
	{
        fscanf(f, "%le", &obs[i]);
        fscanf(f, "%le", &weight[i]);
	}
	/*for(i=1;i<=ol;i++)*/
	/*        printf("%f %f\n", obs[i], weight[i]);*/
	fclose(f);
}

double readdist(char *fn)
{
	int i;
	double mean=0;
	FILE *f;
	
	f=fopen(fn,"r");
	fscanf(f, "%d", &od);
	
	printf("od: %d\n", od);
	mass=(double *)malloc(sizeof(double)*(od+1));
	dist=(double *)malloc(sizeof(double)*(od+1));
	
	/*printf("od: %d\n", od);*/
	for(i=1;i<=od;i++)
	{
		fscanf(f, "%le", &dist[i]);
		fscanf(f, "%le", &mass[i]);
		mean+=dist[i]*mass[i];
	}
	/*for(i=1;i<=ol;i++)*/
	/*	printf("%d %f\n", obs[i], weight[i]);*/
	fclose(f);
	return mean*(dist[2]-dist[1]);
}

double mymin(double m1, double m2)
{
	if( m1>m2 ) return m2;
	return m1;
}

double mymax(double m1, double m2)
{
	if( m1>m2 ) return m1;
	return m2;
}

/* part of randomization */
double MDNRIS (double P)
{
	
	double A, W, WI, SN, SD;
	double  XINF = 1.7014E+38;
	double SQRT2 = 1.414214;
	double EP = 1.1921E-07;
	double G0 = 0.1851159E-3 , G1 = -0.2028152E-2;
	double G2 = -0.1498384, G3 = 0.1078639E-1;
	double H0 = 0.9952975E-1, H1 = 0.5211733;
	double H2 = -0.6888301E-1;
	double Y;
	
	A = P+P;
	W = sqrt(-log(A+(A-A*A)));
	WI = 1/W;
	SN = ((G3*WI+G2)*WI+G1)*WI;
	SD = ((WI+H2)*WI+H1)*WI+H0;
	Y = W + W*(G0+SN/SD);
	Y = -Y*SQRT2;
	return Y;
}

/* part of randomization */
long gaussmax( double qt, double eps)
{
	double y;
	long max;
	
	y = MDNRIS( eps);
	max = (long) floor(-y*(sqrt(qt*2))+qt)+1;
	return max;
}

/* part of randomization */
long gaussmin( double qt, double eps)
{
	double y;
	long min;
	
	y = MDNRIS( eps);
	min = (long) floor(y*(sqrt(qt*2))+qt)+1;
	return min;
}

/* randomization */
void integr_rand( double step, double time, double eps)
{
  double q = 0, qt;
  long i, j;
  long n, index, num;
  long kmin, kmax;
  double tprec, stot;
  double somlog;
  double *w, *w2;
  long nc;
  
  n=l;
  if(ACT!=0) n++;
  
  if( (w = ( double  * ) malloc( n * sizeof(double))) == NULL )
    {
      printf("Not Enough Memory");
      exit(1);
    }
  if( (w2 = ( double  * ) malloc( n * sizeof(double))) == NULL )
    {
      printf("Not Enough Memory");
      exit(1);
    }
  index = - 1;
  for(i=0;i<n;i++)
    {
      /*printf( "i:%d, index : %d\n", i, index);*/
      index = index + NUMI[ACT][i];
      if( ( 0 - INTENSITY[ACT][index]) > q ) q = ( 0 - INTENSITY[ACT][index]);
    }
  qt = q * step;
  index = 0;
  for( i = 0; i < n; i++)
    {
      num = NUMI[ACT][i];
      for( j = 0; j < num; j++)
	INTENSITY[ACT][ index + j ] /= q;
      INTENSITY[ACT][ index + j - 1 ] += 1;
      index = index + num;
    }
  if( qt < 10 )
    {
      kmin = 0;
      tprec = exp(-qt);
      stot = tprec;
      while( stot < eps/2 )
	{
	  kmin ++;
	  tprec = tprec * qt / kmin;
	  stot = stot + tprec;
	}
      kmin --;
    }
  else
    {
      kmin = gaussmin( qt, eps/2);
    }
  if( kmin <= 0 ) kmin = 1;
  if( qt < 10 )
    {
      kmax = 0;
      tprec = exp(-qt);
      stot = tprec;
      while( stot < ( 1 - eps/2) )
	{
	  kmax ++;
	  tprec = tprec * qt / kmax;
	  stot = stot + tprec;
	}
    }
  else
    {
      kmax = gaussmax( qt, eps/2);
    }
  
  somlog = 0;
  tprec = exp(-qt);
  stot = tprec;
  for( i = 0; i < n ; i++)
    {
      w[i] = PROB[ACT][ACTSTART][i];
      PROB[ACT][ACTSTART][i] = w[i] * tprec;
    }
  
  for( nc = 1; nc <= kmax; nc++)
    {
      index = -1;
      for( i = 0; i < n ; i++ )
	{
	  w2[i] = 0;
	  index += NUMI[ACT][i];
	  for( j = index - NUMI[ACT][i] + 1; j <= index; j++)
	    w2[i] = w2[i] + w[ROWI[ACT][j]] * INTENSITY[ACT][j];
	}
      for( i = 0; i < n; i++)
	w[i] = w2[i];
      
      if( nc < kmin)
	somlog += log(nc);
      
      if( nc == kmin)
	{
	  somlog += log(nc);
	  tprec = exp( -qt + kmin*log(qt) - somlog);
	  stot += tprec;
	  
	  for( i = 0; i < n; i++)
	    PROB[ACT][ACTSTART][i] += w[i] * tprec;
	}
      
      if( nc > kmin)
	{
	  tprec = tprec * qt / nc;
	  stot += tprec;
	  
	  for( i = 0; i < n; i++)
	    PROB[ACT][ACTSTART][i] += w[i] * tprec;
	  
	}
    }
  
  /*norma();*/
  
  index = -1;
  for( i = 0; i < n; i++)
    {
      num = NUMI[ACT][i];
      index += num;
      INTENSITY[ACT][ index ] -= 1;
      for( j = 0; j < num; j++)
	INTENSITY[ACT][ index - j ] *= q;
    }
  free(w);
  free(w2);
}

/* setting parameters for simplex */
void lp(void)
{
  int n,m1,m2,m3,m,i,j;
  int icase;
  
  n=2*l;
  m1=2*l+l-1;
  m2=0;
  m3=1;
  m=3*l;
  
  /* clear a */
  for(i=1;i<=m+2;i++)
    for(j=1;j<=n+1;j++) a[i][j]=0;
  /* objective function */
  a[1][1]=0;
  for(i=1;i<=l;i++)
    {
      a[1][1+i]=derp[i];
      a[1][l+1+i]=derb[i];
    }
  
  /* 2*l m1 type ineqaulities */
  for(i=2;i<=l+1;i++)
    {
      a[i][1]=2*p[i-1]*Deltap;
      a[i+l][1]=mymin(beta[i-1],Deltabeta)+mymin(1-beta[i-1],Deltabeta);
      a[i][i]=-1;
      a[i+l][i+l]=-1;
    }
  /* l-1 m1 type inequalities describing the odering of p values*/
  for(i=1;i<=l-1;i++)
    {
      a[2*l+1+i][1+i]=1;
      a[2*l+1+i][2+i]=-1;
      a[2*l+1+i][1]=-(p[i+1]-p[i]+Deltap*p[i]-Deltap*p[i+1]);
      if(a[2*l+1+i][1]<0) a[2*l+1+i][1]=0;
    }
  
  /* eqaulity */
  for(i=1;i<=l;i++)
    {
      a[3*l+1][1]+=mymin(Deltabeta,beta[i]);
      a[3*l+1][l+1+i]=-1;
    }
  
  /* calling simplex */
  //printf(" Calling simplex\n");
  //fflush(stdout);
  simplx(a, m, n, m1, m2, m3, &icase, izrov, iposv);
  //printf(" Simplex finished\n");
  //fflush(stdout);
  
  /* results */
  for(i=1;i<=n;i++)
    {
      if(izrov[i]<=n)
	{
	  if(izrov[i]<=l) { dp[izrov[i]]=0-Deltap*p[izrov[i]]; }
	  else    { dbeta[izrov[i]-l]=0-mymin(Deltabeta,beta[izrov[i]-l]); }
	}
    }
  for(i=1;i<=m;i++)
    {
      if(iposv[i]<=n)
	{
	  if(iposv[i]<=l) { dp[iposv[i]]=a[i+1][1]-Deltap*p[iposv[i]]; }
	  else { dbeta[iposv[i]-l]=a[i+1][1]-mymin(Deltabeta,beta[iposv[i]-l]); }
	}
    }
  
  /*printf("Parameters:\n");*/
  /*printp(p,beta);*/
  /*printf("Derivative:\n");*/
  /*printp(derp,derb);*/
  /*printf("Result of lp:\n");*/
  /*printp(dp,dbeta);*/
  
}

/* write parameters of the CTMC-s */
/* if a=0 -> original CTMC */
/* if a=1 -> one phase added CTMC */
/* void printstr(int a) */
/* { */
/*   int i,j,k,n; */
  
/*   n=l; */
/*   if(a!=0) n++; */
  
/*   j=0; */
/*   for(i=0;i<n;i++) */
/*     { */
/*       printf("%d\n", i); */
/*       for(k=0;k<NUMI[a][i];k++) */
/* 	printf("%d %d %f\n", COLI[a][j], ROWI[a][j], INTENSITY[a][j++]); */
/*     } */
  
/*   for(i=0;i<n;i++) printf(" %f ", PROB[a][ACTSTART][i]); */
/*   printf("\n"); */
/* } */

/* rare matrice of original CTMC */
void makeQ(double *pp, double *bb)
{
  int i;
  
  ACTSTART=0;
  for(i=0;i<l-1;i++)
    {
      NUMI[0][i]=2;
      PROB[0][ACTSTART][i]=bb[i+1];
    }
  NUMI[0][l-1]=1;
  PROB[0][ACTSTART][l-1]=bb[l];
  for(i=0;i<l-1;i++)
    {
      COLI[0][2*i]=i;     
      COLI[0][2*i+1]=i;
      ROWI[0][2*i]=i+1; 
      ROWI[0][2*i+1]=i;
      INTENSITY[0][2*i]=pp[i+2];
      INTENSITY[0][2*i+1]=-pp[i+1];
    }
  COLI[0][2*(l-1)]=l-1;
  ROWI[0][2*(l-1)]=l-1;
  INTENSITY[0][2*(l-1)]=-pp[l];
  
  for(ACTSTART=1;ACTSTART<=l;ACTSTART++)
    for(i=0;i<l;i++)
      if(ACTSTART-1==i) PROB[0][ACTSTART][i]=1;
      else      PROB[0][ACTSTART][i]=0;
}

/* rare matrice of one phase added CTMC */
void makeser(void)
{
  int i;
  
  for(ACT=1;ACT<=l;ACT++)
    {
      for(i=0;i<l;i++)
	NUMI[ACT][i]=2;
      NUMI[ACT][l]=1;
      
      p[0]=p[ACT];
      for(i=0;i<l;i++)
	{
	  COLI[ACT][2*i]=i;     COLI[ACT][2*i+1]=i;
	  ROWI[ACT][2*i]=i+1;   ROWI[ACT][2*i+1]=i;
	  INTENSITY[ACT][2*i]=p[i+1];
	  if(i>=0) INTENSITY[ACT][2*i+1]=-p[i];
	}
      COLI[ACT][2*l]=l;
      ROWI[ACT][2*l]=l;
      INTENSITY[ACT][2*l]=-p[l];
      
      for(ACTSTART=1;ACTSTART<=l;ACTSTART++)
	for(i=0;i<=l;i++)
	  if(ACTSTART==i) PROB[ACT][ACTSTART][i]=1;
	  else    PROB[ACT][ACTSTART][i]=0;
    }
}

void memalloc(void)
{
  int i,j,m=3*l,n=2*l;
 
  a = (double **)malloc( (m+2+1)*sizeof(double *));
  for(i=1; i<=m+2; i++)
    a[i] = (double *)malloc( (n+1+1)*sizeof(double));
  izrov = (int *)malloc(sizeof(int)*(n+1));
  iposv = (int *)malloc(sizeof(int)*(m+1));
  
  PROB=(double ***)malloc((l+1)*sizeof(double**));
  Q=(double ***)malloc(7*sizeof(double**));
  for(i=0;i<=6;i++)
    {
      if( (Q[i] = ( double ** ) malloc( (l+1) * sizeof(double*))) == NULL )
	{
	  printf("\nNot Enough Memory1\n");
	  exit(1);
	}
      for(j=1;j<=l;j++)
	if( (Q[i][j] = ( double * ) malloc( (l+1) * sizeof(double))) == NULL )
	  {
	    printf("\nNot Enough Memory1\n");
	    exit(1);
	  }
    }
  for(i=0;i<=l;i++)
    {
      if( (PROB[i] = ( double ** ) malloc( (l+1) * sizeof(double*))) == NULL )
	{
	  printf("\nNot Enough Memory1\n");
	  exit(1);
	}
      for(j=0;j<=l;j++)
	if( (PROB[i][j] = ( double * ) malloc( (l+1) * sizeof(double))) == NULL )
	  {
	    printf("\nNot Enough Memory1\n");
	    exit(1);
	  }
    }
	
  NUMI=(int **)malloc((l+1)*sizeof(int *));
  numofstates=l+1;
  for(i=0;i<=l;i++)
    {
      if( (NUMI[i] = ( int  * ) malloc( (unsigned long) numofstates * sizeof(int))) == NULL )
	{
	  printf("Not Enough Memory2");
	  exit(1);
	}
      numofstates=l+2;
    }
  
  COLI=(int **)malloc((l+1)*sizeof(int *));
  ROWI=(int **)malloc((l+1)*sizeof(int *));
  INTENSITY=(double **)malloc((l+1)*sizeof(double *));
  nonzeros=2*l+1;
  for(i=0;i<=l;i++)
    {
      if( (COLI[i] = ( int  * ) malloc( nonzeros * sizeof(int))) == NULL )
	{
	  printf("Not Enough Memory3");
	  exit(1);
	}
      if( (ROWI[i] = ( int  * ) malloc(  nonzeros * sizeof(int))) == NULL )
	{
	  printf("Not Enough Memory4");
	  exit(1);
	}
      if( (INTENSITY[i] = ( double  * ) malloc( nonzeros * sizeof(double))) == NULL )
	{
	  printf("Not Enough Memory5");
	  exit(1);
	}
      nonzeros=2*l+3;
    }
	
  if(lh>0)
    {
      lambda = (double *)malloc( (lh)*sizeof(double));
      h = (double *)malloc( (lh)*sizeof(double));
    }
  prb2 = (double *)malloc( (l+1)*sizeof(double));
  prb = (double *)malloc( (l+1)*sizeof(double));
  p = (double *)malloc( (l+1)*sizeof(double));
  beta = (double *)malloc( (l+1)*sizeof(double));
  stp = (double *)malloc( (l+1)*sizeof(double));
  stbeta = (double *)malloc( (l+1)*sizeof(double));
  bestp = (double *)malloc( (l+1)*sizeof(double));
  bestbeta = (double *)malloc( (l+1)*sizeof(double));
  dp = (double *)malloc( (l+1)*sizeof(double));
  dbeta = (double *)malloc( (l+1)*sizeof(double));
  actp = (double *)malloc( (l+1)*sizeof(double));
  actbeta = (double *)malloc( (l+1)*sizeof(double));
  derp=(double *)malloc(sizeof(double)*(l+1));
  derb=(double *)malloc(sizeof(double)*(l+1));
}

void memalloc2(void)
{
  int i,j;
  
  PROB=(double ***)malloc((l+1)*sizeof(double**));
  Q=(double ***)malloc(7*sizeof(double**));
  for(i=0;i<=6;i++)
    {
      if( (Q[i] = ( double ** ) malloc( (l+1) * sizeof(double*))) == NULL )
	{
	  printf("\nNot Enough Memory1\n");
	  exit(1);
	}
      for(j=1;j<=l;j++)
	if( (Q[i][j] = ( double * ) malloc( (l+1) * sizeof(double))) == NULL )
	  {
	    printf("\nNot Enough Memory1\n");
	    exit(1);
	  }
    }
  for(i=0;i<=l;i++)
    {
      if( (PROB[i] = ( double ** ) malloc( (l+1) * sizeof(double*))) == NULL )
	{
	  printf("\nNot Enough Memory1\n");
	  exit(1);
	}
      for(j=0;j<=l;j++)
	if( (PROB[i][j] = ( double * ) malloc( (l+1) * sizeof(double))) == NULL )
	  {
	    printf("\nNot Enough Memory1\n");
	    exit(1);
	  }
    }
	
  NUMI=(int **)malloc((l+1)*sizeof(int *));
  numofstates=l+1;
  for(i=0;i<=l;i++)
    {
      if( (NUMI[i] = ( int  * ) malloc( (unsigned long) numofstates * sizeof(int))) == NULL )
	{
	  printf("Not Enough Memory2");
	  exit(1);
	}
      numofstates=l+2;
    }
  
  COLI=(int **)malloc((l+1)*sizeof(int *));
  ROWI=(int **)malloc((l+1)*sizeof(int *));
  INTENSITY=(double **)malloc((l+1)*sizeof(double *));
  nonzeros=2*l+1;
  for(i=0;i<=l;i++)
    {
      if( (COLI[i] = ( int  * ) malloc( nonzeros * sizeof(int))) == NULL )
	{
	  printf("Not Enough Memory3");
	  exit(1);
	}
      if( (ROWI[i] = ( int  * ) malloc(  nonzeros * sizeof(int))) == NULL )
	{
	  printf("Not Enough Memory4");
	  exit(1);
	}
      if( (INTENSITY[i] = ( double  * ) malloc( nonzeros * sizeof(double))) == NULL )
	{
	  printf("Not Enough Memory5");
	  exit(1);
	}
      nonzeros=2*l+3;
    }
	
  prb2 = (double *)malloc( (l+1)*sizeof(double));
  prb = (double *)malloc( (l+1)*sizeof(double));
  p = (double *)malloc( (l+1)*sizeof(double));
  beta = (double *)malloc( (l+1)*sizeof(double));
  stp = (double *)malloc( (l+1)*sizeof(double));
  stbeta = (double *)malloc( (l+1)*sizeof(double));
  dp = (double *)malloc( (l+1)*sizeof(double));
  dbeta = (double *)malloc( (l+1)*sizeof(double));
  actp = (double *)malloc( (l+1)*sizeof(double));
  actbeta = (double *)malloc( (l+1)*sizeof(double));
  derp=(double *)malloc(sizeof(double)*(l+1));
  derb=(double *)malloc(sizeof(double)*(l+1));
}

void memfree(void)
{
  int i,j,m=3*l;
	
  for(i=1; i<=m+2; i++) free(a[i]);
  free(a);
  free(iposv);
  free(izrov);

  for(i=0;i<=l;i++)
    {
      free(NUMI[i]);
      free(ROWI[i]);
      free(COLI[i]);
      free(INTENSITY[i]);
      for(j=0;j<=l;j++)
	free(PROB[i][j]);
      free(PROB[i]);
    }
  for(i=0;i<=6;i++)
    {
      for(j=1;j<=l;j++)
	free(Q[i][j]);
      free(Q[i]);
    }
  
  free(p);   free(actp);  free(stp);
  free(beta);   free(actbeta);  free(stbeta);
  free(dp);  free(dbeta);
  free(derp); free(derb);
  free(obs);
  free(weight);
  free(dist);
  free(mass);
  free(prb);
  free(prb2);
  if(lh>0)
    {
      free(h);	
      free(lambda);
    }
	
  free(NUMI);
  free(ROWI);
  free(COLI);
  free(INTENSITY);
  free(PROB);
}

void memfree2(void)
{
  int i,j;
	
  for(i=0;i<=l;i++)
    {
      free(NUMI[i]);
      free(ROWI[i]);
      free(COLI[i]);
      free(INTENSITY[i]);
      for(j=0;j<=l;j++)
	free(PROB[i][j]);
      free(PROB[i]);
    }
  for(i=0;i<=6;i++)
    {
      for(j=1;j<=l;j++)
	free(Q[i][j]);
      free(Q[i]);
    }
  
  free(p);   free(actp);  free(stp);
  free(beta);   free(actbeta);  free(stbeta);
  free(dp);  free(dbeta);
  free(derp); free(derb);
  free(obs);
  free(weight);
  free(dist);
  free(mass);
  free(prb);
  free(prb2);
  if(lh>0)
    {
      free(h);	
      free(lambda);
    }
	
  free(NUMI);
  free(ROWI);
  free(COLI);
  free(INTENSITY);
  free(PROB);
}


/* computing measures based on the distribution header dist.h */
void compmeasures2(struct measures *m)
{
  int i,j;
  double nst,rm2=0,rm3=0,am2=0,am3=0;
  double realmean=0,appmean=0,a,b,newt,last;
  int intprec,goal=0,faded=0;
  
  intprec=100*intpr;
  realmean=distmean(param1,param2);
  for( i=1; i<=l; i++)
    for( j=1; j<=i; j++) appmean+=bestbeta[i]/bestp[j];
  for( i=0; i<lh; i++) appmean+=h[i]/lambda[i];
  
  // the interval [0;2*appmean] is divided into 100intpr intervals
  b=2*appmean/intprec;
  a=b/2;
  last=0;
  makeQ(bestp,bestbeta);
  ACT=0;
  ACTSTART=0;
  
  m->m1=fabs(realmean-appmean)/realmean;
  m->ad=0; m->rad=0; m->ent=0;         
  
  for(i=1;i<=intprec;i++)
    {
      for(j=0;j<12;j++)
	{
	  newt=a+b*C[j];
	  integr_rand(newt-last,last,epsran);
	  last=newt;
	  nst=bestp[1]*PROB[0][ACTSTART][0];
	  // + mixture of exponentials
	  nst += mixture(newt);
	  rm2 += b*A[j]*pdf(newt,param1,param2)*(newt-realmean)*(newt-realmean);
	  rm3 += b*A[j]*pdf(newt,param1,param2)*(newt-realmean)*(newt-realmean)*(newt-realmean); 
	  am2 += b*A[j]*nst*(newt-appmean)*(newt-appmean);
	  am3 += b*A[j]*nst*(newt-appmean)*(newt-appmean)*(newt-appmean); 
	  m->ad += b*A[j]*fabs(nst-pdf(newt,param1,param2));
	  /*printf("newt: %f, cdf: %f\n", newt, cdf(newt));*/
	  if(pdf(newt,param1,param2)>0.00001) m->rad += b*A[j]*fabs(nst-pdf(newt,param1,param2))/pdf(newt,param1,param2);
	  /*printf("%d %e %e %e\n", i, mass[i], nst, fabs(nst-mass[i]));*/
	  if(nst>0) m->ent += b*A[j]*pdf(newt,param1,param2)*log(nst);        
	}
      a+=b;
    }
  
  while(!goal)
    {
      for(j=0;j<12;j++)
	{
	  newt=a+b*C[j];
	  nst=0;
	  if(!faded) 
	    {
	      integr_rand(newt-last,last,epsran);
	      nst=bestp[1]*PROB[0][ACTSTART][0];
	      if( lh!=0 && mixture(newt)/nst>1e8) faded=1;
	    }
	  last=newt;
	  // + mixture of exponentials
	  nst += mixture(newt);
	  rm2 += b*A[j]*pdf(newt,param1,param2)*(newt-realmean)*(newt-realmean);
	  rm3 += b*A[j]*pdf(newt,param1,param2)*(newt-realmean)*(newt-realmean)*(newt-realmean); 
	  am2 += b*A[j]*nst*(newt-appmean)*(newt-appmean);
	  am3 += b*A[j]*nst*(newt-appmean)*(newt-appmean)*(newt-appmean); 
	  m->ad += b*A[j]*fabs(nst-pdf(newt,param1,param2));
	  /*printf("newt: %f, cdf: %f\n", newt, cdf(newt));*/
	  if(pdf(newt,param1,param2)>0.00001) m->rad += b*A[j]*fabs(nst-pdf(newt,param1,param2))/pdf(newt,param1,param2);
	  /*printf("%d %e %e %e\n", i, mass[i], nst, fabs(nst-mass[i]));*/
	  if(nst>0) m->ent += b*A[j]*pdf(newt,param1,param2)*log(nst);        
	}
      if(pdf(newt,param1,param2)<1e-20) goal=1;
      a+=1.05*b;
      b*=1.1;
    }
  
  printf(" newt: %e, m2: %f, m3: %f\n", newt, am2, am3);
  m->m2=fabs(rm2-am2)/rm2;
  m->m3=fabs(rm3-am3)/rm3;	        
}

/* computing measures based on the distribution file */
void compmeasures( struct measures *m)
{
  int i;
  double nst,rm2=0,rm3=0,am2=0,am3=0;
  double realmean=0, appmean=0;
  double diff;
  
  diff=dist[2]-dist[1]; 
  for(i=1;i<=od;i++) realmean+=dist[i]*mass[i]*diff; 
  /*printf("realmean: %f\n", realmean);*/
  
  makeQ(bestp,bestbeta);
  ACT=0;
  ACTSTART=0;
  nst=1; i=1;
  while(nst>LIMIT)
    {
      integr_rand(diff,(i-1)*diff,epsran);
      /*printf("obs: %f, prev: %f, pr: %f: \n", obs[i], prevobs, pp[1]*PROB[0][ACTSTART][0]);*/
      nst=bestp[1]*PROB[0][ACTSTART][0];
      appmean += i*diff*nst*diff;
      i++;
    }
  
  m->m1=fabs(realmean-appmean)/realmean;	
  m->ad=0; m->rad=0; m->ent=0;
  
  makeQ(bestp,bestbeta);
  ACT=0;
  ACTSTART=0;
  for(i=1;i<=od;i++)
    {
      /*printf("%d \n", i);*/
      integr_rand(diff,(i-1)*diff,epsran);
      nst=bestp[1]*PROB[0][ACTSTART][0];
      rm2 += diff*mass[i]*(dist[i]-realmean)*(dist[i]-realmean);
      rm3 += diff*mass[i]*(dist[i]-realmean)*(dist[i]-realmean)*(dist[i]-realmean); 
      am2 += diff*nst*(dist[i]-appmean)*(dist[i]-appmean);
      am3 += diff*nst*(dist[i]-appmean)*(dist[i]-appmean)*(dist[i]-appmean); 
      m->ad += diff*fabs(nst-mass[i]);
      m->rad += diff*fabs(nst-mass[i])/mass[i];
      /*printf("%d %e %e %e\n", i, mass[i], nst, fabs(nst-mass[i]));*/
      m->ent += diff*mass[i]*log(nst);        
    }
  /*printf(" m2: %f, m3: %f\n", rm2, rm3);*/	
  m->m2=fabs(rm2-am2)/rm2;
  m->m3=fabs(rm3-am3)/rm3;	
}

/* computing area difference based on the observations */
double areadiff( double *pp, double *bb)
{
  int i;
  double prevobs;
  double sum,nst;
  
  sum=0;
  prevobs=0;
  makeQ(pp,bb);
  ACT=0;
  ACTSTART=0;
  for(i=1;i<=ol;i++)
    {
      integr_rand(obs[i]-prevobs,prevobs,epsran);
      /*printf("obs: %f, prev: %f, pr: %f: \n", obs[i], prevobs, pp[1]*PROB[0][ACTSTART][0]);*/
      nst=pp[1]*PROB[0][ACTSTART][0];
      sum += fabs(weight[i]-nst);
      prevobs=obs[i];
    }
  return -sum;
}

void computeQ(double *pp, double *bb)
{
  double a,b,previous,diff;
  int i,j;
  
  previous=0;
  b=tail/intpr;
  a=b/2;
  for(j=0;j<=6;j++)
    {
      diff=a+b*C[j]-previous;
      if(j==6) diff/=2;
      makeQ(pp,bb);
      ACT=0;
      for(ACTSTART=1;ACTSTART<=l;ACTSTART++)
	{
	  integr_rand(diff,0,epsran);
	  for(i=1;i<=l;i++)
	    Q[j][ACTSTART][i]=PROB[ACT][ACTSTART][i-1];
	}
      previous=a+b*C[j];
    }
}	


/* matrice-vector multiplication */
void mul( double **m)
{
  int i,j;
  
  for(i=1;i<=l;i++)
    {
      prb2[i]=0;
      for(j=1;j<=l;j++)
	prb2[i]+=prb[j]*m[j][i];
    }
  for(i=1;i<=l;i++) prb[i]=prb2[i];
}

/* computing relative area difference based on the distribution in dist.h */
double int_relareadiff( double *pp, double *bb)
{
  int i,j;
  double sum,nst,newt,a,b;
  
  b=tail/intpr;
  a=b/2;
  sum=0;
  computeQ(pp,bb);
  for(i=1;i<=l;i++) prb[i]=bb[i];   
 
  for(i=1;i<=intpr;i++)
    {
      for(j=0;j<12;j++)
	{
	  newt=a+b*C[j];
	  if( j==0 && i>1 ) mul(Q[0]);
	  if( j==6 ) mul(Q[6]);
	  if( j<=6 ) mul(Q[j]);
	  if( j>6 )  mul(Q[j-(j-6)*2]);
	  /*printf("obs: %f, prev: %f, pr: %f: \n", obs[i], prevobs, pp[1]*PROB[0][ACTSTART][0]);*/
	  nst=pp[1]*prb[1];
	  nst+=mixture(newt);
	  if(pdf(newt,param1,param2)>0.00001)
	    sum += b*A[j]*fabs(pdf(newt,param1,param2)-nst)/
	      pdf(newt,param1,param2);
	}
      a+=b;
    }
  return -sum;
}          

/* computing area difference based on the distribution in dist.h */
double int_areadiff( double *pp, double *bb)
{
  int i,j;
  double sum,nst,newt,a,b;
	
  b=tail/intpr;
  a=b/2;
  sum=0;
  computeQ(pp,bb);
  for(i=1;i<=l;i++) prb[i]=bb[i];
	
  for(i=1;i<=intpr;i++)
    {
      for(j=0;j<12;j++)
	{
	  newt=a+b*C[j];
	  if( j==0 && i>1 ) mul(Q[0]);
	  if( j==6 ) mul(Q[6]);
	  if( j<=6 ) mul(Q[j]);
	  if( j>6 )  mul(Q[j-(j-6)*2]);
	  nst=pp[1]*prb[1];
	  nst+=mixture(newt);
	  sum += b*A[j]*fabs(pdf(newt,param1,param2)-nst);
	}
      a+=b;
    }
  return -sum;
}


/* computing cdf relative area difference based */
/* on the distribution in dist.h */
double int_cdfareadiff( double *pp, double *bb)
{
  int i,j,i2;
  double sum,nst,newt,a,b;
	
  b=tail/intpr;
  a=b/2;
  sum=0;
  computeQ(pp,bb);
  for(i=1;i<=l;i++) prb[i]=bb[i];
	
  for(i=1;i<=intpr;i++)
    {
      for(j=0;j<12;j++)
	{
	  newt=a+b*C[j];
	  if( j==0 && i>1 ) mul(Q[0]);
	  if( j==6 ) mul(Q[6]);
	  if( j<=6 ) mul(Q[j]);
	  if( j>6 )  mul(Q[j-(j-6)*2]);
	  //printf("obs: %f, prev: %f, pr: %f: \n", 
	  //obs[i], prevobs, pp[1]*PROB[0][ACTSTART][0]);
	  nst=1;
	  for(i2=1;i2<=l;i2++) nst-=prb[i2];
	  //nst+=mixture(newt);
	  for(i2=0;i2<lh;i2++) nst-=h[i2]*exp(-lambda[i2]*newt);
      	  sum += b*A[j]*(cdf(newt,param1,param2)-nst)*
	    (cdf(newt,param1,param2)-nst);
	}
      a+=b;
    }
  return -sum;
}


/* computing likelyhood based on the distribution in dist.h */
double int_likely( double *pp, double *bb)
{
  int i,j;
  double sum,nst,newt,a,b;
	
  //printf("Parameters: \n");
  //printp(pp, bb);
	
  b=tail/intpr;
  a=b/2;
  sum=0;
  computeQ(pp,bb);
  for(i=1;i<=l;i++) prb[i]=bb[i];
	
  for(i=1;i<=intpr;i++)
    {
      for(j=0;j<12;j++)
	{
	  newt=a+b*C[j];
	  if( j==0 && i>1 ) mul(Q[0]);
	  if( j==6 ) mul(Q[6]);
	  if( j<=6 ) mul(Q[j]);
	  if( j>6 )  mul(Q[j-(j-6)*2]);
	  /*printf("obs: %f, prev: %f, pr: %f: \n", obs[i], prevobs, pp[1]*PROB[0][ACTSTART][0]);*/
	  nst=pp[1]*prb[1];
	  nst+=mixture(newt);
	  if(nst>0) sum += b*A[j]*pdf(newt,param1,param2)*log(nst);
	}
      a+=b;
    }
  /*printf("likely: %f\n", sum);*/
  return sum;
}


/* computing relative area difference based on the observations */
double relareadiff( double *pp, double *bb)
{
  int i;
  double prevobs;
  double sum,nst;
  
  sum=0;
  prevobs=0;
  makeQ(pp,bb);
  ACT=0;
  ACTSTART=0;
  for(i=1;i<=ol;i++)
    {
      integr_rand(obs[i]-prevobs,prevobs,epsran);
      /*printf("obs: %f, prev: %f, pr: %f: \n", obs[i], prevobs, pp[1]*PROB[0][ACTSTART][0]);*/
      nst=pp[1]*PROB[0][ACTSTART][0];
      sum += fabs(weight[i]-nst)/weight[i];
      prevobs=obs[i];
    }
  return -sum;
}

/* calculating likelyhood function based on the observations*/
double likely(double *pp, double *bb)
{
  int i;
  double sum,prevobs;
  
  sum=0;
  prevobs=0;
  makeQ(pp,bb);
  
  ACT=0;
  ACTSTART=0;
  for(i=1;i<=ol && obs[i]<tail;i++)
    {
      integr_rand(obs[i]-prevobs,prevobs,epsran);
      /*printf("obs: %f, prev: %f, pr: %f: \n", obs[i], prevobs, pp[1]*PROB[0][ACTSTART][0]);*/
      sum += weight[i]*log(pp[1]*PROB[0][ACTSTART][0]+mixture(obs[i]));
      prevobs=obs[i];
    }
  return sum;
}

/* checking along the line */
int along(void)
{
  int i,j,k,more;
  double bestlik, lik;
  
  //printf(" Itt vagyok, method: %d\n", method);
  
  if( method==1 || method==2 ) bestlik=likely(p,beta);
  if( method==3 ) bestlik=areadiff(p,beta);
  if( method==4 ) bestlik=relareadiff(p,beta);
  if( method==5 ) bestlik=int_likely(p,beta);
  if( method==6 ) bestlik=int_areadiff(p,beta);
  if( method==7 ) bestlik=int_relareadiff(p,beta);   
  if( method==8 ) bestlik=int_cdfareadiff(p,beta);   
  //printf("bestlik: %f\n", bestlik);
  
  for(i=1;i<=l;i++)
    {
      actp[i]=p[i];
      actbeta[i]=beta[i];
    }
  
  for(k=1;k<=9;k++)
    {
      for(j=1;j<=l;j++)
	{
	  actp[j]+=dp[j]/10;
	  actbeta[j]+=dbeta[j]/10;
	}
      if( method==1 || method==2 ) lik=likely(actp,actbeta);
      if( method==3 ) lik=areadiff(actp,actbeta);	
      if( method==4 ) lik=relareadiff(actp,actbeta);
      if( method==5 ) lik=int_likely(actp,actbeta);
      if( method==6 ) lik=int_areadiff(actp,actbeta);
      if( method==7 ) lik=int_relareadiff(actp,actbeta);   
      if( method==8 ) lik=int_cdfareadiff(actp,actbeta);   
      //printp(actp,actbeta);
      //printf("lik: %f\n", lik);
      //if(lik>bestlik) { bestlik=lik; }
      else
	{
	  k=11;
	  for(j=1;j<=l;j++)
	    {
	      actp[j]-=dp[j]/10;
	      actbeta[j]-=dbeta[j]/10;
	    }
	}
      
    }
  for(k=1;k<=9;k++)
    {
      for(j=1;j<=l;j++)
	{
	  actp[j]+=dp[j]/100;
	  actbeta[j]+=dbeta[j]/100;
	}
      if( method==1 || method==2 ) lik=likely(actp,actbeta);
      if( method==3 ) lik=areadiff(actp,actbeta);
      if( method==4 ) lik=relareadiff(actp,actbeta); 
      if( method==5 ) lik=int_likely(actp,actbeta);
      if( method==6 ) lik=int_areadiff(actp,actbeta);
      if( method==7 ) lik=int_relareadiff(actp,actbeta);   
      if( method==8 ) lik=int_cdfareadiff(actp,actbeta);   
      /*printp(actp,actbeta);*/
      //printf("lik: %f\n", lik);
      /*printf("lik: %f\n", lik);*/
      if(lik>bestlik) { bestlik=lik; }
      else
	{
	  k=11;
	  for(j=1;j<=l;j++)
	    {
	      actp[j]-=dp[j]/100;
	      actbeta[j]-=dbeta[j]/100;
	    }
	}
    }
  for(k=1;k<=10;k++)
    {
      for(j=1;j<=l;j++)
	{
	  actp[j]+=dp[j]/1000;
	  actbeta[j]+=dbeta[j]/1000;
	}
      if( method==1 || method==2 ) lik=likely(actp,actbeta);
      if( method==3 ) lik=areadiff(actp,actbeta);
      if( method==4 ) lik=relareadiff(actp,actbeta);
      if( method==5 ) lik=int_likely(actp,actbeta);
      if( method==6 ) lik=int_areadiff(actp,actbeta);
      if( method==7 ) lik=int_relareadiff(actp,actbeta);   
      if( method==8 ) lik=int_cdfareadiff(actp,actbeta);   
      /*printp(actp,actbeta);*/
      //printf("lik: %f\n", lik);
      /*printf("lik: %f\n", lik);*/
      if(lik>bestlik) { bestlik=lik; }
      else
	{
	  k=11;
	  for(j=1;j<=l;j++)
	    {
	      actp[j]-=dp[j]/1000;
	      actbeta[j]-=dbeta[j]/1000;
	    }
	}
    }
  more=0;
  for(k=1;k<=l;k++)
    {
      if(beta[k]!=actbeta[k] || p[k]!=actp[k])
	{
	  more=1;     
	  k=l+1;
	}
      /*printp(actp,actbeta);*/
      /*printp(p,beta);*/
    }
  for(i=1;i<=l;i++)
    {
      if( actp[i]<0.0 ) {p[i]=0.0;}
      else {p[i]=actp[i];}
      if( actbeta[i]<0.0 ) {beta[i]=0.0;}
      else {beta[i]=actbeta[i];}
    }
  
  /*printf("best: %f\n", bestlik);*/
  return more;
}

/* derivatives with respect to p and beta*/
/* according to the likelihood function, analitically */
void derive1(void)
{
  int i,j,k,n;
  double *fn, *dersp, *dersb;
  double fnsum, d, nst, prevobs=0;
  
  fn=(double *)malloc(sizeof(double)*(l+1));
  dersp=(double *)malloc(sizeof(double)*(l+1));
  dersb=(double *)malloc(sizeof(double)*(l+1));
  
  makeser();
  makeQ(p,beta);
  
  prevobs=0;
  for(i=1;i<=l;i++)
    { derp[i]=0; derb[i]=0; }
  for(i=1;i<=ol;i++)
    {
      /* calculating fn and nst*/
      nst=0;
      for(n=1;n<=l;n++)
	{
	  /* starting from phase n */
	  ACT=0;
	  ACTSTART=n;
	  integr_rand(obs[i]-prevobs,prevobs,epsran);
	  fn[n]=PROB[0][ACTSTART][0]*p[1];
	  nst+=beta[n]*fn[n];
	}
      
      /*printf("nst: %f\n", nst);*/
      
      for(j=1;j<=l;j++)
	{
	  fnsum=0;
	  d=0;
	  ACT=j;
	  
	  for(k=j;k<=l;k++)
	    {
	      /* starting from phase k */
	      ACTSTART=k;
	      integr_rand(obs[i]-prevobs,prevobs,epsran);
	      /*printstr(j);  */
	      fnsum += beta[k]*PROB[ACT][ACTSTART][0]*p[ACT];
	      d += beta[k]*fn[k];
	      /*printf("k: %d, fnsum: %f, d: %f", k, fnsum, d);*/
	    }
	  /*printf("fnsum: %f, d: %f\n", fnsum, d);*/
	  dersp[j] = 1/p[j]*(d-fnsum);
	  dersb[j] = fn[j];
	  /*printf("dersp[%d]: %f, dersb[%d]: %f\n", j, dersp[j], j, dersb[j]);*/
	}
      
      for(j=1;j<=l;j++)
	{
	  /*printf("%d %d \n", i,j);*/
	  derp[j] += weight[i]*dersp[j]/nst;
	  derb[j] += weight[i]*dersb[j]/nst;
	  /*printf("derp[%d]: %f, derb[%d]: %f\n", j, derp[j], j, derb[j]);*/
	}
      prevobs=obs[i];
    }
  //printf("Derivatives: \n");        
  //printp(derp,derb);
  free(dersp);
  free(dersb);
  free(fn);
}

/* derivatives with respect to p and beta*/
/* according to the likelihood function, numerically */
void derive2(void)
{
  int i,up;
  double a,b;
  double diff, per=0.000001;
  
  //printf("Parameters: \n");
  //printp(p, beta);
  
  if( method==2 ) a=likely(p,beta);
  if( method==3 ) a=areadiff(p,beta);
  if( method==4 ) a=relareadiff(p,beta);
  if( method==5 ) a=int_likely(p,beta);
  if( method==6 ) a=int_areadiff(p,beta);
  if( method==7 ) a=int_relareadiff(p,beta);
  if( method==8 ) a=int_cdfareadiff(p,beta);
  
  for(i=1;i<=l;i++)
    { 
      /* derivative with respect to p[i] */
      if( p[i]>0.01 ) {diff=p[i]*per;}
      else    {diff=0.001;}
      p[i]+=diff;
      if( method==2 ) b=likely(p,beta);
      if( method==3 ) b=areadiff(p,beta);
      if( method==4 ) b=relareadiff(p,beta);
      if( method==5 ) b=int_likely(p,beta);
      if( method==6 ) b=int_areadiff(p,beta);
      if( method==7 ) b=int_relareadiff(p,beta);
      if( method==8 ) b=int_cdfareadiff(p,beta);
      derp[i]=(b-a)/diff;
      p[i]-=diff;
    }
  for(i=1;i<=l;i++)
    {
      /* derivative with respect to beta[i] */
      if( beta[i]<=0.99 && beta[i]>=0.01 )
	{
	  diff=beta[i]*per;
	  up=1;
	}
      else    {
	if( beta[i]>0.99 )
	  {
	    diff=beta[i]*per;
	    up=-1;
	  }
	if( beta[i]<0.01 )
	  {
	    diff=0.001;
	    up=1;
	  }	
      }
      beta[i]+=up*diff;
      if( method==2 ) b=likely(p,beta);
      if( method==3 ) b=areadiff(p,beta);
      if( method==4 ) b=relareadiff(p,beta);
      if( method==5 ) b=int_likely(p,beta);
      if( method==6 ) b=int_areadiff(p,beta);
      if( method==7 ) b=int_relareadiff(p,beta);
      if( method==8 ) b=int_cdfareadiff(p,beta);
      derb[i]=(b-a)/diff*up;
      //printf("a: b: %e  %e\n",a,b);
      beta[i]-=up*diff;
    }
  //printf("Derivatives: \n");
  //printp(derp, derb);
}	


/* generating pdf */
void genlist(double lower, double upper, double stepsize, int additive)
{
  int i,onlytail=0;
  FILE *f;
  double t,diff,F,H,tailconstant,sptailccdf,sptailpdf,cf1pdf,tmp;

  if(sptailtype!=0)
    {
      if(sptailtype==1)
	tailconstant=ccdf(sptailstart,param1,param2)/
	  pow(sptailstart,-sptailparam);
      else
	tailconstant=ccdf(sptailstart,param1,param2)/
	  exp(-pow(sptailstart,sptailparam));
    }

  f=fopen("TEMPORARYPhFitFILE.data","w");
  
  printf("%1.4e\n",lower);
  if(lower==0.0)
    fprintf(f, "%e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e\n", 0.0, 
	    pdf(0.0001,param1,param2), 
	    0.0,
	    1.0,
	    pdf(0.0001,param1,param2),
	    bestp[1]*bestbeta[1]+mixture(0), 
	    0.0,
	    1.0,
	    bestp[1]*bestbeta[1]+mixture(0),
	    fabs(pdf(0.0001,param1,param2)-(bestp[1]*bestbeta[1]+mixture(0)))/pdf(0.0001,param1,param2),
	    0.0,
	    0.0,
	    fabs(pdf(0.0001,param1,param2)-bestp[1]*bestbeta[1]+mixture(0))/pdf(0.0001,param1,param2),
	    bestp[1]*bestbeta[1], 
	    mixture(0) );
    
  if(lower==0.0)
    {
      if(additive)
	{
	  diff=stepsize;
	  t=stepsize;
	}
      else
	{
	  diff=upper/100;
	  t=diff;
	}
    }
  else
    {
      diff=lower;
      t=lower;
    }

  makeQ(bestp, bestbeta);
  ACT=0;
  ACTSTART=0;
  while(t<=upper)
    {
      F=0;
      cf1pdf=0;
      if(!onlytail || lh==0)
	{
	  integr_rand(diff,t,epsran);
	  for(i=0;i<l;i++) F+=PROB[ACT][ACTSTART][i];
	  cf1pdf=bestp[1]*PROB[ACT][ACTSTART][0];
	}
      H=0;
      for(i=0;i<lh;i++) H+=h[i]*exp(-lambda[i]*t);
      if(lh!=0)
	if(H/F>1e20) onlytail=1;
      F+=H;
      fprintf(f, "%e  ", t); 
      tmp=pdf(t,param1,param2);
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 
      tmp=cdf(t,param1,param2);
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 
      tmp=ccdf(t,param1,param2);
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 
      tmp=pdf(t,param1,param2)/ccdf(t,param1,param2);
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 
      tmp=cf1pdf+mixture(t);
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 
      tmp=1-F;
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 
      tmp=F;
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 
      tmp=(cf1pdf+mixture(t))/F;
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 
      tmp=fabs(pdf(t,param1,param2)-(cf1pdf+mixture(t)))/pdf(t,param1,param2);
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 
      tmp=fabs(1-F-cdf(t,param1,param2))/(cdf(t,param1,param2));
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 
      tmp=fabs(F-ccdf(t,param1,param2))/ccdf(t,param1,param2);
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 
      tmp=fabs(pdf(t,param1,param2)/ccdf(t,param1,param2)-(cf1pdf+mixture(t))/F)/(pdf(t,param1,param2)/ccdf(t,param1,param2));
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 
      tmp=cf1pdf;
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 
      tmp=mixture(t);
      if(!finite(tmp)) tmp=0.0;
      fprintf(f, "%e  ", tmp); 

      if(t>sptailstart&&sptailtype!=0)
	{
	  if(sptailtype==1)
	    {
	      sptailpdf=tailconstant*sptailparam*pow(t,-sptailparam-1.0);
	      sptailccdf=tailconstant*pow(t,-sptailparam);
	    }
	  else
	    {
	      sptailpdf=tailconstant*exp(-pow(t,sptailparam))*sptailparam*pow(t,sptailparam-1);
	      sptailccdf=tailconstant*exp(-pow(t,sptailparam));
	    }
	  tmp=sptailpdf;
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=1-sptailccdf;
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=sptailccdf;
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=sptailpdf/sptailccdf;
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=fabs(sptailpdf-(cf1pdf+mixture(t)))/sptailpdf;
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=fabs(1-F-(1-sptailccdf))/(1-sptailccdf);
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=fabs(F-sptailccdf)/sptailccdf;
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=fabs((cf1pdf+mixture(t))/F-sptailpdf/sptailccdf)/(sptailpdf/sptailccdf);
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e", tmp); 
	}
      fprintf(f,"\n");
      if(!additive)
	{
	  diff=t*stepsize-t;
	  t*=stepsize;
	}
      else
	{
	  diff=stepsize;
	  t+=diff;
	}
      //printf("t: %e\n",t);
    }
  fclose(f);
}

void writeparam(char *fn)
{
	int i;
	FILE *f;
	
	f=fopen(fn,"w");
	
	fprintf(f, "p={");
	for(i=1;i<=l;i++)
	{
        fprintf(f, " %1.6e",bestp[i]);
        if(i<l) fprintf(f, ",");
	}
	fprintf( f, "}\n");
	fprintf(f, "beta={");
	for(i=1;i<=l;i++)
	{
        fprintf(f, " %1.6e",bestbeta[i]);
        if(i<l) fprintf(f, ",");
	}
	fprintf( f, "}\n");
	
	fprintf(f, "h={");
	for(i=0;i<lh;i++)
	{
        fprintf(f, " %1.6e",h[i]);
        if(i<lh-1) fprintf(f, ",");
	}
	fprintf( f, "}\n");
	fprintf(f, "lambda={");
	for(i=0;i<lh;i++)
	{
        fprintf(f, " %1.6e",lambda[i]);
        if(i<lh-1) fprintf(f, ",");
	}
	fprintf( f, "}\n");
	
	fclose(f);
}

/* generating random starting parameters with given mean */
void stvalues(double mean)
{
  int i, j, k, max;
  double sum, bestst=-99999999,actst,expsum=0;

  //printf(" RAND_MAX: %d\n", RAND_MAX);
	
  for(i=0;i<lh;i++) expsum+=h[i];
  // choosing the best of 1000 random starting values
  for(k=1;k<=100;k++)
    {
      /* generating initial probability vector */
      stbeta[0]=0;
      sum=0;
      for(i=1;i<=l;i++)
	{
	  stbeta[i]=(double)rand()/RAND_MAX;
	  sum+=stbeta[i];
	}
      /* normalizing */
      for(i=1;i<=l;i++)
	stbeta[i] = stbeta[i]/sum*(1-expsum);
      
      /* generating transition intensities */
      for(i=1;i<=l;i++)
	actp[i]=rand();
      
      /* decreasing order */
      for(i=1;i<=l;i++)
	{
	  max=1;	
	  for(j=1;j<=l;j++) if(actp[j]>actp[max]) max=j;
	  stp[i]=actp[max];
	  actp[max]=0;
	}
      
      /* modifie intensities in order to have proper mean */
      sum=0;
      for( i=1; i<=l; i++)
	for( j=1; j<=i; j++)
	  sum+=stbeta[i]/stp[j];
      for( i=1; i<=l; i++)
	stp[i] *= sum/mean;	
			
      //printf(" Parameters:\n");
      //printp(stp,stbeta);
      
      if( method==1 || method==2 ) actst=likely(stp,stbeta);
      if( method==3 ) actst=areadiff(stp,stbeta);
      if( method==4 ) actst=relareadiff(stp,stbeta);
      if( method==5 ) actst=int_likely(stp,stbeta);
      if( method==6 ) actst=int_areadiff(stp,stbeta);
      if( method==7 ) actst=int_relareadiff(stp,stbeta);   
      if( method==8 ) actst=int_cdfareadiff(stp,stbeta);   

      if( actst > bestst )
	{
	  bestst=actst;
	  for(i=1;i<=l;i++)
	    {  p[i]=stp[i];  beta[i]=stbeta[i]; }
	}			
    }
  //printf("bestst: %1.4e\n", bestst);
}

void Result2File()
{
  FILE *f;
  int i1,i2;

  f=fopen("TEMPORARYPhFitFILE.result","w");
  fprintf(f,"%d\n", l);
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.8e", bestbeta[i1]);
  fprintf(f,"\n");
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.8e", bestp[i1]);
  fprintf(f,"\n");
  fprintf(f,"%d\n", lh);
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.8e", h[i1]);
  fprintf(f,"\n");
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.8e", lambda[i1]);
  fprintf(f,"\n");
  fclose(f);

  f=fopen("TEMPORARYPhFitFILE.initials","w");
  fprintf(f, "CF-1 part:\n");
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.4e\n", bestbeta[i1]);
  fprintf(f,"\n");
  if(lh>0)  fprintf(f, "hyper. part:\n");
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.4e\n", h[i1]);
  fprintf(f,"\n");
  fclose(f);

  f=fopen("TEMPORARYPhFitFILE.intensities","w");
  fprintf(f, "CF-1 part:\n");
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.4e\n", bestp[i1]);
  fprintf(f,"\n");
  if(lh>0)  fprintf(f, "hyper. part:\n");
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.4e\n", lambda[i1]);
  fprintf(f,"\n");
  fclose(f);

  f=fopen("TEMPORARYPhFitFILE.textual","w");
  fprintf(f, "Initial probabilities of CF-1 part:\n");
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.8e\n", bestbeta[i1]);
  fprintf(f,"\n");
  if(lh>0)  fprintf(f, "Initial probabilities of hyperexponential part:\n");
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.8e\n", h[i1]);
  fprintf(f,"\n");
  fprintf(f, "Intensities of CF-1 part:\n");
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.8e\n", bestp[i1]);
  fprintf(f,"\n");
  if(lh>0)  fprintf(f, "Intensities of hyperexponential part:\n");
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.8e\n", lambda[i1]);
  fprintf(f,"\n");
  fclose(f);

  f=fopen("TEMPORARYPhFitFILE.matrix","w");
  fprintf(f,"a=[");
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.8e ", bestbeta[i1]);
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.8e ", h[i1]);
  fprintf(f,"];\n");
  fprintf(f,"\nB=[");
  for(i1=1;i1<=l;i1++)
    {
      for(i2=1;i2<=l+lh;i2++) 
	{
	  if(i1==i2) fprintf(f," %1.8e ", -bestp[i1]);
	  if(i1-1==i2) fprintf(f," %1.8e ", bestp[i1]);
	  if(i1-1!=i2 && i1!=i2) fprintf(f," %1.8e ", 0.0);
	}
      fprintf(f,";\n");
    }
  for(i1=0;i1<lh;i1++) 
    {
      for(i2=0;i2<l+lh;i2++) 
	{
	  if(i1+l==i2) fprintf(f," %1.8e ", -lambda[i1]);
	  if(i1+l!=i2) fprintf(f," %1.8e ", 0.0);
	}
      fprintf(f,";\n");
    }
  fprintf(f,"];\n");
  fprintf(f,"\n");
  fprintf(f," f=[%1.8e;\n",bestp[1]);
  for(i1=0;i1<l-1;i1++) fprintf(f," %1.8e;\n", 0.0);
  for(i1=0;i1<lh;i1++) fprintf(f," %1.8e;\n", lambda[i1]);
  fprintf(f,"];\n");
  fclose(f);
}

void ResultFromFile()
{
  FILE *f;
  int i1;
  
  f=fopen("TEMPORARYPhFitFILE.result","r");
  fscanf(f,"%d\n", &l);
  bestp = (double *)malloc( (l+1)*sizeof(double));
  bestbeta = (double *)malloc( (l+1)*sizeof(double));
  for(i1=1;i1<=l;i1++) 
    fscanf(f," %le", &bestbeta[i1]);
  fscanf(f,"\n");
  for(i1=1;i1<=l;i1++) 
    fscanf(f," %le", &bestp[i1]);
  fscanf(f,"\n");
  fscanf(f,"%d\n", &lh);
  if(lh>0)
    {
      lambda = (double *)malloc( (lh)*sizeof(double));
      h = (double *)malloc( (lh)*sizeof(double));
    }
  for(i1=0;i1<lh;i1++) 
    fscanf(f," %le", &h[i1]);
  fscanf(f,"\n");
  for(i1=0;i1<lh;i1++) 
    fscanf(f," %le", &lambda[i1]);
  fscanf(f,"\n");
  fclose(f);
}

// checking hyperexp part
int goodgeom()
{
  int i1;
  double sum=0.0;

  for(i1=0;i1<lh;i1++)
    if( !(h[i1]>0.0 && h[i1]<1.0) ) return 0;
  for(i1=0;i1<lh;i1++)
    if(! (lambda[i1]>0.0) ) return 0;
  for(i1=0;i1<lh;i1++)
    sum+=h[i1];
  if(sum>1.0) return 0;
  return 1;
}

// fitting the tail and checking the results + messages
void buildandcheck()
{
  int i;

  if(lh>0) 
    {
      printf("Building Hyperexponential Part...\n");
      fflush(stdout);
      build_exp();
      if(!goodgeom())
	{
	  printf("  Failed to build hyperexponential part.\n");
	  printf("  Resulting parameters are inappropriate:\n");
	  for(i=0;i<lh;i++)
	    {
	      printf("   Init. prob.s \tIntensity\n");
	      for(i=0;i<lh;i++) 
		printf("   %1.4e \t%1.4e\n",h[i],lambda[i]); 
	    }
	  printf("  Tailfitting parameters should be changed.\n");
	  exit(1);
	}
      printf("Hyperexponential Part Built\n\n");
      fflush(stdout);
    }
}


int main ( int argc, char* argv[])
{
  int i, j, numofit;
  double bestlike, lik;
  struct measures *m;
  double mean,stmean;

  // Input parameters:
  // 1: Function: "1": Fitting a distribution given by pdf
  //              "2": Producing data file to plot curves in case of pdf
  //              "3": Fitting distribution given by samples
  //              "4": Producing data file to plot curves in case of samples
  
  // If 1.Param is "1" (Fitting a distribution given by pdf), then
  // 2: Distribution: "1": Weibull
  //                  "2": Lognormal
  //                  "3": Uniform
  //                  "4": Pareto-like I
  //                  "5": Pareto-like II
  // 3: 1st Parameter of the Distribution
  // 4: 2nd Parameter of the Distribution
  // 5: Number of Phases to Fit the Main Part
  // 6: Limit of Fitting the Main Part
  // 7: Distance Measure: "1": Entropy
  //                      "2": pdf Area Difference
  //                      "***": Relative pdf Area Difference
  //                      "3": cdf Area Difference
  // 8: Number of Phases to Fit the Tail Part
  // 9: Lower Limit of Tail fitting
  // 10: Upper Limit of Tail Fitting
  // 11: Distance of Adjacent Points
  // 12: Special Tail: "0" No special tail
  //                   "1" Polynomial tail
  //                   "2" Weibull tail
  // 13: Parameter of the tail
  // 14: Start of the tail (positive->itself, negative->quantile)
  // 15: Number of round
  // 16: Number of iterations in a round
  // 17: Number of intervals for integrataion

  // If 1.Param is "2" (Producing data file to plot curves in case of pdf), then
  // 2: Distribution: "1": Weibull
  //                  "2": Lognormal
  //                  "3": Uniform
  //                  "4": Pareto-like I
  //                  "5": Pareto-like II
  // 3: 1st Parameter of the Distribution
  // 4: 2nd Parameter of the Distribution
  // 5: Lower Limit of List
  // 6: Upper Limit of List
  // 7: Stepsize of List (positive -> additive,negative->multiplicative)
  // 8: Special Tail: "0" No special tail
  //                   "1" Polynomial tail
  //                   "2" Weibull tail
  // 9: Parameter of the tail
  // 10: Start of the tail (positive->itself, negative->quantile)

  // If 1.Param is "3" (Fitting a distribution given by samples), then
  // 2: name of file describing the distribution
  // 3: type of file: 1: empirical cdf
  //                  2: empirical ccdf
  //                  3: the samples itself
  //                  4: samples with weights
  // 4: Number of Phases to Fit the Main Part
  // 5: Limit of Fitting the Main Part
  // 6: Distance Measure: "1": Entropy
  //                      "***": pdf Area Difference
  //                      "***": Relative pdf Area Difference
  //                      "3": cdf Area Difference
  // 7: Number of Phases to Fit the Tail Part
  // 8: Lower Limit of Tail fitting
  // 9: Upper Limit of Tail Fitting
  // 10: Distance of Adjacent Points
  // 11: Special Tail: "0" No special tail
  //                   "1" Polynomial tail
  //                   "2" Weibull tail
  // 12: Parameter of the tail
  // 13: Start of the tail (positive->itself, negative->quantile)
  // 14: Number of round
  // 15: Number of iterations in a round
  // 16: Number of intervals for integrataion

  // If 1.Param is "4" (Producing data file to plot curves in case of samples), then
  // 2: name of file describing the distribution
  // 3: type of file: 1: empirical cdf
  //                  2: empirical ccdf
  //                  3: the samples itself
  //                  4: samples with weights
  // 4: Lower Limit of List
  // 5: Upper Limit of List
  // 6: Stepsize of List (positive -> additive,negative->multiplicative)
  // 7: Special Tail: "0" No special tail
  //                   "1" Polynomial tail
  //                   "2" Weibull tail
  // 8: Parameter of the tail
  // 9: Start of the tail (positive->itself, negative->quantile)


  // Fitting a distribution given by pdf;
  if(atoi(argv[1])==1)
    {
      switch(atoi(argv[2]))
	{
	case 1:
	  pdf=&pdf_weib; distmean=&mean_weib; cdf=&cdf_weib;		
	  invcdf=&invcdf_weib; ccdf=&ccdf_weib; invccdf=&invccdf_weib;
	  break;
	case 2:
	  pdf=&pdf_logn; distmean=&mean_logn; cdf=&cdf_logn;		
	  invcdf=&invcdf_logn; ccdf=&ccdf_logn; invccdf=&invccdf_logn;
	  break;
	case 3:
	  pdf=&pdf_uni; distmean=&mean_uni; cdf=&cdf_uni;		
	  invcdf=&invcdf_uni; ccdf=&ccdf_uni; invccdf=&invccdf_uni;
	  break;
	case 4:
	  pdf=&pdf_par1; distmean=&mean_par1; cdf=&cdf_par1;		
	  invcdf=&invcdf_par1; ccdf=&ccdf_par1; invccdf=&invccdf_par1;
	  break;
	case 5:
	  pdf=&pdf_par2; distmean=&mean_par2; cdf=&cdf_par2;		
	  invcdf=&invcdf_par2; ccdf=&ccdf_par2; invccdf=&invccdf_par2;
	  break;
	}
      param1=atof(argv[3]);
      param2=atof(argv[4]);
      l=atoi(argv[5]);
      if(atof(argv[6])>=0) 
	tail=atof(argv[6]);
      else 
	{ 
	  tail=invccdf(-atof(argv[6]),param1,param2);	
	  printf(" End of fitting the body: %1.4e\n",tail);
	}
      method=atoi(argv[7]);
      if(method==1) method=5;
      if(method==2) method=6;
      if(method==3) method=8;
      lh=atoi(argv[8]);
      if(atof(argv[9])>=0) 
	mtail1=atof(argv[9]);	
      else 
	{
	  mtail1=invccdf(-atof(argv[9]),param1,param2); 
	  printf(" Lower limit of fitting the tail: %1.4e\n", mtail1);
	}
      if(atof(argv[10])>=0) 
	{
	  mtail2=atof(argv[10]);	
	}
      else 
	{
	  mtail2=invccdf(-atof(argv[10]),param1,param2); 
	  printf(" Upper limit of fitting the tail: %1.4e\n", mtail2);
	}
      adj=atof(argv[11]);		
      sptailtype=atoi(argv[12]);
      if(sptailtype!=0)
	{
	  sptailparam=atof(argv[13]);
	  if(atof(argv[14])>0)
	    sptailstart=atof(argv[14]);
	  else
	    {
	      sptailstart=invccdf(-atof(argv[14]),param1,param2);
	      printf(" Start of special tail: %1.4e\n", sptailstart);
	    }
	}
      NUMBEROFROUNDS=atoi(argv[15]);
      NUMBEROFITERATIONS=atoi(argv[16]);
      intpr=atoi(argv[17]);

      memalloc();
      buildandcheck();
      mean=distmean(param1,param2);
      for(i=0;i<lh;i++) mean-=h[i]/lambda[i];
      //printf("Mean changed by The Mixture: %f\n",mean);
      //printf("Mean unchanged by The Mixture: %f\n",distmean(param1,param2));
      //printf("ML_Limit: %f, Limit1: %f, Limit2: %f\n", tail,mtail1,mtail2);
      //fflush(stdout);
      srand(1);
      
      numofit=NUMBEROFITERATIONS;

      bestlike=-10000000.0;
      for(i=1;i<=NUMBEROFROUNDS;i++)
	{
	  printf("Round n. %2d/%2d\n",i,NUMBEROFROUNDS);
	  fflush(stdout);
	  printf("  Looking for good starting values...\n");
	  fflush(stdout);
	  stmean=(double)rand()/RAND_MAX*mean+0.5*mean;
	  if(stmean<0) stmean=0.1;
	  stvalues(stmean);
	  printf("  Starting values are defined\n");
	  fflush(stdout);
	  //printf("startting values n. %d:\n ", i);
	  //printp(p,beta);
	  
	  // Maximum of change in parameters
	  Deltap=0.1;
	  Deltabeta=0.1;
	  
	  for(j=1;j<=numofit;j++)
	    {
	      printf("  Modification n. %4d/%4d\n",j,numofit);
	      fflush(stdout);
/* 	      printf(".",j); */
/* 	      if( j % 50 == 0 )  */
/* 		{ */
/* 		  printf("\n "); */
/* 		  fflush(stdout); */
/* 		} */
	      //printf(" Parameters:\n");
	      //printp(p,beta);
	      //fflush(stdout);
	      if( method==1 ) derive1();
	      if( method > 1 ) derive2();
	      //printf(" Derivatives:\n");
	      //printp(derp, derb);
	      //fflush(stdout);
	      lp();
	      //printf(" Lin prog results:\n");
	      //printp(dp,dbeta);
	      //fflush(stdout);
	      // printf("  lp\n");
	      if(along()==0)
		{
		  if( Deltap<0.01 )
		    {
		      //printf("\n   Number of modifications: %d\n", j);
		      j=1000;
		    }
		  else 
		    {
		      Deltap*=0.9;
		      Deltabeta*=0.9;
		    }
		}
	      // Delta=0.995*Delta;
	    }
	  
	  printf("\nEnd of round n. %d:\n\n", i);
	  fflush(stdout);
	  //printp(p,beta);
	  
	  if( method==1 || method==2 ) lik=likely(p,beta);
	  if( method==3 ) lik=areadiff(p,beta);
	  if( method==4 ) lik=relareadiff(p,beta);
	  if( method==5 ) lik=int_likely(p,beta);
	  if( method==6 ) lik=int_areadiff(p,beta);
	  if( method==7 ) lik=int_relareadiff(p,beta);   
	  if( method==8 ) lik=int_cdfareadiff(p,beta);   
	  
	  //printf("lik: %1.4e\n",lik);
	  if( bestlike<lik)
	    {
	      bestlike=lik;
	      for(j=1;j<=l;j++)
		{
		  bestbeta[j]=beta[j];
		  bestp[j]=p[j];
		}
	    }
	}
      //printp(bestp,bestbeta);
      Result2File();
      memfree();
      printf("Fitting has finished.\n");
      return 0;
    }
  // End of Fitting a distribution given by pdf 

  // Producing data file to plot curves in case of pdf
  if(atoi(argv[1])==2)
    {
      switch(atoi(argv[2]))
	{
	case 1:
	  pdf=&pdf_weib; distmean=&mean_weib; cdf=&cdf_weib;		
	  invcdf=&invcdf_weib; ccdf=&ccdf_weib; invccdf=&invccdf_weib;
	  break;
	case 2:
	  pdf=&pdf_logn; distmean=&mean_logn; cdf=&cdf_logn;		
	  invcdf=&invcdf_logn; ccdf=&ccdf_logn; invccdf=&invccdf_logn;
	  break;
	case 3:
	  pdf=&pdf_uni; distmean=&mean_uni; cdf=&cdf_uni;		
	  invcdf=&invcdf_uni; ccdf=&ccdf_uni; invccdf=&invccdf_uni;
	  break;
	case 4:
	  pdf=&pdf_par1; distmean=&mean_par1; cdf=&cdf_par1;		
	  invcdf=&invcdf_par1; ccdf=&ccdf_par1; invccdf=&invccdf_par1;
	  break;
	case 5:
	  pdf=&pdf_par2; distmean=&mean_par2; cdf=&cdf_par2;		
	  invcdf=&invcdf_par2; ccdf=&ccdf_par2; invccdf=&invccdf_par2;
	  break;
	}
      param1=atof(argv[3]);
      param2=atof(argv[4]);
      ResultFromFile();
      printp(bestp,bestbeta);
      memalloc2();

      sptailtype=atoi(argv[8]);
      if(sptailtype!=0)
	{
	  sptailparam=atof(argv[9]);
	  if(atof(argv[10])>0)
	    sptailstart=atof(argv[10]);
	  else
	    {
	      sptailstart=invccdf(-atof(argv[10]),param1,param2);
	      printf(" Start of special tail: %1.4e\n", sptailstart);
	    }
	}
      if(atof(argv[7])>0.0)
	genlist(atof(argv[5]),atof(argv[6]),fabs(atof(argv[7])),1);
      else
	genlist(atof(argv[5]),atof(argv[6]),fabs(atof(argv[7])),0);
      
      memfree2();
      return 0;
    }
  // End of Producing data file to plot curves in case of pdf

  // Fitting a distribution given by samples;
  if(atoi(argv[1])==3)
    {
      switch(atoi(argv[3]))
	{
	case 1:
	  loadcdf(argv[2]);
	  break;
	case 2:
	  loadccdf(argv[2]);
	  break;
	case 3:
	  loadsamples(argv[2]);
	  break;
	case 4:
	  loadcdf(argv[2]);
	  break;
	}
      pdf=&pdf_loaded;
      distmean=&mean_loaded;	
      cdf=&cdf_loaded;
      invcdf=&invcdf_loaded;	
      ccdf=&ccdf_loaded;	
      invccdf=&invccdf_loaded;

      if(atoi(argv[3])==3 || atoi(argv[3])==4)
	createcdf();
      if(atoi(argv[3])==1 || atoi(argv[3])==2)
	createsamples();

      l=atoi(argv[4]);
      if(atof(argv[5])>=0) 
	tail=atof(argv[5]);
      else 
	{ 
	  tail=invccdf(-atof(argv[5]),param1,param2);	
	  //printf(" End of fitting the body: %1.4e\n",tail);
	  printf(" End of fitting the body: %f\n",tail);
	}
      method=atoi(argv[6]);
      if(method==2) method=6;
      if(method==3) method=8;
      if(method==1) method=2;
      lh=atoi(argv[7]);
      if(atof(argv[8])>=0) 
	mtail1=atof(argv[8]);	
      else 
	{
	  mtail1=invccdf(-atof(argv[8]),param1,param2); 
	  printf(" Lower limit of fitting the tail: %1.4e\n", mtail1);
	}
      if(atof(argv[9])>=0) 
	{
	  mtail2=atof(argv[9]);	
	}
      else 
	{
	  mtail2=invccdf(-atof(argv[9]),param1,param2); 
	  //printf(" Upper limit of fitting the tail: %1.4e\n", mtail2);
	  printf(" Upper limit of fitting the tail: %f\n", mtail2);
	}
      adj=atof(argv[10]);		
      sptailtype=atoi(argv[11]);
      if(sptailtype!=0)
	{
	  sptailparam=atof(argv[12]);
	  if(atof(argv[13])>0)
	    sptailstart=atof(argv[13]);
	  else
	    {
	      sptailstart=invccdf(-atof(argv[13]),param1,param2);
	      printf(" Start of special tail: %1.4e\n", sptailstart);
	    }
	}
      NUMBEROFROUNDS=atoi(argv[14]);
      NUMBEROFITERATIONS=atoi(argv[15]);
      intpr=atoi(argv[16]);

      memalloc();

      mean=distmean(param1,param2);
      buildandcheck();
      for(i=0;i<lh;i++) mean-=h[i]/lambda[i];
	  
      //printf("Mean changed by The Mixture: %f\n",mean);
      //printf("Mean unchanged by The Mixture: %f\n",distmean(param1,param2));
      //printf("ML_Limit: %f, Limit1: %f, Limit2: %f\n", tail,mtail1,mtail2);
      //fflush(stdout);
      srand(1);
      
      numofit=NUMBEROFITERATIONS;

      bestlike=-10000000.0;
      for(i=1;i<=NUMBEROFROUNDS;i++)
	{
	  printf("%d. round\n",i);
	  fflush(stdout);
	  printf("  Looking for good starting values...\n");
	  fflush(stdout);
	  stmean=(double)rand()/RAND_MAX*mean+0.5*mean;
	  //printf(" stmean: %1.4e\n", stmean);
	  if(stmean<0) stmean=0.1;
	  stvalues(stmean);
	  printf("  Starting values are defined\n");
	  fflush(stdout);
	  //printf("startting values n. %d:\n ", i);
	  //printp(p,beta);
	  
	  // Maximum of change in parameters
	  Deltap=0.1;
	  Deltabeta=0.1;
	  
	  for(j=1;j<=numofit;j++)
	    {
	      printf("  Modification n. %3d\n",j);
	      fflush(stdout);
/* 	      printf(".",j); */
/* 	      if( j % 50 == 0 )  */
/* 		{ */
/* 		  printf("\n "); */
/* 		  fflush(stdout); */
/* 		} */
	      //printf(" Parameters:\n");
	      //printp(p,beta);
	      //fflush(stdout);
	      if( method==1 ) derive1();
	      if( method>1 ) derive2();
	      //printf(" Derivatives:\n");
	      //printp(derp, derb);
	      //fflush(stdout);
	      lp();
	      //printf(" Lin prog results:\n");
	      //printp(dp,dbeta);
	      //fflush(stdout);
	      //fprintf("  lp\n");
	      if(along()==0)
		{
		  if( Deltap<0.01 )
		    {
		      //printf("\n   Number of modifications: %d\n", j);
		      j=1000;
		    }
		  else 
		    {
		      Deltap*=0.9;
		      Deltabeta*=0.9;
		    }
		}
	      // Delta=0.995*Delta;
	    }
	  
	  printf("\nEnd of round n. %d:\n\n", i);
	  fflush(stdout);
	  //printp(p,beta);
	  
	  if( method==1 || method==2 ) lik=likely(p,beta);
	  if( method==3 ) lik=areadiff(p,beta);
	  if( method==4 ) lik=relareadiff(p,beta);
	  if( method==5 ) lik=int_likely(p,beta);
	  if( method==6 ) lik=int_areadiff(p,beta);
	  if( method==7 ) lik=int_relareadiff(p,beta);   
	  if( method==8 ) lik=int_cdfareadiff(p,beta);   
	  
	  //printf("lik: %1.4e\n",lik);
	  if( bestlike<lik)
	    {
	      bestlike=lik;
	      for(j=1;j<=l;j++)
		{
		  bestbeta[j]=beta[j];
		  bestp[j]=p[j];
		}
	    }
	}
      //printp(bestp,bestbeta);
      Result2File();
      memfree();
      printf("Fitting has finished.\n");
      return 0;
    }
  // End of Fitting a distribution given by pdf 

  // Producing data file to plot curves in case of samples
  if(atoi(argv[1])==4)
    {
      switch(atoi(argv[3]))
	{
	case 1:
	  loadcdf(argv[2]);
	  break;
	case 2:
	  loadccdf(argv[2]);
	  break;
	case 3:
	  loadsamples(argv[2]);
	  break;
	case 4:
	  loadcdf(argv[2]);
	  break;
	}
      pdf=&pdf_loaded;
      distmean=&mean_loaded;	
      cdf=&cdf_loaded;
      invcdf=&invcdf_loaded;	
      ccdf=&ccdf_loaded;	
      invccdf=&invccdf_loaded;

      if(atoi(argv[3])==3 || atoi(argv[3])==4)
	createcdf();

      ResultFromFile();
      printp(bestp,bestbeta);
      memalloc2();

      sptailtype=atoi(argv[7]);
      if(sptailtype!=0)
	{
	  sptailparam=atof(argv[8]);
	  if(atof(argv[9])>0)
	    sptailstart=atof(argv[9]);
	  else
	    {
	      sptailstart=invccdf(-atof(argv[9]),param1,param2);
	      printf(" Start of special tail: %1.4e\n", sptailstart);
	    }
	}
      if(atof(argv[6])>0.0)
	genlist(atof(argv[4]),atof(argv[5]),fabs(atof(argv[6])),1);
      else
	genlist(atof(argv[4]),atof(argv[5]),fabs(atof(argv[6])),0);
      
      memfree2();
      return 0;
    }
  // End of Producing data file to plot curves in case of samples


  if(argc==17)
    {
      switch(atoi(argv[1]))
	{
	case 0:
	  // distribution function loaded from file given in param1
	  loadcdf(argv[2]);
	  pdf=&pdf_loaded;
	  distmean=&mean_loaded;	
	  cdf=&cdf_loaded;
	  invcdf=&invcdf_loaded;	
	  ccdf=&ccdf_loaded;	
	  invccdf=&invccdf_loaded;
	  break;
	case 1:
	  pdf=&pdf_weib;	
	  distmean=&mean_weib;	
	  cdf=&cdf_weib;		
	  invcdf=&invcdf_weib;	
	  ccdf=&ccdf_weib;	
	  invccdf=&invccdf_weib;
	  break;
	case 2:
	  pdf=&pdf_logn;	
	  distmean=&mean_logn;	
	  cdf=&cdf_logn;		
	  invcdf=&invcdf_logn;	
	  ccdf=&ccdf_logn;	
	  invccdf=&invccdf_logn;
	  break;
	case 3:
	  pdf=&pdf_uni;	
	  distmean=&mean_uni;	
	  cdf=&cdf_uni;		
	  invcdf=&invcdf_uni;	
	  ccdf=&ccdf_uni;	
	  invccdf=&invccdf_uni;
	  break;
	case 4:
	  pdf=&pdf_par1;	
	  distmean=&mean_par1;	
	  cdf=&cdf_par1;		
	  invcdf=&invcdf_par1;	
	  ccdf=&ccdf_par1;	
	  invccdf=&invccdf_par1;
	  break;
	case 5:
	  pdf=&pdf_par2;	
	  distmean=&mean_par2;	
	  cdf=&cdf_par2;		
	  invcdf=&invcdf_par2;	
	  ccdf=&ccdf_par2;	
	  invccdf=&invccdf_par2;
	  break;
	}
      
      param1=atof(argv[2]);
      param2=atof(argv[3]);
      l=atoi(argv[4]);
      if(atof(argv[5])>=0) 
	tail=atof(argv[5]);
      else 
	tail=invccdf(atof(argv[5]),param1,param2);	
      method=atoi(argv[6])+4;
      lh=atoi(argv[7]);
      if(atof(argv[8])>=0) 
	mtail1=atof(argv[8]);	
      else 
	mtail1=invccdf(-atof(argv[8]),param1,param2); 
      if(atof(argv[9])>=0) 
	mtail2=atof(argv[9]);	
      else 
	mtail2=invccdf(-atof(argv[9]),param1,param2); 
      mtail3=mymax(tail,mtail2);	// upper limit for computing measures (should be modified)
      adj=atof(argv[10]);		
    }
  else
    {
      
      l=atoi(argv[1]);
      method=atoi(argv[6])+4;
      lh=atoi(argv[7]);
    }
  
  memalloc();
  
  if(lh>0) build_exp();
  mean=distmean(param1,param2);
  for(i=0;i<lh;i++) mean-=h[i]/lambda[i];
  printf("Mean changed by The Mixture: %f\n",mean);
  printf("Mean unchanged by The Mixture: %f\n",distmean(param1,param2));
  printf("ML_Limit: %f, Limit1: %f, Limit2: %f\n", tail,mtail1,mtail2);
  
  srand(1);
  
  numofit=800;
  if( l<=6 ) numofit=400; 
  
  bestlike=-10000000.0;
  for(i=1;i<=10;i++)
    {
      stvalues((double)rand()/RAND_MAX*(2*mean-0.5*mean)+0.5*mean);
      //printf("startting values n. %d:\n ", i);
      //printp(p,beta);
            
      // Maximum of change in parameters
      Deltap=0.1;
      Deltabeta=0.1;
      
      for(j=1;j<=numofit;j++)
	{
	  printf(".",j);
	  if( j % 50 == 0 ) printf("\n ");
	  if( method==1 ) derive1();
	  
	  printf(" Parameters:\n");
	  printp(p,beta);
	  if( method > 1 ) derive2();
	  printf(" Derivatives:\n");
	  printp(derp, derb);
	  lp();
	  printf(" Lin prog results::\n");
	  printp(dp,dbeta);
	  // printf("  lp\n");
	  if(along()==0)
	    {
	      if( Deltap<0.01 )
		{
		  printf("\n   Number of modifications: %d\n", j);
		  j=1000;
		}
	      else 
		{
		  Deltap*=0.9;
		  Deltabeta*=0.9;
		}
	    }
	  // Delta=0.995*Delta;
	}
      
      printf("\nEnd of round n. %d:\n\n ", i);
      //printp(p,beta);

      if( method==1 || method==2 ) lik=likely(p,beta);
      if( method==3 ) lik=areadiff(p,beta);
      if( method==4 ) lik=relareadiff(p,beta);
      if( method==5 ) lik=int_likely(p,beta);
      if( method==6 ) lik=int_areadiff(p,beta);
      if( method==7 ) lik=int_relareadiff(p,beta);   
      if( method==8 ) lik=int_cdfareadiff(p,beta);   
      
      printf("lik: %1.4e\n",lik);
      if( bestlike<lik)
	{
	  bestlike=lik;
	  for(j=1;j<=l;j++)
	    {
	      bestbeta[j]=beta[j];
	      bestp[j]=p[j];
	    }
	}
    }
  
  printp(bestp,bestbeta);
  
  m = (struct measures *)malloc(sizeof(struct measures));
  
  if(argc==17)
    {
      //compmeasures2(m);  
      writeparam(argv[11]);
      //genlist(argv[12], atof(argv[13]), atof(argv[14]), atof(argv[15]), atof(argv[16]));
    }
  else
    {
      printf("Comp Meas Started\n");
      compmeasures2(m);
      printf("Gen List Started\n");
      writeparam(argv[2]);
      //genlist(argv[3], atof(argv[4]), atof(argv[5]),0,0);
    }	
  
  printf("\nMeasures:\n");
  printf("  Likelyhood:                    %1.4e\n", -m->ent);	
  printf("  Areadifference:                %1.4e\n", m->ad); 
  printf("  Relative Areadifference:       %1.4e\n", m->rad);
  printf("  Relative Error in 1st Moment:  %1.4e\n", m->m1);
  printf("  Relative Error in 2nd Moment:  %1.4e\n", m->m2);
  printf("  Relative Error in 3rd Moment:  %1.4e\n", m->m3);	
  
  // Printing tex tabular like format:
  printf("\n %1.4e & %1.4e & %1.4e & %1.4e & %1.4e & %1.4e \\\\ \n",
	 -m->ent,m->ad,m->rad,m->m1,m->m2,m->m3);
  
  free(m);
  memfree();
  return 0;
}


