#include<stdio.h>
#include<math.h>
#include<malloc.h>
#include<stdlib.h>

#include"linprog.h"
#include "alldist.h"

//#define DEBUG

int NUMBEROFROUNDS;       // Number of rounds
int NUMBEROFITERATIONS;   // Max. number of modifications in 1 round
#define CDFINTEG 10000           // Integrating cdf with this number

// functions to describe the distribution
double (* pdf)(double,double,double);	
double (* distmean)(double,double);
double (* cdf)(double,double,double);
double (* invcdf)(double,double,double);
double (* ccdf)(double,double,double);
double (* invccdf)(double,double,double);
double param1, param2;
double endofmainfitting;

int lastcdfindex=0;
int samples;

int cdfnum;
double *cdfpoints,*cdfvalues;

double sptailstart,sptailparam;
int sptailtype;

struct measures
{
  double m1,m2,m3,ent,ad,rad;
};

struct matrixserie
{
  double **m;
};

int exact_repr, nec_phases;
int method;
int tail, tailw;
double mean, appmean, sofw;
double **T;
struct matrixserie *ms;
double *derp, *derb;    /* derivatives with respect to b and beta */
int *obs; 	        /* set of observations */
double *weight; 	/* weights of observations*/
double *weight2; 	/* weights of observations*/
int *dist; 	        /* discrete distribution */
double *mass;	 	/* masses of the distribution */
double *mass2;	 	/* masses of the distribution */
double *p, *beta, *bestp, *bestbeta, *beststp, *beststbeta;	
double *dp, *dbeta;
double *actp, *actbeta;
int l, ol;			/* length of chain and observations*/
int od;				/* length of distribution */			
double Delta;	        	/* maximum change in the variables */
double fitting_delta;           /* stepsize of fitting */

double **a;		        /* variables used for lp */	
int *izrov, *iposv;


// variables describing the mixture of exp. part
double mtail1,mtail2;	// limits to compute hypergem. part
double *h,*lambda;      // hypergem parameters
int lh;			// number of exponentials mixed
double adj;		// to define distance of adjacent points
double ph;		// sum of initial pr.-s of exponentials mixed
double decay;           // decay of the tail part
double norm_const;       // normalizing constant of the tail

// ------------------------
// mean based on loaded cdf
// ------------------------
double mean_loaded(double y, double z)
{
  int i1;
  double s;
  
  //printf(" s: %1.4e\n", s);
  s=cdfpoints[0]*cdfvalues[0];
  for(i1=1;i1<cdfnum;i1++)
    {
      //printf(" s: %1.4e\n", s);
      s+=cdfpoints[i1]*(cdfvalues[i1]-cdfvalues[i1-1]);
    }
  
  return s;
}

double pdf_loaded(double x, double p1, double p2)
{
  int i1=0;
  double tmp1;

  while(cdfpoints[i1]<x) 
    {
      i1++;
      if(i1==cdfnum)
	{
	  return 0;
	}
    }
  if(i1==0)
    {
      return cdfvalues[0]/2/cdfpoints[0];
    }
  if(i1==1)
    {
      tmp1=cdfvalues[0]/2-(cdfvalues[2]+cdfvalues[1])/2;
      return tmp1/cdfpoints[0];
    }
  
  tmp1=-(cdfvalues[i1-1]+cdfvalues[i1-2])/2+(cdfvalues[i1]+cdfvalues[i1-1])/2;
  //printf( "%1.4e %1.4e\n", tmp1,cdfpoints[i1]-cdfpoints[i1-1]);
  return tmp1/(cdfpoints[i1]-cdfpoints[i1-1]);
}

double cdf_loaded(double x, double p1, double p2)
{
  int i1=0;
  
  if(x<cdfpoints[0]) 
    {
      //printf("%1.8e\n", 0.0);
      return 0.0;
    }
  if(lastcdfindex>0)
    if(x>cdfpoints[lastcdfindex-1])
      i1=lastcdfindex;
  //printf(" cdf_loaded(%1.8e)=", x);
  while(x>cdfpoints[i1]) 
    {
      i1++;
      if(i1==cdfnum) 
	{
	  //printf("%1.8e\n", 1.0);
	  return 1.0;
	}
    }
  lastcdfindex=i1-1;
  //printf("%1.8e\n", cdfvalues[i1-1]);
  return cdfvalues[i1-1];
}
	
double invcdf_loaded(double x, double p1, double p2)
{
  int i1=0;
  
  if(x<cdfvalues[0]) return cdfpoints[0];
  while(cdfvalues[i1]<x)
    {
      i1++;
      if(i1==cdfnum)
	{
	  return cdfpoints[cdfnum-1];
	}
    }
  return cdfpoints[i1];
}

double ccdf_loaded(double x, double p1, double p2)
{
  return 1-cdf_loaded(x,0,0);
} 

double invccdf_loaded(double x, double p1, double p2)
{
  return invcdf_loaded(1-x,0,0);
} 



// ----------------------
// read samples from file
// ----------------------
void loadsamples(char *fn)
{
  FILE *f;
  int i1=0,success,bytes;
  double tmp1,tmp2;

  cdfpoints=NULL;
  cdfvalues=NULL;
  f=fopen(fn,"r");
  while(!feof(f))
    {
      success=1;
      if((bytes=fscanf(f, "%le", &tmp1))==0 || bytes==-1)
	success=0;
      //printf("bytes: %d", bytes);
      if(success)
	{
	  i1++;
	  cdfpoints=(double*)realloc(cdfpoints,i1*sizeof(double));
	  cdfvalues=(double*)realloc(cdfvalues,i1*sizeof(double));
	  cdfpoints[i1-1]=tmp1;
	  cdfvalues[i1-1]=1.0;
	  //printf(" %le %le\n", cdfpoints[i1-1], cdfvalues[i1-1]);
	}
    }
  cdfnum=i1;
}

// ===============================
// create cdf if samples are given
void createcdf(void)
{
  int i1;
  double sum=0;

  //ol=cdfnum;
  //weight=(double *)malloc(sizeof(double)*(ol+1));
  //obs=(double *)malloc(sizeof(double)*(ol+1));
  for(i1=0;i1<cdfnum;i1++)
    {
      //obs[i1+1]=cdfpoints[i1];
      //weight[i1+1]=cdfvalues[i1];
      sum+=cdfvalues[i1];
    }
  cdfvalues[0]=cdfvalues[0]/sum;
  for(i1=1;i1<cdfnum;i1++)
    {
      cdfvalues[i1]=cdfvalues[i1]/sum+cdfvalues[i1-1];
    }
}

// =============================
//create samples is cdf is given
void createsamples()
{
  int i1;

  ol=cdfnum;
  weight=(double *)malloc(sizeof(double)*(ol+1));
  obs=(int *)malloc(sizeof(double)*(ol+1));

  obs[1]=cdfpoints[0];
  weight[1]=cdfvalues[0];
  for(i1=1;i1<cdfnum;i1++)
    {
      obs[i1+1]=cdfpoints[i1];
      weight[i1+1]=cdfvalues[i1]-cdfvalues[i1-1];
    }
  //for(i1=1;i1<=cdfnum;i1++)
  //printf(" %1.4e, %1.4e\n", obs[i1], weight[i1]);
}  


// ------------------
// read cdf from file
// ------------------
void loadcdf(char *fn)
{
  FILE *f;
  int i1=0,success,bytes;
  double tmp1,tmp2;

  cdfpoints=NULL;
  cdfvalues=NULL;
  f=fopen(fn,"r");
  while(!feof(f))
    {
      success=1;
      if((bytes=fscanf(f, "%le", &tmp1))==0 || bytes==-1)
	success=0;
      //printf("bytes: %d", bytes);
      if((bytes=fscanf(f, "%le", &tmp2))==0 || bytes==-1)
	success=0;
      //printf("bytes: %d", bytes);
      if(success)
	{
	  i1++;
	  cdfpoints=(double*)realloc(cdfpoints,i1*sizeof(double));
	  cdfvalues=(double*)realloc(cdfvalues,i1*sizeof(double));
	  cdfpoints[i1-1]=tmp1;
	  cdfvalues[i1-1]=tmp2;
	  //printf(" %le %le\n", cdfpoints[i1-1], cdfvalues[i1-1]);
	}
    }
  cdfnum=i1;
}

// ------------------
// read ccdf from file
// ------------------
void loadccdf(char *fn)
{
  FILE *f;
  int i1=0,success,bytes;
  double tmp1,tmp2;

  cdfpoints=NULL;
  cdfvalues=NULL;
  f=fopen(fn,"r");
  while(!feof(f))
    {
      success=1;
      if((bytes=fscanf(f, "%le", &tmp1))==0 || bytes==-1)
	success=0;
      //printf("bytes: %d", bytes);
      if((bytes=fscanf(f, "%le", &tmp2))==0 || bytes==-1)
	success=0;
      //printf("bytes: %d", bytes);
      if(success)
	{
	  i1++;
	  cdfpoints=(double*)realloc(cdfpoints,i1*sizeof(double));
	  cdfvalues=(double*)realloc(cdfvalues,i1*sizeof(double));
	  cdfpoints[i1-1]=tmp1;
	  cdfvalues[i1-1]=1-tmp2;
	  //printf(" %le %le\n", cdfpoints[i1-1], cdfvalues[i1-1]);
	}
    }
  cdfnum=i1;
}



// ************************** //
// Discretizing the cont. pdf //
// ************************** //
void discretize(void)
{
  int i1,i2;
  double w,sum=0.0;
  
  ol=ceil(endofmainfitting/fitting_delta);
  weight=(double *)malloc(sizeof(double)*(ol+1));
  weight2=(double *)malloc(sizeof(double)*(ol+1));
  obs=(int *)malloc(sizeof(int)*(ol+1));
  
  for(i1=1;i1<=ol;i1++)
    {
      //printf(" %d\n", i1);
      obs[i1]=i1;
      if(samples==0)
        weight[i1]=pdf(i1*fitting_delta,param1,param2);
      else
        if(i1==1)
	  weight[i1]=cdf((i1+0.5)*fitting_delta,param1,param2);
        else
	  weight[i1]=cdf((i1+0.5)*fitting_delta,param1,param2)-
	    cdf((i1-0.5)*fitting_delta,param1,param2);
      if(method==5)
	{
	  weight[i1]=0.0;
	  weight2[i1]=0.0;
	  for(i2=0;i2<CDFINTEG;i2++)
	    {
	      weight[i1]=weight[i1]+
		cdf((i1-1)*fitting_delta+fitting_delta/CDFINTEG*i2,param1,param2)
		/CDFINTEG;
	      weight2[i1]=weight2[i1]+
		cdf((i1-1)*fitting_delta+fitting_delta/CDFINTEG*i2,param1,param2)*
		cdf((i1-1)*fitting_delta+fitting_delta/CDFINTEG*i2,param1,param2)
		/CDFINTEG;
	    }
	}
      sum+=weight[i1];
      // should be different in case of cdf areadiff.
    }
  if(method!=5)
    {
      w=ccdf(ol*fitting_delta,param1,param2);
      for(i1=1;i1<=ol;i1++) 
	{
	  weight[i1]=weight[i1]/sum-weight[i1]/sum*w;
	  weight2[i1]=0.0;
	}
    }
  else
    {
      //weight[ol]=1.0;
      //weight2[ol]=1.0;
    }
  //for(i1=1;i1<=ol;i1++)
  //printf("%d %f %f\n", obs[i1], weight[i1], weight2[i1]);
}

/**************************/
/* density of The Mixture */
/**************************/
double mixture(int t)
{
  double ft=0;
  int i;
  
  for(i=0;i<lh;i++) 
    ft+=h[i]*lambda[i]*pow(1-lambda[i],t-1);
  return ft;
}

/**************************/
/* density of The Mixture */
/**************************/
double mixtureccdf(int t)
{
  double ft=0;
  int i;
  
  for(i=0;i<lh;i++) 
    ft+=h[i]*pow(1-lambda[i],t);
  return ft;
}

// This was used to create the example in TOOLS2002
/***********************************/
/* build part of mixed geometrical */
/***********************************/
void build_geomTOOLS2002(void)
{
  int i,j,i1;
  double x1,x2,t,m,b,sum=0,distsum=0,tailsum=0;
  
  m=pow(mtail2/mtail1,1/((double)lh-1));  // distance of quantiles
  b=pow(m,adj);				  // distance of adjecent points 
  printf("%e %e m: %e\n",mtail2/mtail1,1/((double)lh-1),m);
  t=mtail1;
  for(i=0;i<lh-1;i++) t*=m;
  sum=0;
  
  //for(i1=1;i1<=ol;i1++) 
  //  distsum+=weight[i1];
  //for(i1=ol+1;i1<=2*floor(t*b);i1++)
  //  {
  //    tailsum+=weight[ol]*pow((double)i1/(double)ol,-(decay+1));
  //    mean+=i1*weight[ol]*pow((double)i1/(double)ol,-(decay+1));
  //  }
  //norm_const=(1-distsum)/tailsum;
  //printf(" distsum: %1.4e, tailsum: %1.4e, norm_const: %1.4e\n",
  // distsum, tailsum,norm_const);
  
  printf("t: %e, b*t: %e\n",t,b*t);

  norm_const=(1-weight[ol])/fitting_delta/pow(ol,-decay);

  for(i=0;i<lh;i++)
    {
      //x1=distsum;
      //x2=distsum;
      //for(i1=ol+1;i1<floor(b*t);i1++)
      //	{
      //  x2+=weight[ol]*pow((double)i1/(double)ol,-(decay+1))*norm_const;
      //  if(i1<=floor(t))
      //    x1+=weight[ol]*pow((double)i1/(double)ol,-(decay+1))*norm_const;
      //	}
      x1=norm_const*pow(floor(t),-decay);
      x2=norm_const*pow(floor(b*t),-decay);
      //x1=1-x1;
      //x2=1-x2;
      printf("ccdf1: %1.4e, ccdf2: %1.4e\n",x1,x2);
      for(j=0;j<i;j++)
	{
	  x1-=h[j]*pow(1-lambda[j],floor(t));
	  x2-=h[j]*pow(1-lambda[j],floor(t*b));
	}
      printf("ccdf1: %1.4e, ccdf2: %1.4e\n",x1,x2);
      lambda[i]=1-pow(x1/x2,1/(floor(t)-floor(t*b)));
      h[i]=x1*pow(1-lambda[i],-floor(t));
      sum+=h[i];	
      t=t/m;
      if(i!=lh-1)
	printf("t: %e, b*t: %e\n",t,b*t);
    }
  
  printf("Mixture: \n\th\t\tlambda\n");
  for(i=0;i<lh;i++) printf("\t%e\t%e\n",h[i],lambda[i]); 
  printf("Mixture: %e\n", mixture(1));
}

/***********************************/
/* build part of mixed geometrical */
/***********************************/
void build_geom(void)
{
  int i,j,i1;
  double x1,x2,t,m,b,sum=0,distsum=0,tailsum=0,tailconstant;

  if(sptailtype!=0)
    {
      if(sptailtype==1)
	tailconstant=ccdf(sptailstart,param1,param2)/
	  pow(sptailstart,-sptailparam);
      else
	tailconstant=ccdf(sptailstart,param1,param2)/
	  exp(-pow(sptailstart,sptailparam));
    }

  m=pow(mtail2/mtail1,1/((double)lh-1));  // distance of quantiles
  b=pow(m,adj);				  // distance of adjecent points 
  //printf("%e %e m: %e\n",mtail2/mtail1,1/((double)lh-1),m);
  t=mtail1;
  for(i=0;i<lh-1;i++) t*=m;
  sum=0;
  
  //for(i1=1;i1<=ol;i1++) 
  //  distsum+=weight[i1];
  //for(i1=ol+1;i1<=2*floor(t*b);i1++)
  //  {
  //    tailsum+=weight[ol]*pow((double)i1/(double)ol,-(decay+1));
  //    mean+=i1*weight[ol]*pow((double)i1/(double)ol,-(decay+1));
  //  }
  //norm_const=(1-distsum)/tailsum;
  //printf(" distsum: %1.4e, tailsum: %1.4e, norm_const: %1.4e\n",
  // distsum, tailsum,norm_const);
  
  //printf("t: %e, b*t: %e\n",t,b*t);

  norm_const=(1-weight[ol])/fitting_delta/pow(ol,-decay);

  for(i=0;i<lh;i++)
    {
      if(t>sptailstart&&sptailtype!=0)
	if(sptailtype==1)
	  x1=tailconstant*pow(t,-sptailparam);
	else
	  x1=tailconstant*exp(-pow(t,sptailparam));
      else
	x1=ccdf(t,param1,param2);
      if(b*t>sptailstart&&sptailtype!=0)
	if(sptailtype==1)
	  x2=tailconstant*pow(b*t,-sptailparam);
	else
	  x2=tailconstant*exp(-pow(b*t,sptailparam));
      else
	x2=ccdf(b*t,param1,param2);
      printf("   ccdf(%1.4e)=%1.4e\n   ccdf(%1.4e)=%1.4e\n",b*t,x2,t,x1);

      //x1=distsum;
      //x2=distsum;
      //for(i1=ol+1;i1<floor(b*t);i1++)
      //	{
      //  x2+=weight[ol]*pow((double)i1/(double)ol,-(decay+1))*norm_const;
      //  if(i1<=floor(t))
      //    x1+=weight[ol]*pow((double)i1/(double)ol,-(decay+1))*norm_const;
      //	}
      //x1=1-x1;
      //x2=1-x2;
      //printf("ccdf1: %1.4e, ccdf2: %1.4e\n",x1,x2);
      for(j=0;j<i;j++)
	{
	  x1-=h[j]*pow(1-lambda[j],floor(t/fitting_delta));
	  x2-=h[j]*pow(1-lambda[j],floor(t*b/fitting_delta));
	}
      //printf("ccdf1: %1.4e, ccdf2: %1.4e\n",x1,x2);
      lambda[i]=1-pow(x1/x2,1/(floor(t/fitting_delta)-floor(t*b/fitting_delta)));
      h[i]=x1*pow(1-lambda[i],-floor(t/fitting_delta));
      sum+=h[i];	
      t=t/m;
      //if(i!=lh-1)
      //	printf("t: %e, b*t: %e\n",t,b*t);
    }
  
  //printf("Mixture: \n\th\t\tlambda\n");
  //for(i=0;i<lh;i++) printf("\t%e\t%e\n",h[i],lambda[i]); 
  //printf("Mixture: %e\n", mixture(1));
}


// ********************************
// Check if exact repr. is possible
// ********************************
void is_it_exact()
{
  int i1;
  
  if(method==5)
    {
      exact_repr=0;
      for(i1=1;i1<=l+1 && i1<=ol;i1++)
	if(weight[i1]>=0.99)
	  {
	    exact_repr=1;
	    nec_phases=i1-1;
	    i1=l+2;
	  }
    }
  else
    {
      exact_repr=1;
      for(i1=l+1;i1<=ol;i1++)
	{
	  if(weight[i1]>=0.01)
	    exact_repr=0;
	}
      nec_phases=l;
    }
  // meg kene irni mas modszerekre is !!!!
}

// *********************
// Reading pdf from file
// *********************
double readobs(char *fn)
{
  int i;
  FILE *f;
  
  f=fopen(fn,"r");
  fscanf(f, "%d", &ol);
  
  mean=0;
  sofw=0;
  weight=(double *)malloc(sizeof(double)*(ol+1));
  obs=(int *)malloc(sizeof(int)*(ol+1));
  
  /*printf("ol: %d\n", ol);*/
  for(i=1;i<=ol;i++)
    {
      fscanf(f, "%d", &obs[i]);
      fscanf(f, "%lf", &weight[i]);
      mean+=obs[i]*weight[i];
      sofw+=weight[i];
    }
  /*for(i=1;i<=ol;i++)*/
  /*	printf("%d %f\n", obs[i], weight[i]);*/
  fclose(f);
  return mean/sofw;
}

// ******************************************
// Reading info to compute cdf diff from file
// ******************************************
double readobs2(char *fn)
{
  int i;
  FILE *f;
  
  f=fopen(fn,"r");
  fscanf(f, "%d", &ol);
  
  mean=0;
  sofw=0;
  weight=(double *)malloc(sizeof(double)*(ol+1));
  weight2=(double *)malloc(sizeof(double)*(ol+1));
  obs=(int *)malloc(sizeof(int)*(ol+1));
  weight[0]=0.0;

  /*printf("ol: %d\n", ol);*/
  for(i=1;i<=ol;i++)
    {
      fscanf(f, "%d", &obs[i]);
      fscanf(f, "%lf", &weight[i]);
      fscanf(f, "%lf", &weight2[i]);
      mean+=obs[i]*(weight[i]-weight[i-1])/fitting_delta;
      sofw+=weight[i];
    }
  /*for(i=1;i<=ol;i++)*/
  /*	printf("%d %f\n", obs[i], weight[i]);*/
  fclose(f);
  return mean;
}

// *****************
// Read distribution
// *****************
void readdist(char *fn)
{
  int i;
  FILE *f;
  
  f=fopen(fn,"r");
  fscanf(f, "%d", &od);
  
  mass=(double *)malloc(sizeof(double)*(od+1));
  dist=(int *)malloc(sizeof(int)*(od+1));
  
  /*printf("od: %d\n", od);*/
  for(i=1;i<=od;i++)
    {
      fscanf(f, "%d", &dist[i]);
      fscanf(f, "%lf", &mass[i]);
    }
  /*for(i=1;i<=ol;i++)*/
  /*	printf("%d %f\n", obs[i], weight[i]);*/
  fclose(f);
}

// ******************************************
// Reading info to compute cdf diff from file
// ******************************************
void readdist2(char *fn)
{
  int i;
  FILE *f;
  
  f=fopen(fn,"r");
  fscanf(f, "%d", &od);
  
  mass=(double *)malloc(sizeof(double)*(od+1));
  mass2=(double *)malloc(sizeof(double)*(od+1));
  dist=(int *)malloc(sizeof(int)*(od+1));
  
  /*printf("od: %d\n", od);*/
  for(i=1;i<=od;i++)
    {
      fscanf(f, "%d", &dist[i]);
      fscanf(f, "%lf", &mass[i]);
      fscanf(f, "%lf", &mass2[i]);
    }
  /*for(i=1;i<=ol;i++)*/
  /*	printf("%d %f\n", obs[i], weight[i]);*/
  fclose(f);
}


// ****************
// Print parameters
// ****************
void printp(double *p, double *b)
{
  int i;
  printf("\n");
  for(i=1;i<=l;i++)
    printf("p[%d]: %f, beta[%d]: %f\n", i, p[i], i, b[i]);
  printf("\n");
}

// *******
// Minimum
// *******
double min(double m1, double m2)
{
  if( m1>m2 ) return m2;
  return m1;
}

// *********************
// Set Parameters for LP
// *********************
void lp(void)
{
  int n,m1,m2,m3,m,i,j,k;
  int icase;
  double res;

#ifdef DEBUG
  printf(" Start simplex...\n");
#endif 

  n=2*l;
  m1=2*l+l-1;
  m2=0;
  m3=1;
  m=3*l;
  
  /* clear a */
  for(i=1;i<=m+2;i++)
    for(j=1;j<=n+1;j++) a[i][j]=0;
  
  /*printf("hulye\n");*/
  
  /* objective function */
  a[1][1]=0;
  for(i=1;i<=l;i++)
    {
      a[1][1+i]=derp[i];
      a[1][l+1+i]=derb[i];
    }
  
  /* 2*l m1 type ineqaulities */
  for(i=2;i<=l+1;i++)
    {
      a[i][1]=min(Delta,p[i-1])+min(Delta, 1-p[i-1]);
      a[i+l][1]=min(Delta,beta[i-1])+min(1-beta[i-1],Delta);
      a[i][i]=-1;
      a[i+l][i+l]=-1;
    }
  /* l-1 m1 type inequalities describing the odering of p values*/
  for(i=1;i<=l-1;i++)
    {
      a[2*l+1+i][1+i]=1;
      a[2*l+1+i][2+i]=-1;
      a[2*l+1+i][1]=-(p[i+1]-p[i]+min(Delta,p[i])-min(Delta,p[i+1]));
      if(a[2*l+1+i][1]<0) a[2*l+1+i][1]=0;
    }
  
  /* eqaulity */
  for(i=1;i<=l;i++)
    {
      a[3*l+1][1]+=min(Delta,beta[i]);
      a[3*l+1][l+1+i]=-1;
    }
  
  simplx(a, m, n, m1, m2, m3, &icase, izrov, iposv);
  
  /* results */
  for(i=1;i<=n;i++)
    {
      if(izrov[i]<=n) 
	{
	  if(izrov[i]<=l) { dp[izrov[i]]=0-min(Delta,p[izrov[i]]); }
	  else	{ dbeta[izrov[i]-l]=0-min(Delta,beta[izrov[i]-l]); }	
	}	
    }
  for(i=1;i<=m;i++)
    {
      if(iposv[i]<=n) 
	{
	  if(iposv[i]<=l) { dp[iposv[i]]=a[i+1][1]-min(Delta,p[iposv[i]]); }
	  else { dbeta[iposv[i]-l]=a[i+1][1]-min(Delta,beta[iposv[i]-l]); }
	}
    }

#ifdef DEBUG
  printf(" Simplex completed...\n");
#endif 

}

// ************
// Memory alloc
// ************
void memalloc(void)
{
  int i,j;
  
  T=(double **)malloc((l+1)*sizeof(double *));
  for(i=1;i<=l;i++)
    T[i]=(double *)malloc((l+1)*sizeof(double));
  
  ms = (struct matrixserie *)malloc(sizeof(struct matrixserie)*(l+1));
  for(i=1;i<=l;i++)
    {
      ms[i].m=(double **)malloc(sizeof(double *)*(l+2));
      for(j=1;j<=l+1;j++)
	ms[i].m[j]=(double *)malloc((l+2)*sizeof(double));
    }
  
  p = (double *)malloc( (l+1)*sizeof(double));
  beta = (double *)malloc( (l+1)*sizeof(double));
  bestp = (double *)malloc( (l+1)*sizeof(double));
  bestbeta = (double *)malloc( (l+1)*sizeof(double));
  beststp = (double *)malloc( (l+1)*sizeof(double));
  beststbeta = (double *)malloc( (l+1)*sizeof(double));
  dp = (double *)malloc( (l+1)*sizeof(double));
  dbeta = (double *)malloc( (l+1)*sizeof(double));
  actp = (double *)malloc( (l+1)*sizeof(double));
  actbeta = (double *)malloc( (l+1)*sizeof(double));
  derp=(double *)malloc(sizeof(double)*(l+1));
  derb=(double *)malloc(sizeof(double)*(l+1));
  
  a = (double **)malloc( (3*l+3)*sizeof(double *));
  for(i=1; i<=3*l+2; i++)
    a[i] = (double *)malloc( (2*l+2)*sizeof(double));
  izrov = (int *)malloc(sizeof(int)*(2*l+1));
  iposv = (int *)malloc(sizeof(int)*(3*l+1));

  if(lh>0)
    {
      lambda = (double *)malloc( (lh)*sizeof(double));
      h = (double *)malloc( (lh)*sizeof(double));
    }
}

// ************
// Memory alloc
// ************
void memalloc2(void)
{
  int i,j;
  
  T=(double **)malloc((l+1)*sizeof(double *));
  for(i=1;i<=l;i++)
    T[i]=(double *)malloc((l+1)*sizeof(double));
  
  ms = (struct matrixserie *)malloc(sizeof(struct matrixserie)*(l+1));
  for(i=1;i<=l;i++)
    {
      ms[i].m=(double **)malloc(sizeof(double *)*(l+2));
      for(j=1;j<=l+1;j++)
	ms[i].m[j]=(double *)malloc((l+2)*sizeof(double));
    }
  
  p = (double *)malloc( (l+1)*sizeof(double));
  beta = (double *)malloc( (l+1)*sizeof(double));
  beststp = (double *)malloc( (l+1)*sizeof(double));
  beststbeta = (double *)malloc( (l+1)*sizeof(double));
  dp = (double *)malloc( (l+1)*sizeof(double));
  dbeta = (double *)malloc( (l+1)*sizeof(double));
  actp = (double *)malloc( (l+1)*sizeof(double));
  actbeta = (double *)malloc( (l+1)*sizeof(double));
  derp=(double *)malloc(sizeof(double)*(l+1));
  derb=(double *)malloc(sizeof(double)*(l+1));
  
  a = (double **)malloc( (3*l+3)*sizeof(double *));
  for(i=1; i<=3*l+2; i++)
    a[i] = (double *)malloc( (2*l+2)*sizeof(double));
  izrov = (int *)malloc(sizeof(int)*(2*l+1));
  iposv = (int *)malloc(sizeof(int)*(3*l+1));
}


/* Building the Matrix Describing the Chain in CF-1 */
/* p: vector of transitions, l: length of p */
void makeT(void)
{
  int i,j;
  
  for(i=1;i<=l;i++)
    for(j=1;j<=l;j++)
      T[i][j]=0;
  T[1][1]=1;
  for(i=2;i<=l;i++)
    T[i][i]=1;
}


/* Building series of matrices adding different stages to the end of the chain */
void makeserT(void)
{
  int i,j,k;
  
  for(k=1;k<=l;k++)
    for(j=1;j<=l+1;j++)
      for(i=1;i<=l+1;i++)
	ms[k].m[j][i]=0;
  for(i=1;i<=l;i++)
    for(j=1;j<=l+1;j++)
      ms[i].m[j][j]=1;
}

// **********************
// One step normal matrix
// **********************
void step1T(double *pp)
{
  int i,j;
  
  for(i=1;i<=l;i++)
    {
      for(j=1;j<l;j++)
	{
	  T[i][j]=T[i][j]*(1-pp[j]);
	  T[i][j]+=T[i][j+1]*pp[j+1];
	}	
      T[i][l]=T[i][l]*(1-pp[l]);
    }
}

// ************************
// One step extended matrix
// ************************
void step1serT(void)
{
  int i,j,k;
  for(k=1;k<=l;k++)
    {
      for(i=1;i<=l+1;i++)
	{
	  ms[k].m[i][1]=ms[k].m[i][1]*(1-p[k]);
	  ms[k].m[i][1]+=ms[k].m[i][2]*p[1];
	  for(j=2;j<l+1;j++)
	    {
	      ms[k].m[i][j]=ms[k].m[i][j]*(1-p[j-1]);
	      ms[k].m[i][j]+=ms[k].m[i][j+1]*p[j];
	    }	
	  ms[k].m[i][l+1]=ms[k].m[i][l+1]*(1-p[l]);
	}
    }
}

// ************
// Print Matrix
// ************
void printT(void)
{
  int m,j;
  
  printf("T: \n");
  for(m=1;m<=4;m++)
    {
      for(j=1;j<=4;j++) printf(" %f, ", T[m][j]);
      printf("\n");
    }
}

// ********************************
// Print serie of extended matrices
// ********************************
void printserT(void)
{
  int k,m,j;
  
  for(k=1;k<=l;k++)
    {
      printf("T[%d]: \n", k);
      for(m=1;m<=l+1;m++)
	{
	  for(j=1;j<=5;j++) printf(" %f, ", ms[k].m[m][j]);
	  printf("\n");
	}
    }	
}

// ****************
// Compute Measures
// ****************
void compmeasures( struct measures *m)
{
  int prevobs,i,j;
  double nst,rm2=0,rm3=0,am2=0,am3=0;
  double realmean=0, appmean=0;
  
  for(i=1;i<=od;i++) realmean+=dist[i]*mass[i]; 
  i=1;
  nst=1;
  makeT();
  while(nst>EPS)
    {
      step1T(bestp);
      nst=0;
      for(j=1;j<=l;j++)
	nst+=bestbeta[j] * T[j][1] * bestp[1];
      appmean += nst*i;
      i++;
    }
  m->m1=fabs(realmean-appmean)/realmean;	
  m->ad=0; m->rad=0; m->ent=0;
  prevobs=1;
  makeT();
  for(i=1;i<=od;i++)
    {
      for(j=prevobs;j<=dist[i]-1;j++)
	step1T(bestp);
      prevobs=dist[i];
      nst=0;
      for(j=1;j<=l;j++)
	nst+=bestbeta[j] * T[j][1] * bestp[1];
      rm2 += mass[i]*(dist[i]-realmean)*(dist[i]-realmean);
      rm3 += mass[i]*(dist[i]-realmean)*(dist[i]-realmean)*(dist[i]-realmean); 
      am2 += nst*(dist[i]-appmean)*(dist[i]-appmean);
      am3 += nst*(dist[i]-appmean)*(dist[i]-appmean)*(dist[i]-appmean); 
      m->ad += fabs(nst-mass[i]);
      m->rad += fabs(nst-mass[i])/mass[i];
      /*printf("%d %e %e %e\n", i, mass[i], nst, fabs(nst-mass[i]));*/
      m->ent += mass[i]*log(nst);        
    }
  m->m2=fabs(rm2-am2)/rm2;
  m->m3=fabs(rm3-am3)/rm3;	
}

// ****************************
/* computing area difference */
// ****************************
double areadiff( double *pp, double *bb)
{
  int prevobs,i,j;
  double sum,nst;
  
  sum=0.0;
  prevobs=1;
  makeT();
  
  for(i=1;i<=ol;i++)
    {
      for(j=prevobs;j<=obs[i]-1;j++)
	step1T(pp);
      
      prevobs=obs[i];
      
      nst=0;
      for(j=1;j<=l;j++)
	nst+=bb[j] * T[j][1] * pp[1];
      nst+=mixture(obs[i]);
      if( obs[i]>=tail ) {sum += tailw*fabs(weight[i]-nst);}
      else  {sum += fabs(weight[i]-nst);}
      //printf("obs: %d, weight: %f, nst: %f, sum: %f\n", prevobs, weight[i], nst, sum);
    }
  return -sum;
}


// ****************************
/* computing cdf area difference */
// ****************************
// csak akkor mukodik ha obs[i]=obs[i-1]+1 !!!!!
// *********************************************
double cdfareadiff( double *pp, double *bb)
{
  int prevobs,i,j;
  double sum,nst;
  
  sum=0;
  prevobs=1;
  makeT();
  
  nst=0;
  for(i=1;i<=ol;i++)
    {
      for(j=prevobs;j<=obs[i]-1;j++)
	step1T(pp);
      
      prevobs=obs[i];
      
      if( obs[i]>=tail ) 
	{
	  //sum += tailw*(weight2[i]-2*nst*weight[i]+nst*nst*fitting_delta);
	  sum += tailw*(weight2[i]-2*nst*weight[i]+nst*nst);
	}
      else       
	{
	  //sum += weight2[i]-2*nst*weight[i]+nst*nst*fitting_delta;
	  sum += weight2[i]-2*nst*weight[i]+nst*nst;
	}
      //printf("obs: %d, weight: %f, weight2: %f, nst: %f, sum: %f\n",
      //     prevobs, weight[i], weight2[i], nst, sum);
      for(j=1;j<=l;j++)
	nst+=bb[j] * T[j][1] * pp[1];
      nst+=mixture(obs[i]);
    }
  return -sum;
}

// ************************************
/* computing relative area difference */
// ************************************
double relareadiff( double *pp, double *bb)
{
  int prevobs,i,j;
  double sum,nst;
  
  sum=0;
  prevobs=1;
  makeT();
  
  for(i=1;i<=ol;i++)
    {
      for(j=prevobs;j<=obs[i]-1;j++)
	step1T(pp);
      
      prevobs=obs[i];
      
      nst=0;
      for(j=1;j<=l;j++)
	nst+=bb[j] * T[j][1] * pp[1];
      nst+=mixture(obs[i]);
      if( obs[i]>=tail ) {sum += tailw*fabs(weight[i]-nst)/weight[i];}
      else       {sum += fabs(weight[i]-nst)/weight[i];}
      printf("obs: %d, weight: %f, nst: %f, sum: %f\n", prevobs, weight[i], nst, sum);
    }
  return -sum;
}


// ***********************
// Compute Likelyhood 
// ***********************
double likely(double *pp, double *bb)
{
  int prevobs,i,j;
  double sum,nst;
  
  sum=0;
  prevobs=1;
  makeT();
  
  for(i=1;i<=ol;i++)
    {
      for(j=prevobs;j<=obs[i]-1;j++)
	step1T(pp);
      
      prevobs=obs[i];
      
      nst=0;
      for(j=1;j<=l;j++)
	nst+=bb[j] * T[j][1] * pp[1];
      nst+=mixture(obs[i]);
      //printf("i: %d, log(nst): %f\n", i, log(nst));
      if(nst!=0)
	sum += weight[i]*log(nst);
    }
  return sum;
}

// **********************
// Random Starting values
// **********************
void stvalues(double mean)
{
  int i, j, k, max, more=1,upper_limit=l;
  double sum, bestlik=-1e+20,actlik,geomsum=0;;
  
  //printf(" Looking for good starting values...\n");

  // sum of initial pr.s of geom. mixture
  for(i=0;i<lh;i++) 
    geomsum+=h[i];

  for(k=1;k<=1000;k++)
    {
      for(i=1;i<=l;i++)
	p[i]=1;
      for(i=1;i<=l;i++)
	beta[i]=(double)rand()/(double)RAND_MAX;
      sum=0;
      for(i=1;i<=l;i++)
	sum+=beta[i];
      for(i=1;i<=l;i++)
	beta[i] = beta[i]/sum*(1-geomsum);
  
      if(!exact_repr)
	{
	  more=1;
	  while(more)
	    {
	      for(i=1;i<=l;i++)
		actp[i]=(double)(rand()+1)/(double)RAND_MAX;
	      sum=0;
	      //printp(actp,beta);
	      for(i=1;i<=l;i++)
		sum+=1/actp[i];
	      for(i=1;i<=l;i++)
		actp[i] *= sum/mean;
	      more=0;
	      for(i=1;i<=l;i++) if(actp[i]>1) more = 1;
	    }
	  //printp(actp,beta);
	  for(i=1;i<=l;i++)
	    {
	      max=1;
	      for(j=1;j<=l;j++) if(actp[j]>actp[max]) max=j;
	      p[i]=actp[max];
	      actp[max]=0;
	    }
	}
	
      //printp(p,beta);
      if( method==1 || method==2 ) actlik=likely(p,beta);
      if( method==3 ) actlik=areadiff(p,beta);
      if( method==4 ) actlik=relareadiff(p,beta);
      if( method==5 ) actlik=cdfareadiff(p,beta);
      //printf("actlik: %1.4e\n",actlik);

      if(bestlik<actlik)
	{
	  for(i=1;i<=l;i++)
	    {
	      beststp[i]=p[i];
	      beststbeta[i]=beta[i];
	    }
	  bestlik=actlik;
	}
    }
  for(i=1;i<=l;i++)
    {
      p[i]=beststp[i];
      beta[i]=beststbeta[i];
    }

#ifdef DEBUG
  printf(" Found good starting values...\n");
#endif 

}

// *************************
/* checking along the line */
// *************************
int along(void)
{
  int i,j,k,more=1;
  double bestlik, lik, ratio=1;
  
  if( method==1 || method==2 ) bestlik=likely(p,beta);
  if( method==3 ) bestlik=areadiff(p,beta);
  if( method==4 ) bestlik=relareadiff(p,beta);
  if( method==5 ) bestlik=cdfareadiff(p,beta);
  //printf("lik along elejen: %1.8e\n",bestlik);

  for(i=1;i<=l;i++)
    {
      actp[i]=p[i];
      actbeta[i]=beta[i];
    }
  
  while(more && ratio<=1e15)
    {
      ratio*=10;
      for(k=1;k<=10;k++)
	{
	  for(j=1;j<=l;j++)
	    {
	      actp[j]+=dp[j]/ratio;
	      actbeta[j]+=dbeta[j]/ratio;
	    }
	  if( method==1 || method==2 ) lik=likely(actp,actbeta);
	  if( method==3 ) lik=areadiff(actp,actbeta);	
	  if( method==4 ) lik=relareadiff(actp,actbeta);
	  if( method==5 ) lik=cdfareadiff(actp,actbeta);
	  //printf("along_%f k: %d, lik: %1.8e\n", ratio, k, lik);
	  if(lik>bestlik) 
	    { 
	      bestlik=lik; 
	      more=0;
	    }
	  else 
	    { 
	      k=11;
	      for(j=1;j<=l;j++)
		{
		  actp[j]-=dp[j]/ratio;
		  actbeta[j]-=dbeta[j]/ratio;
		}
	    }
      	}
      if(more)
	{
	  for(j=1;j<=l;j++)
	    {
	      actp[j]=p[j];
	      actbeta[j]=beta[j];
	    }
	}
    }


  //  for(k=1;k<=9;k++)
  //{
  //  for(j=1;j<=l;j++)
  //{
  //  actp[j]+=dp[j]/10;
  //  actbeta[j]+=dbeta[j]/10;
  //}
  //  if( method==1 || method==2 ) lik=likely(actp,actbeta);
  //  if( method==3 ) lik=areadiff(actp,actbeta);	
  //  if( method==4 ) lik=relareadiff(actp,actbeta);
  //  if( method==5 ) lik=cdfareadiff(actp,actbeta);
  //  printf("along_1 k: %d, lik: %1.8e\n", k, lik);
  //  if(lik>bestlik) 
  //{ 
  //  bestlik=lik; 
  //}
  //  else 
  //{ 
  //  k=11;
  //  for(j=1;j<=l;j++)
  //    {
  //      actp[j]-=dp[j]/10;
  //      actbeta[j]-=dbeta[j]/10;
  //    }
  //}
  //  
  //}
  //for(k=1;k<=9;k++)
  //{
  //  for(j=1;j<=l;j++)
  //{
  //  actp[j]+=dp[j]/100;
  //  actbeta[j]+=dbeta[j]/100;
  //}
  //  if( method==1 || method==2 ) lik=likely(actp,actbeta);
  //  if( method==3 ) lik=areadiff(actp,actbeta);
  //  if( method==4 ) lik=relareadiff(actp,actbeta);
  //  if( method==5 ) lik=cdfareadiff(actp,actbeta);
  //  printf("along_2 k: %d, lik: %1.8e\n", k, lik);
  //  if(lik>bestlik) { bestlik=lik; }
  //  else 
  //{ 
  //  k=11;
  //  for(j=1;j<=l;j++)
  //    {
  //      actp[j]-=dp[j]/100;
  //      actbeta[j]-=dbeta[j]/100;
  //    }
  //}
  //}
  //for(k=1;k<=10;k++)
  //{
  //  for(j=1;j<=l;j++)
  //{
  //  actp[j]+=dp[j]/1000;
  //  actbeta[j]+=dbeta[j]/1000;
  //}
  //  if( method==1 || method==2 ) lik=likely(actp,actbeta);
  //  if( method==3 ) lik=areadiff(actp,actbeta);
  //  if( method==4 ) lik=relareadiff(actp,actbeta);
  //  if( method==5 ) lik=cdfareadiff(actp,actbeta);
  //  printf("along_3 k: %d, lik: %1.8e\n", k, lik);
  //  if(lik>bestlik) 
  //{ 
  //  bestlik=lik; 
  //}
  //  else 
  //{ 
  //  k=11;
  //  for(j=1;j<=l;j++)
  //    {
  //      actp[j]-=dp[j]/1000;
  //      actbeta[j]-=dbeta[j]/1000;
  //    }
  //}
  //}
  
  more=0;
  for(k=1;k<=l;k++)
    {
      if(beta[k]!=actbeta[k] || p[k]!=actp[k]) 
	{
	  more=1;
	  k=l+1;
	}
      /*printp(actp,actbeta);*/
      /*printp(p,beta);*/
    }
  for(i=1;i<=l;i++)
    {
      if( actp[i]<0.0 ) {p[i]=0.0;}
      else {p[i]=actp[i];}
      if( actbeta[i]<0.0 ) {beta[i]=0.0;}
      else {beta[i]=actbeta[i];}
    }
  
  /*printf("best: %f\n", bestlik);*/
  return more;
}

/* derivatives with respect to p and beta*/
/* according to the likelihood function, analitically */
void derive1(void)
{
  int i,j,k,prevobs,m;
  double *fn, *dersp, *dersb;
  double fnsum, d, nst;
  
  fn=(double *)malloc(sizeof(double)*(l+1));
  dersp=(double *)malloc(sizeof(double)*(l+1));
  dersb=(double *)malloc(sizeof(double)*(l+1));
  
  makeserT();
  makeT();
  
  prevobs=1;
  for(i=1;i<=l;i++)
    { derp[i]=0; derb[i]=0; }
  for(i=1;i<=ol;i++)
    {
      /*printf("prevobs: %d, obs[%d][2]: %d\n", prevobs, i, obs[i]);*/
      for(j=prevobs;j<=obs[i]-1;j++)
	{
	  step1T(p);
	  step1serT();
	}
      
      prevobs=obs[i];
      for(j=1;j<=l;j++)
	fn[j]=T[j][1]*p[1];
      for(j=1;j<=l;j++)
	{
	  fnsum=0;
	  d=0;
	  
	  for(k=j;k<=l;k++)	
	    {		
	      fnsum += beta[k]*ms[j].m[k+1][1]*p[j];
	      d += beta[k]*fn[k];
	    }
	  /*printf("fnsum: %f, d: %f\n", fnsum, d);*/
	  
	  dersp[j] = 1/p[j]*(d-fnsum);
	  dersb[j] = T[j][1]*p[1];	
	  /*printf("dersp[%d]: %f, dersb[%d]: %f\n", j, dersp[j], j, dersb[j]);*/
	  
	}	
      nst=0;
      for(j=1;j<=l;j++)
	{	
	  /*printf("%d ",j);*/
	  nst+=beta[j] * T[j][1] * p[1];
	}
      /*printf("nst: %f\n", nst);*/
      
      for(j=1;j<=l;j++)
	{
	  /*printf("%d %d \n", i,j);*/
	  derp[j] += weight[i]*dersp[j]/nst;
	  derb[j] += weight[i]*dersb[j]/nst;
	  /*printf("derp[%d]: %f, derb[%d]: %f\n", j, derp[j], j, derb[j]);*/
	}	
    }
  
  /*printf("Derivatives: \n");*/
  /*printp(derp, derb);*/
  
  free(dersp);
  free(dersb);
  free(fn);
}

/* derivatives with respect to p and beta*/
/* according to the likelihood function, numerically */
void derive2(void)
{
  int i,j,up,m;
  double a;
  double diff, per=0.001;
  
  a=likely(p,beta);
  
  for(i=1;i<=l;i++)
    { 
      /* derivative with respect to p[i] */
      if( p[i]<=0.99 && p[i]>=0.01 )
	{
	  diff=p[i]*per;
	  up=1;
	}
      else  
	{
	  if( p[i]>0.99 )
	    {
	      diff=p[i]*per;
	      up=-1;
	    }
	  if( p[i]<0.01 )
	    {
	      diff=0.001;
	      up=1;
	    }	
	}
      p[i]+=up*diff;
      derp[i]=(likely(p,beta)-a)/diff*up;
      p[i]-=up*diff;
    }
  
  for(i=1;i<=l;i++)	
    {
      /* derivative with respect to beta[i] */
      if( beta[i]<=0.99 && beta[i]>=0.01 )
	{
	  diff=beta[i]*per;
	  up=1;
	}
      else    
	{
	  if( beta[i]>0.99 )
	    {
	      diff=beta[i]*per;
	      up=-1;
	    }
	  if( beta[i]<0.01 )
	    {
	      diff=0.001;
	      up=1;
	    }	
	}
      beta[i]+=up*diff;
      //printp(p,beta);
      //printf("a: %f, likely(p,beta): %f\n", a, likely(p,beta));
      derb[i]=(likely(p,beta)-a)/diff*up;
      beta[i]-=up*diff;
    }

  //printf("Derivatives: \n");
  //printp(derp, derb);
}	

/* derivatives with respect to p and beta*/
/* according to the areadifference, numerically */
void derive3(void)
{
  int i,j,up,m;
  double a;
  double diff, per=0.001;
  
  a=areadiff(p,beta);
  
  for(i=1;i<=l;i++)
    { 
      /* derivative with respect to p[i] */
      if( p[i]<=0.99 && p[i]>=0.01 )
	{
	  diff=p[i]*per;
	  up=1;
	}
      else
	{
	  if( p[i]>0.99 )
	    {
	      diff=p[i]*per;
	      up=-1;
	    }
	  if( p[i]<0.01 )
	    {
	      diff=0.001;
	      up=1;
	    }	
	}
	p[i]+=up*diff;
	derp[i]=(areadiff(p,beta)-a)/diff*up;
	p[i]-=up*diff;
    }
  
  for(i=1;i<=l;i++)
    {
      /* derivative with respect to beta[i] */
      if( beta[i]<=0.99 && beta[i]>=0.01 )
	{
	  diff=beta[i]*per;
	  up=1;
	}
      else
	{
	  if( beta[i]>0.99 )
	    {
	      diff=beta[i]*per;
	      up=-1;
	    }
	  if( beta[i]<0.01 )
	    {
	      diff=0.001;
	      up=1;
	    }	
	}
      beta[i]+=up*diff;
      derb[i]=(areadiff(p,beta)-a)/diff*up;
      beta[i]-=up*diff;
    }
  /*printf("Derivatives: \n");*/
  /*printp(derp, derb);*/
}		

/* derivatives with respect to p and beta*/
/* according to the relative areadifference, numerically */
void derive4(void)
{
  int i,j,up,m;
  double a;
  double diff, per=0.001;
  
  a=relareadiff(p,beta);
  
  for(i=1;i<=l;i++)
    { 
      /* derivative with respect to p[i] */
      if( p[i]<=0.99 && p[i]>=0.01 )
	{
	  diff=p[i]*per;
	  up=1;
	}
      else
	{
	  if( p[i]>0.99 )
	    {
	      diff=p[i]*per;
	      up=-1;
	    }
	  if( p[i]<0.01 )
	    {
	      diff=0.001;
	      up=1;
	    }	
	}
      p[i]+=up*diff;
      derp[i]=(relareadiff(p,beta)-a)/diff*up;
      p[i]-=up*diff;
    }
	
  for(i=1;i<=l;i++)
    {	
      /* derivative with respect to beta[i] */
      if( beta[i]<=0.99 && beta[i]>=0.01 )
	{
	  diff=beta[i]*per;
	  up=1;
	}
      else
	{
	  if( beta[i]>0.99 )
	    {
	      diff=beta[i]*per;
	      up=-1;
	    }
	  if( beta[i]<0.01 )
	    {
	      diff=0.001;
	      up=1;
	    }	
	}
      beta[i]+=up*diff;
      derb[i]=(relareadiff(p,beta)-a)/diff*up;
      beta[i]-=up*diff;
    }
  /*printf("Derivatives: \n");*/
  /*printp(derp, derb);*/
}

/* derivatives with respect to p and beta*/
/* according to the cdfareadiff, numerically */
void derive5(void)
{
  int i,j,up,m;
  double a;
  double diff, per=0.000001;
  
  a=cdfareadiff(p,beta);
  
  for(i=1;i<=l;i++)
    { 
      /* derivative with respect to p[i] */
      if( p[i]<=0.99 && p[i]>=0.01 )
	{
	  diff=p[i]*per;
	  up=1;
	}
      else
	{
	  if( p[i]>0.99 )
	    {
	      diff=p[i]*per;
	      up=-1;
	    }
	  if( p[i]<0.01 )
	    {
	      diff=0.001;
	      up=1;
	    }	
	}
      p[i]+=up*diff;
      derp[i]=(cdfareadiff(p,beta)-a)/diff*up;
      p[i]-=up*diff;
    }
	
  for(i=1;i<=l;i++)
    {	
      /* derivative with respect to beta[i] */
      if( beta[i]<=0.99 && beta[i]>=0.01 )
	{
	  diff=beta[i]*per;
	  up=1;
	}
      else
	{
	  if( beta[i]>0.99 )
	    {
	      diff=beta[i]*per;
	      up=-1;
	    }
	  if( beta[i]<0.01 )
	    {
	      diff=0.001;
	      up=1;
	    }	
	}
      beta[i]+=up*diff;
      derb[i]=(cdfareadiff(p,beta)-a)/diff*up;
      beta[i]-=up*diff;
    }
  //printf("Derivatives: \n");
  //printp(derp, derb);
}

// ************
// Memory free
// ************
void memfree(void)
{
  int i,j,k;
  
  for(i=1;i<=l;i++)
    for(j=1;j<=l+1;j++)
      free(ms[i].m[j]);
  for(i=1; i<=l; i++)	
    {
      free(T[i]);
      free(ms[i].m);
    }
  free(ms);
  free(T);
  
  free(p);   free(actp);
  free(beta);   free(actbeta);
  free(dp);  free(dbeta);
  free(derp); free(derb);
  free(obs);
  free(weight);
  free(weight2);
  free(dist);
  free(mass);
  free(mass2);
  free(bestp);
  free(bestbeta);
  free(beststp);
  free(beststbeta);

  for(i=1; i<=3*l; i++)
    free(a[i]);
  free(a);
  free(iposv);
  free(izrov);
  if(lh>0)
    {
      free(h);	
      free(lambda);
    }
}


// ***************
// generating list
// ***************
void genlist(double lower, double upper, double stepsize, int additive)
{
  int i,j,onlytail=0;
  FILE *f;
  double nst,accdf=1.0,actdouble,t,tailconstant,sptailccdf,sptailpdf,cf1pdf,tmp;
  int actint,first=1,exactint;
  
  if(sptailtype!=0)
    {
      if(sptailtype==1)
	tailconstant=ccdf(sptailstart,param1,param2)/
	  pow(sptailstart,-sptailparam);
      else
	tailconstant=ccdf(sptailstart,param1,param2)/
	  exp(-pow(sptailstart,sptailparam));
    }

  f=fopen("TEMPORARYPhFitFILE.data","w");
  
  printf("fitting delta:%1.4e\n",fitting_delta);
  if(lower==0.0)
    fprintf(f, "%e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e\n", 0.0, 0.0,
	    pdf(0.0001,param1,param2), 
	    0.0,
	    1.0,
	    pdf(0.0001,param1,param2),
	    0.0, 
	    0.0,
	    1.0,
	    0.0,
	    1.0,
	    0.0,
	    0.0,
	    1.0,
	    0.0, 
	    0.0);
  
  makeT();
  
  if(lower!=0.0)
    {
      actdouble=lower;
      actint=(int)floor(actdouble/fitting_delta+0.5);
    }
  else
    {
      actdouble=stepsize;
      actint=(int)floor(actdouble/fitting_delta+0.5);
    }
  exactint=0;
  while(actdouble<upper)
    { 
      printf("actdouble: %1.4e\n",actdouble);
      printf("actint: %d\n",actint);
      
      if(additive)
	{
	  if(onlytail)
	    {
	      nst=mixture(actint);
	      accdf=mixtureccdf(actint);
	    }
	  else
	    {
	      for(i=exactint;i<actint;i++) 
		{
		  nst=0.0;
		  for(j=1;j<=l;j++)
		    nst+=bestbeta[j] * T[j][1] * bestp[1];
		  step1T(bestp);
		  tmp=mixture(i+1);
		  if(tmp/nst>1e20) onlytail=1;
		  nst+=tmp;
		  accdf-=nst;
		}
	    }
	}
      else
	{
	  if(onlytail)
	    {
	      nst=mixture(actint);
	      accdf=mixtureccdf(actint);
	    }
	  else
	    {
	      for(i=exactint;i<actint;i++) 
		{
		  nst=0;
		  for(j=1;j<=l;j++)
		    nst+=bestbeta[j] * T[j][1] * bestp[1];
		  step1T(bestp);
		  tmp=mixture(i+1);
		  if(tmp/nst>1e20) onlytail=1;
		  nst+=tmp;
		  accdf-=nst;
		}
	    }
	}
      
      t=(double)actint*fitting_delta;
      if(actint==0)
	fprintf(f, "%e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e  %e\n", 
		t, actdouble,
		pdf(actdouble,param1,param2),
		cdf(actdouble,param1,param2),
		ccdf(actdouble,param1,param2),
		pdf(actdouble,param1,param2)/(ccdf(actdouble,param1,param2)),
		0.0,
		0.0,
		1.0,
		0.0,
		1.0,
		fabs(1.0-accdf-0.0)/cdf(actdouble,param1,param2),
		fabs(1.0-accdf-1.0)/(ccdf(actdouble,param1,param2)),
		fabs(pdf(actdouble,param1,param2)/(ccdf(actdouble,param1,param2))-0.0)/(pdf(actdouble,param1,param2)/(ccdf(actdouble,param1,param2))),
		0.0,
		0.0);
      else
	{
	  fprintf(f, "%e  %e  ", t, actdouble);
	  tmp=pdf(actdouble,param1,param2);
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=cdf(actdouble,param1,param2);
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=ccdf(actdouble,param1,param2);
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=pdf(actdouble,param1,param2)/(ccdf(actdouble,param1,param2));
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=nst;
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=1-accdf;
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=accdf;
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=nst/(accdf);
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=fabs(pdf(t,param1,param2)-nst/fitting_delta)/pdf(t,param1,param2);
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=fabs(1.0-accdf-cdf(actdouble,param1,param2))/cdf(actdouble,param1,param2);
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=fabs(accdf-ccdf(actdouble,param1,param2))/(ccdf(actdouble,param1,param2));
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=fabs(pdf(t,param1,param2)/(ccdf(t,param1,param2))-nst/fitting_delta/(accdf))/(pdf(t,param1,param2)/(ccdf(t,param1,param2)));
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=nst-mixture(actint);
	  if(onlytail) tmp=0.0;
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  tmp=mixture(actint);
	  if(!finite(tmp)) tmp=0.0;
	  fprintf(f, "%e  ", tmp); 
	  if(actdouble>sptailstart&&sptailtype!=0)
	    {
	      if(sptailtype==1)
		{
		  sptailpdf=tailconstant*sptailparam*pow(t,-sptailparam-1.0);
		  sptailccdf=tailconstant*pow(t,-sptailparam);
		}
	      else
		{
		  sptailpdf=tailconstant*exp(-pow(t,sptailparam))*sptailparam*pow(t,sptailparam-1);
		  sptailccdf=tailconstant*exp(-pow(t,sptailparam));
		}
	      tmp=sptailpdf;
	      if(!finite(tmp)) tmp=0.0;
	      fprintf(f, "%e  ", tmp); 
	      tmp=1-sptailccdf;
	      if(!finite(tmp)) tmp=0.0;
	      fprintf(f, "%e  ", tmp); 
	      tmp=sptailccdf;
	      if(!finite(tmp)) tmp=0.0;
	      fprintf(f, "%e  ", tmp); 
	      tmp=sptailpdf/sptailccdf;
	      if(!finite(tmp)) tmp=0.0;
	      fprintf(f, "%e  ", tmp); 
	      tmp=fabs(sptailpdf-nst/fitting_delta)/sptailpdf;
	      if(!finite(tmp)) tmp=0.0;
	      fprintf(f, "%e  ", tmp); 
	      tmp=fabs(1-accdf-(1-sptailccdf))/(1-sptailccdf);
	      if(!finite(tmp)) tmp=0.0;
	      fprintf(f, "%e  ", tmp); 
	      tmp=fabs(accdf-sptailccdf)/sptailccdf;
	      if(!finite(tmp)) tmp=0.0;
	      fprintf(f, "%e  ", tmp); 
	      tmp=fabs((nst/fitting_delta)/accdf-sptailpdf/sptailccdf)/(sptailpdf/sptailccdf);
	      if(!finite(tmp)) tmp=0.0;
	      fprintf(f, "%e", tmp); 
	    }
	  fprintf(f,"\n");
	}
      exactint=actint;
      if(additive)
	{
	  actdouble+=stepsize;
	  actint=(int)floor(actdouble/fitting_delta+0.5);
	}
      else
	{
	  actdouble*=stepsize;
	  actint=(int)floor(actdouble/fitting_delta+0.5);
	}
    }
  fclose(f);
}

// ************
// Write Matrix
// ************
void writematrix(char *fn)
{
  int i;
  FILE *f;
  double sum;
  
  f=fopen(fn,"w");
  
  // This part writes the result in the formused by WebSPN
  //fprintf(f, "%d\n", l+1);
  //fprintf(f, "%d ", l+1);
  //fprintf(f, "%d %1.6e ", 1, 1-bestp[1]);
  //for(i=1;i<=l;i++) fprintf(f, "%d %1.6e ", i+1, bestbeta[l-i+1]*bestp[1]);
  //fprintf(f, "\n");
  
  //for(i=2;i<=l;i++)
  //  fprintf(f,"%d %d %1.6e %d %1.6e\n", 2, i, 1-bestp[l-i+2], i+1, bestp[l-i+2]);
  // ===============================================================================

  fprintf(f,"p=[");
  for(i=1;i<=l;i++) fprintf(f, "%1.6e ", bestp[i]);
  fprintf(f,"]\n");
  fprintf(f,"beta=[");
  for(i=1;i<=l;i++) fprintf(f, "%1.6e ", bestbeta[i]);
  fprintf(f,"]\n");

fclose(f);
}

void Result2File()
{
  FILE *f;
  int i1,i2;

  f=fopen("TEMPORARYPhFitFILE.result","w");
  fprintf(f,"%d\n", l);
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.8e", bestbeta[i1]);
  fprintf(f,"\n");
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.8e", bestp[i1]);
  fprintf(f,"\n");
  fprintf(f,"%d\n", lh);
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.8e", h[i1]);
  fprintf(f,"\n");
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.8e", lambda[i1]);
  fprintf(f,"\n");
  fclose(f);

  f=fopen("TEMPORARYPhFitFILE.initials","w");
  fprintf(f, "CF-1 part:\n");
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.4e\n", bestbeta[i1]);
  fprintf(f,"\n");
  if(lh>0)  fprintf(f, "hyper. part:\n");
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.4e\n", h[i1]);
  fprintf(f,"\n");
  fclose(f);

  f=fopen("TEMPORARYPhFitFILE.intensities","w");
  fprintf(f, "CF-1 part:\n");
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.4e\n", bestp[i1]);
  fprintf(f,"\n");
  if(lh>0)  fprintf(f, "hyper. part:\n");
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.4e\n", lambda[i1]);
  fprintf(f,"\n");
  fclose(f);

  f=fopen("TEMPORARYPhFitFILE.textual","w");
  fprintf(f, "Initial probabilities of CF-1 part:\n");
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.8e\n", bestbeta[i1]);
  fprintf(f,"\n");
  if(lh>0)  fprintf(f, "Initial probabilities of hypergeometric part:\n");
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.8e\n", h[i1]);
  fprintf(f,"\n");
  fprintf(f, "Intensities of CF-1 part:\n");
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.8e\n", bestp[i1]);
  fprintf(f,"\n");
  if(lh>0)  fprintf(f, "Intensities of hypergeometric part:\n");
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.8e\n", lambda[i1]);
  fprintf(f,"\n");
  fclose(f);

  f=fopen("TEMPORARYPhFitFILE.matrix","w");
  for(i1=1;i1<=l;i1++) 
    fprintf(f," %1.8e ", bestbeta[i1]);
  for(i1=0;i1<lh;i1++) 
    fprintf(f," %1.8e ", h[i1]);
  fprintf(f,"\n");
  fprintf(f,"\n");
  for(i1=1;i1<=l;i1++)
    {
      for(i2=1;i2<=l+lh;i2++) 
	{
	  if(i1==i2) fprintf(f," %1.8e ", 1-bestp[i1]);
	  if(i1-1==i2) fprintf(f," %1.8e ", bestp[i1]);
	  if(i1-1!=i2 && i1!=i2) fprintf(f," %1.8e ", 0.0);
	}
      fprintf(f,"\n");
    }
  for(i1=0;i1<lh;i1++) 
    {
      for(i2=0;i2<l+lh;i2++) 
	{
	  if(i1+l==i2) fprintf(f," %1.8e ", 1-lambda[i1]);
	  if(i1+l!=i2) fprintf(f," %1.8e ", 0.0);
	}
      fprintf(f,"\n");
    }
  fprintf(f,"\n");
  fprintf(f,"\n");
  fprintf(f," %1.8e\n",bestp[1]);
  for(i1=0;i1<l-1;i1++) fprintf(f," %1.8e\n", 0.0);
  for(i1=0;i1<lh;i1++) fprintf(f," %1.8e\n", lambda[i1]);
  fclose(f);
}

void ResultFromFile()
{
  FILE *f;
  int i1;
  
  f=fopen("TEMPORARYPhFitFILE.result","r");
  fscanf(f,"%d\n", &l);
  bestp = (double *)malloc( (l+1)*sizeof(double));
  bestbeta = (double *)malloc( (l+1)*sizeof(double));
  for(i1=1;i1<=l;i1++) 
    fscanf(f," %le", &bestbeta[i1]);
  fscanf(f,"\n");
  for(i1=1;i1<=l;i1++) 
    fscanf(f," %le", &bestp[i1]);
  fscanf(f,"\n");
  fscanf(f,"%d\n", &lh);
  if(lh>0)
    {
      lambda = (double *)malloc( (lh)*sizeof(double));
      h = (double *)malloc( (lh)*sizeof(double));
    }
  for(i1=0;i1<lh;i1++) 
    fscanf(f," %le", &h[i1]);
  fscanf(f,"\n");
  for(i1=0;i1<lh;i1++) 
    fscanf(f," %le", &lambda[i1]);
  fscanf(f,"\n");
  fclose(f);
}

// checking hyperexp part
int goodgeom()
{
  int i1;
  double sum=0.0;
  
  for(i1=0;i1<lh;i1++)
    if( !(h[i1]>0.0 && h[i1]<1.0) ) return 0;
  for(i1=0;i1<lh;i1++)
    if(! (lambda[i1]>0.0) ) return 0;
  for(i1=0;i1<lh;i1++)
    sum+=h[i1];
  if(sum>1.0) return 0;
  return 1;
}


// fitting the tail and checking the results + messages
void buildandcheck()
{
  int i;

  if(lh>0) 
    {
      printf("Building Hyperexponential Part...\n");
      fflush(stdout);
      build_geom();
      if(!goodgeom())
	{
	  printf("  Failed to build hyperexponential part.\n");
	  printf("  Resulting parameters are inappropriate:\n");
	  for(i=0;i<lh;i++)
	    {
	      printf("   Init. prob.s \tIntensity\n");
	      for(i=0;i<lh;i++) 
		printf("   %1.4e \t%1.4e\n",h[i],lambda[i]); 
	    }
	  printf("  Tailfitting parameters should be changed.\n");
	  exit(1);
	}
      printf("Hypergeometric Part Built\n\n");
      fflush(stdout);
    }
}


// ****
// MAIN
// ****
int main(int argc, char* argv[])
{
  int i,j,k, more,changed,emptyr,nr=1000;
  double bestlike, lik;
  struct measures *m;

  // Input parameters:
  // 1: Function: "1": Fitting a distribution given by pdf
  //              "2": Producing data file to plot curves 
  //              "3": Fitting a distribution given by samples
  //              "4": Producing data file to plot curves

  // If 1.Param is "1" (Fitting a distribution given by pdf), then
  // 2: Distribution: "1": Weibull
  //                  "2": Lognormal
  //                  "3": Uniform
  //                  "4": Pareto-like I
  //                  "5": Pareto-like II
  // 3: 1st Parameter of the Distribution
  // 4: 2nd Parameter of the Distribution
  // 5: Number of Phases to Fit the Main Part
  // 6: Limit of Fitting the Main Part
  // 7: Distance Measure: "1": Entropy
  //                      "2": pdf Area Difference
  //                      "***": Relative pdf Area Difference
  //                      "3": cdf Area Difference
  // 8: Number of Phases to Fit the Tail Part
  // 9: Lower Limit of Tail fitting
  // 10: Upper Limit of Tail Fitting
  // 11: Distance of Adjacent Points
  // 12: Stepsize or Scalefactor
  // 13: Special Tail: "0" No special tail
  //                   "1" Polynomial tail
  //                   "2" Weibull tail
  // 14: Parameter of the tail
  // 15: Start of the tail (positive->itself, negative->quantile)

  // If 1.Param is "2" (Producing data file to plot curves), then
  // 2: Distribution: "1": Weibull
  //                  "2": Lognormal
  //                  "3": Uniform
  //                  "4": Pareto-like I
  //                  "5": Pareto-like II
  // 3: 1st Parameter of the Distribution
  // 4: 2nd Parameter of the Distribution
  // 5: Lower Limit of List
  // 6: Upper Limit of List
  // 7: Stepsize of List (positive -> additive,negative->multiplicative)
  // 8: Special Tail: "0" No special tail
  //                   "1" Polynomial tail
  //                   "2" Weibull tail
  // 9: Parameter of the tail
  // 10: Start of the tail (positive->itself, negative->quantile)
  // 11: Special Tail: "0" No special tail
  //                   "1" Polynomial tail
  //                   "2" Weibull tail
  // 12: Parameter of the tail
  // 13: Start of the tail (positive->itself, negative->quantile)


  // If 1.Param is "3" (Fitting a distribution given by samples), then
  // 2: name of file describing the distribution
  // 3: type of file: 1: empirical cdf
  //                  2: empirical ccdf
  //                  3: the samples itself
  //                  4: samples with weights
  // 4: Number of Phases to Fit the Main Part
  // 5: Limit of Fitting the Main Part
  // 6: Distance Measure: "1": Entropy
  //                      "2": pdf Area Difference
  //                      "***": Relative pdf Area Difference
  //                      "3": cdf Area Difference
  // 7: Number of Phases to Fit the Tail Part
  // 8: Lower Limit of Tail fitting
  // 9: Upper Limit of Tail Fitting
  // 10: Distance of Adjacent Points
  // 11: Stepsize or Scalefactor
  // 12: Special Tail: "0" No special tail
  //                   "1" Polynomial tail
  //                   "2" Weibull tail
  // 13: Parameter of the tail
  // 14: Start of the tail (positive->itself, negative->quantile)
  
  // If 1.Param is "4" (Producing data file to plot curves), then
  // 2: name of file describing the distribution
  // 3: type of file: 1: empirical cdf
  //                  2: empirical ccdf
  //                  3: the samples itself
  //                  4: samples with weights
  // 4: 1st Parameter of the Distribution
  // 5: 2nd Parameter of the Distribution
  // 6: Lower Limit of List
  // 7: Upper Limit of List
  // 8: Stepsize of List (positive -> additive,negative->multiplicative)
  // 9: Special Tail: "0" No special tail
  //                   "1" Polynomial tail
  //                   "2" Weibull tail
  // 10: Parameter of the tail
  // 11: Start of the tail (positive->itself, negative->quantile)


  // Ez itt a regi parameterezes
  /*    if(argc != 16) */
  /*      { */
  /*        printf("1. param.: filename of observations\n"); */
  /*        printf("2. param.: filename of distribution\n"); */
  /*        printf("3. param.: num. of phases\n"); */
  /*        printf("4. param.: filename of matrix describing the chain\n"); */
  /*        printf("5. param.: filename of probabilities given by the chain\n"); */
  /*        printf("6. param.: last time point in the list of probabilities\n"); */
  /*        printf("7. param.: optimization option (1: analyitic LikelyHood,\n"); */
  /*        printf("           2: numerical LikelyHood, 3: numerical areadiff.\n"); */
  /*        printf("           4: numerical relative areadifference,\n"); */
  /*        printf("           5: areadiff. between cdf's,\n"); */
  /*        printf("8. param.: start of tail\n"); */
  /*        printf("9. param.: weight of tail\n"); */
  /*        printf("10. param.: stepsize of DPH\n"); */
  /*        // parameters for heavy-tails */
  /*        printf("11. param.: number of tail-phases\n"); */
  /*        printf("12. param.: Start of tail fitting (handled as 5. param\n"); */
  /*        printf("13. param.: Upper limit of tail fitting (handled as 5. param\n"); */
  /*        printf("14. param.: To define the distance of adjacent points: x\n"); */
  /*        printf("             b=m^x, x in [0:1]\n"); */
  /*        printf("15. param.: decay of tail\n"); */
  
  /*        exit(1); */
  /*      } */
  // IDAIG REGI PARAMETEREZES
  
  if(atoi(argv[1])==1)
    {
      samples=0;
      switch(atoi(argv[2]))
	{
	case 1:
	  pdf=&pdf_weib; distmean=&mean_weib; cdf=&cdf_weib;		
	  invcdf=&invcdf_weib; ccdf=&ccdf_weib; invccdf=&invccdf_weib;
	  break;
	case 2:
	  pdf=&pdf_logn; distmean=&mean_logn; cdf=&cdf_logn;		
	  invcdf=&invcdf_logn; ccdf=&ccdf_logn; invccdf=&invccdf_logn;
	  break;
	case 3:
	  pdf=&pdf_uni; distmean=&mean_uni; cdf=&cdf_uni;		
	  invcdf=&invcdf_uni; ccdf=&ccdf_uni; invccdf=&invccdf_uni;
	  break;
	case 4:
	  pdf=&pdf_par1; distmean=&mean_par1; cdf=&cdf_par1;		
	  invcdf=&invcdf_par1; ccdf=&ccdf_par1; invccdf=&invccdf_par1;
	  break;
	case 5:
	  pdf=&pdf_par2; distmean=&mean_par2; cdf=&cdf_par2;		
	  invcdf=&invcdf_par2; ccdf=&ccdf_par2; invccdf=&invccdf_par2;
	  break;
	}
      param1=atof(argv[3]);
      param2=atof(argv[4]);
      l=atoi(argv[5]);
      if(atof(argv[6])>=0) 
	endofmainfitting=atof(argv[6]);
      else 
	{ 
	  endofmainfitting=invccdf(-atof(argv[6]),param1,param2);	
	  printf(" End of fitting the body: %1.4e\n",endofmainfitting);
	}
      method=atoi(argv[7]);
      method++;
      if(method==4) method=5;
      lh=atoi(argv[8]);
      if(atof(argv[9])>=0) 
	mtail1=atof(argv[9]);	
      else 
	{
	  mtail1=invccdf(-atof(argv[9]),param1,param2); 
	  printf(" Lower limit of fitting the tail: %1.4e\n", mtail1);
	}
      if(atof(argv[10])>=0) 
	{
	  mtail2=atof(argv[10]);	
	}
      else 
	{
	  mtail2=invccdf(-atof(argv[10]),param1,param2); 
	  printf(" Upper limit of fitting the tail: %1.4e\n", mtail2);
	}
      adj=atof(argv[11]);		
      fitting_delta=atof(argv[12]);
      tailw=1;
      sptailtype=atoi(argv[13]);
      if(sptailtype!=0)
	{
	  sptailparam=atof(argv[14]);
	  if(atof(argv[15])>0)
	    sptailstart=atof(argv[15]);
	  else
	    {
	      sptailstart=invccdf(-atof(argv[15]),param1,param2);
	      printf(" Start of special tail: %1.4e\n", sptailstart);
	    }
	}
      NUMBEROFROUNDS=atoi(argv[16]);
      NUMBEROFITERATIONS=atoi(argv[17]);

      discretize();
      mean=distmean(param1,param2)/fitting_delta;
      //printf("Mean: %1.4e\n",mean);
      exact_repr=0;
      is_it_exact();
      //printf(" Exact representation: %d, number of phases needed: %d\n",
      //	     exact_repr, nec_phases);
      if(exact_repr) l=nec_phases;

      memalloc();
      
      buildandcheck();
      //printf("Mean with the tail: %f\n", mean);
      for(i=0;i<lh;i++) mean-=h[i]/lambda[i];
      //printf("Mean changed by The Mixture: %f\n",mean);

      
      srand(1);
      
      bestlike=-10000000.0;
      for(i=1;i<=NUMBEROFROUNDS;i++)
	{
	  printf("Round %2d/%2d\n",i,NUMBEROFROUNDS);
	  fflush(stdout);
	  printf("  Looking for good starting values...\n");
	  fflush(stdout);
	  if( mean<1.5*l ) 
	    { 
	      stvalues(1.5*l); 
	    }
	  else 
	    { 
	      stvalues(mean); 
	    }
	  printf("  Starting values are defined\n");
	  fflush(stdout);
	  
	  //printp(p,beta);
	  /*printf(" likeli: %f\n", likely(p,beta));*/
	  /*printf(" areadifference: %f\n", -areadiff(p,beta));*/
	  
	  Delta=0.1;
	  changed=1;
	  emptyr=0;
	  for(j=1;j<=NUMBEROFITERATIONS;j++)
	    {
	      if(j%10==0)
		{
		  printf("  Modification n. %4d/%4d\n",j,NUMBEROFITERATIONS);
		  fflush(stdout);
		}

	      //printf("%d ",j);
	      if( method==1 && changed) derive1();
	      if( method==2 && changed) derive2();
	      if( method==3 && changed) derive3();
	      if( method==4 && changed) derive4();
	      if( method==5 && changed) derive5();
	      //printf("Parameters: \n");
	      //printp(p,beta);
	      //printf("Derivatives: \n");
	      //printp(derp, derb);
	      
	      lp();
	      if(exact_repr) 
		{
		  for(k=1;k<=l;k++)
		    dp[k]=0;
		}
	      // Result of simplex:
	      //printf("result of simplex: \n");
	      //printp(dp, dbeta);
	      
	      changed=1;
	      if(along()==0) 
		{
		  emptyr++;
		  changed=0;
		  Delta=0.999*Delta;
		  if(Delta<0.01) 
		    {
		      //printf("  Number of rounds (empty rounds):
		      //  %d (%d), Delta: %1.4e\n",j,emptyr,Delta);
		      j=nr+2;
		    }
		}
	      /*printp(p,beta);*/
	      /*printf("likeli: %f\n", likely(p,beta));*/
	    }
	  printf("\nEnd of round n. %d:\n\n", i);
	  fflush(stdout);
	  //if(j!=nr+2)
	  //  printf("  Number of rounds (empty rounds): %d (%d), Delta: %1.4e\n",
	  //   nr,emptyr,Delta);
	  if( method==1 || method==2 ) lik=likely(p,beta);
	  if( method==3 ) lik=areadiff(p,beta);
	  if( method==4 ) lik=relareadiff(p,beta);
	  if( method==5 ) lik=cdfareadiff(p,beta);
	  /*printp(p,beta);*/
	  /*printf("likeli: %f\n", lik);*/
	  if( bestlike<lik)
	    {
	      bestlike=lik;
	      for(j=1;j<=l;j++)
		{
		  bestbeta[j]=beta[j];
		  bestp[j]=p[j];
		}
	    }
	}
      
      //printp(bestp,bestbeta);
      Result2File();
      memfree();
      printf("Fitting has finished.\n");
      return 0;
    }  
  // End of Fitting a distribution given by pdf 
  // ==========================================

  // ==================================
  // Producing data file to plot curves
  if(atoi(argv[1])==2)
    {
      samples=0; 
      switch(atoi(argv[2]))
	{
	case 1:
	  pdf=&pdf_weib; distmean=&mean_weib; cdf=&cdf_weib;		
	  invcdf=&invcdf_weib; ccdf=&ccdf_weib; invccdf=&invccdf_weib;
	  break;
	case 2:
	  pdf=&pdf_logn; distmean=&mean_logn; cdf=&cdf_logn;		
	  invcdf=&invcdf_logn; ccdf=&ccdf_logn; invccdf=&invccdf_logn;
	  break;
	case 3:
	  pdf=&pdf_uni; distmean=&mean_uni; cdf=&cdf_uni;		
	  invcdf=&invcdf_uni; ccdf=&ccdf_uni; invccdf=&invccdf_uni;
	  break;
	case 4:
	  pdf=&pdf_par1; distmean=&mean_par1; cdf=&cdf_par1;		
	  invcdf=&invcdf_par1; ccdf=&ccdf_par1; invccdf=&invccdf_par1;
	  break;
	case 5:
	  pdf=&pdf_par2; distmean=&mean_par2; cdf=&cdf_par2;		
	  invcdf=&invcdf_par2; ccdf=&ccdf_par2; invccdf=&invccdf_par2;
	  break;
	}
      param1=atof(argv[3]);
      param2=atof(argv[4]);
      fitting_delta=atof(argv[8]);
      ResultFromFile();
      printp(bestp,bestbeta);
      memalloc2();
      sptailtype=atoi(argv[9]);
      if(sptailtype!=0)
	{
	  sptailparam=atof(argv[10]);
	  if(atof(argv[11])>0)
	    sptailstart=atof(argv[11]);
	  else
	    {
	      sptailstart=invccdf(-atof(argv[11]),param1,param2);
	      printf(" Start of special tail: %1.4e\n", sptailstart);
	    }
	}
      if(atof(argv[7])>0.0)
	genlist(atof(argv[5]),atof(argv[6]),fabs(atof(argv[7])),1);
      else
	genlist(atof(argv[5]),atof(argv[6]),fabs(atof(argv[7])),0);
      memfree();
      return 0;
    }
  // Producing data file to plot curves finished
  // ===========================================
  
  if(atoi(argv[1])==3)
    {
      samples=1;
      switch(atoi(argv[3]))
	{
	case 1:
	  loadcdf(argv[2]);
	  break;
	case 2:
	  loadccdf(argv[2]);
	  break;
	case 3:
	  loadsamples(argv[2]);
	  break;
	case 4:
	  loadcdf(argv[2]);
	  break;
	}
      pdf=&pdf_loaded;
      distmean=&mean_loaded;	
      cdf=&cdf_loaded;
      invcdf=&invcdf_loaded;	
      ccdf=&ccdf_loaded;	
      invccdf=&invccdf_loaded;

      if(atoi(argv[3])==3 || atoi(argv[3])==4)
	createcdf();
      //if(atoi(argv[3])==1 || atoi(argv[3])==2)
      //createsamples();

      l=atoi(argv[4]);
      if(atof(argv[5])>=0) 
	endofmainfitting=atof(argv[5]);
      else 
	{ 
	  endofmainfitting=invccdf(-atof(argv[5]),param1,param2);	
	  printf(" End of fitting the body: %1.4e\n",endofmainfitting);
	  fflush(stdout);
	}
      method=atoi(argv[6]);
      method++;
      if(method==4) method=5;
      lh=atoi(argv[7]);
      if(atof(argv[8])>=0) 
	mtail1=atof(argv[8]);	
      else 
	{
	  mtail1=invccdf(-atof(argv[8]),param1,param2); 
	  printf(" Lower limit of fitting the tail: %1.4e\n", mtail1);
	  fflush(stdout);
	}
      if(atof(argv[9])>=0) 
	{
	  mtail2=atof(argv[9]);	
	}
      else 
	{
	  mtail2=invccdf(-atof(argv[9]),param1,param2); 
	  printf(" Upper limit of fitting the tail: %1.4e\n", mtail2);
	  fflush(stdout);
	}
      adj=atof(argv[10]);		
      fitting_delta=atof(argv[11]);
      tailw=1;
      sptailtype=atoi(argv[12]);
      if(sptailtype!=0)
	{
	  sptailparam=atof(argv[13]);
	  if(atof(argv[14])>0)
	    sptailstart=atof(argv[14]);
	  else
	    {
	      sptailstart=invccdf(-atof(argv[14]),param1,param2);
	      printf(" Start of special tail: %1.4e\n", sptailstart);
	      fflush(stdout);
	    }
	}
      NUMBEROFROUNDS=atoi(argv[15]);
      NUMBEROFITERATIONS=atoi(argv[16]);
      
      discretize();
      mean=distmean(param1,param2)/fitting_delta;
      //printf("Mean: %1.4e\n",mean);
      exact_repr=0;
      is_it_exact();
      //printf(" Exact representation: %d, number of phases needed: %d\n",
      //	     exact_repr, nec_phases);
      if(exact_repr) l=nec_phases;

      memalloc();

      buildandcheck();
      //printf("Mean with the tail: %f\n", mean);
      for(i=0;i<lh;i++) mean-=h[i]/lambda[i];
      //printf("Mean changed by The Mixture: %f\n",mean);

      srand(1);
      
      bestlike=-10000000.0;
      for(i=1;i<=NUMBEROFROUNDS;i++)
	{
	  printf("%d. round\n",i);
	  fflush(stdout);
	  printf("  Looking for good starting values...\n");
	  fflush(stdout);
	  if( mean<1.5*l ) 
	    { 
	      stvalues(1.5*l); 
	    }
	  else 
	    { 
	      stvalues(mean); 
	    }
	  printf("  Starting values are defined\n");
	  fflush(stdout);
	  
	  //printp(p,beta);
	  /*printf(" likeli: %f\n", likely(p,beta));*/
	  /*printf(" areadifference: %f\n", -areadiff(p,beta));*/
	  
	  Delta=0.1;
	  changed=1;
	  emptyr=0;
	  for(j=1;j<=NUMBEROFITERATIONS;j++)
	    {
	      if(j%10==0)
		{
		  printf("  Modification n. %3d\n",j);
		  fflush(stdout);
		}

	      //printf("%d ",j);
	      if( method==1 && changed) derive1();
	      if( method==2 && changed) derive2();
	      if( method==3 && changed) derive3();
	      if( method==4 && changed) derive4();
	      if( method==5 && changed) derive5();
	      //printf("Parameters: \n");
	      //printp(p,beta);
	      //printf("Derivatives: \n");
	      //printp(derp, derb);
	      
	      lp();
	      if(exact_repr) 
		{
		  for(k=1;k<=l;k++)
		    dp[k]=0;
		}
	      // Result of simplex:
	      //printf("result of simplex: \n");
	      //printp(dp, dbeta);
	      
	      changed=1;
	      if(along()==0) 
		{
		  emptyr++;
		  changed=0;
		  Delta=0.999*Delta;
		  if(Delta<0.01) 
		    {
		      //printf("  Number of rounds (empty rounds):
		      //  %d (%d), Delta: %1.4e\n",j,emptyr,Delta);
		      j=nr+2;
		    }
		}
	      /*printp(p,beta);*/
	      /*printf("likeli: %f\n", likely(p,beta));*/
	    }
	  printf("\nEnd of round n. %d:\n\n", i);
	  fflush(stdout);
	  //if(j!=nr+2)
	  //  printf("  Number of rounds (empty rounds): %d (%d), Delta: %1.4e\n",
	  //   nr,emptyr,Delta);
	  if( method==1 || method==2 ) lik=likely(p,beta);
	  if( method==3 ) lik=areadiff(p,beta);
	  if( method==4 ) lik=relareadiff(p,beta);
	  if( method==5 ) lik=cdfareadiff(p,beta);
	  /*printp(p,beta);*/
	  /*printf("likeli: %f\n", lik);*/
	  if( bestlike<lik)
	    {
	      bestlike=lik;
	      for(j=1;j<=l;j++)
		{
		  bestbeta[j]=beta[j];
		  bestp[j]=p[j];
		}
	    }
	}
      
      //printp(bestp,bestbeta);
      Result2File();
      memfree();
      printf("Fitting has finished.\n");
      return 0;
    }  
  // End of Fitting a distribution given by samples 
  // ==============================================


  // ==================================
  // Producing data file to plot curves
  if(atoi(argv[1])==4)
    {
      samples=1;
      switch(atoi(argv[3]))
	{
	case 1:
	  loadcdf(argv[2]);
	  break;
	case 2:
	  loadccdf(argv[2]);
	  break;
	case 3:
	  loadsamples(argv[2]);
	  break;
	case 4:
	  loadcdf(argv[2]);
	  break;
	}
      pdf=&pdf_loaded;
      distmean=&mean_loaded;	
      cdf=&cdf_loaded;
      invcdf=&invcdf_loaded;	
      ccdf=&ccdf_loaded;	
      invccdf=&invccdf_loaded;

      if(atoi(argv[3])==3 || atoi(argv[3])==4)
	createcdf();
      //if(atoi(argv[3])==1 || atoi(argv[3])==2)
      //createsamples();

      fitting_delta=atof(argv[7]);
      ResultFromFile();
      printp(bestp,bestbeta);
      memalloc2();
      sptailtype=atoi(argv[8]);
      if(sptailtype!=0)
	{
	  sptailparam=atof(argv[9]);
	  if(atof(argv[10])>0)
	    sptailstart=atof(argv[10]);
	  else
	    {
	      sptailstart=invccdf(-atof(argv[10]),param1,param2);
	      printf(" Start of special tail: %1.4e\n", sptailstart);
	    }
	}
      if(atof(argv[6])>0.0)
	genlist(atof(argv[4]),atof(argv[5]),fabs(atof(argv[6])),1);
      else
	genlist(atof(argv[4]),atof(argv[5]),fabs(atof(argv[6])),0);
      memfree();
      return 0;
    }
  // Producing data file to plot curves finished
  // ===========================================
  

  fitting_delta=atof(argv[10]);
 
  //parameters of tail fitting
  lh=atoi(argv[11]);
  mtail1=atof(argv[12]);	
  mtail2=atof(argv[13]);	
  adj=atof(argv[14]);		
  decay=atof(argv[15]);		
  
  method=atoi(argv[7]);
  if(method<5)
    {
      mean=readobs(argv[1]);
      readdist(argv[2]);
      printf("Mean without the tail: %f\n", mean);
    }
  else
    {
      mean=readobs2(argv[1]);
      readdist2(argv[2]);
      printf("mean=%f\n", mean);
    }
  l=atoi(argv[3]);
  //mean=1.5;

  tail=atoi(argv[8]);
  tailw=atoi(argv[9]);
  /*printf("tail: %d, tailw: %d\n", tail, tailw);*/
  printf("fitting_delta %f\n", fitting_delta);
  is_it_exact();
  printf(" Exact representation: %d, number of phases needed: %d\n",
	 exact_repr, nec_phases);
  if(exact_repr) l=nec_phases;

  memalloc();

  build_geom();
  printf("Mean with the tail: %f\n", mean);
  for(i=0;i<lh;i++) mean-=h[i]/lambda[i];
  printf("Mean changed by The Mixture: %f\n",mean);

  srand(1);
  
  bestlike=-10000000.0;
  for(i=1;i<=1;i++)
    {
      printf( " %d. approximation \n", i);
      if( mean<1.5*l ) 
	{ 
	  stvalues(1.5*l); 
	}
      else 
	{ 
	  stvalues(mean); 
	}

      printf("startting values n. %d:\n", i);
      printp(p,beta);
      /*printf(" likeli: %f\n", likely(p,beta));*/
      /*printf(" areadifference: %f\n", -areadiff(p,beta));*/
      
      Delta=0.1;
      changed=1;
      emptyr=0;
      for(j=1;j<=nr;j++)
	{
	  //printf("%d ",j);
	  if( method==1 && changed) derive1();
	  if( method==2 && changed) derive2();
	  if( method==3 && changed) derive3();
	  if( method==4 && changed) derive4();
	  if( method==5 && changed) derive5();
	  //printf("Parameters: \n");
	  //printp(p,beta);
	  //printf("Derivatives: \n");
	  //printp(derp, derb);

	  lp();
	  if(exact_repr) 
	    {
	      for(k=1;k<=l;k++)
		dp[k]=0;
	    }
	  // Result of simplex:
	  //printf("result of simplex: \n");
	  //printp(dp, dbeta);

	  changed=1;
	  if(along()==0) 
	    {
	      emptyr++;
	      changed=0;
	      Delta=0.99*Delta;
	      if(Delta<0.01) 
		{
		  printf("  Number of rounds (empty rounds):
                          %d (%d), Delta: %1.4e\n",j,emptyr,Delta);
		  j=nr+2;
		}
	    }
	  /*printp(p,beta);*/
	  /*printf("likeli: %f\n", likely(p,beta));*/
	}
      if(j!=nr+2)
	printf("  Number of rounds (empty rounds): %d (%d), Delta: %1.4e\n",
	     nr,emptyr,Delta);
      if( method==1 || method==2 ) lik=likely(p,beta);
      if( method==3 ) lik=areadiff(p,beta);
      if( method==4 ) lik=relareadiff(p,beta);
      if( method==5 ) lik=cdfareadiff(p,beta);
      /*printp(p,beta);*/
      /*printf("likeli: %f\n", lik);*/
      if( bestlike<lik)
	{
	  bestlike=lik;
	  for(j=1;j<=l;j++)
	    {
	      bestbeta[j]=beta[j];
	      bestp[j]=p[j];
	    }
	}
    }
  
  printp(bestp,bestbeta);
  printf("likeli: %f\n", bestlike);
  /*printf("areadifference: %f\n", -areadiff(bestp, bestbeta));*/  
  
  m = (struct measures *)malloc(sizeof(struct measures));
  
  compmeasures(m);
  printf("Measures:\n");
  printf("  Likelyhood:                    %1.4e\n", m->ent);
  printf("  Areadifference:                %1.4e\n", m->ad); 
  printf("  Relative Areadifference:       %1.4e\n", m->rad);
  printf("  Relative Error in 1st Moment:  %1.4e\n", m->m1);
  printf("  Relative Error in 2nd Moment:  %1.4e\n", m->m2);
  printf("  Relative Error in 3rd Moment:  %1.4e\n", m->m3);
  
  writematrix(argv[4]);
  //genlist(argv[5], atoi(argv[6]));
  
  memfree();
  free(m);
  return 0;
}
		

