package v1;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.io.*;
import java.util.Timer;
import java.util.TimerTask;

/**
 * <p>Title: PhFit</p>
 * <p>Description: General Phase-Type Fitting Tool</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Andr\uFFFDs Horv\uFFFDth
 * @version 1.0
 */

public class PhFitFrame extends JFrame {

  static String[] PDFFamilies = {
    "Weibull","Lognormal","Uniform","Pareto-like I","Pareto-like II"};
  static String[] FileTypes = {
    "Samples","Samples & Weights"};
  static String[] Measures = {
    "Entropy (numerical)","pdf area difference","cdf area difference"};
  static String[] SaveFormats = {
    "Textual", "Matrix"};
  static String[] Curves = {
    "pdf","cdf","ccdf","hazard rate","relative error of pdf",
    "relative error of cdf","relative error of ccdf",
    "rel. err. of haz. rate"};
  int imagecounter=1;

  // ---------------
  // Execute command
  private int execProcess(String commandLine)
    {
	try
	    {
		System.out.println("trying to exec: " + commandLine);
		Process p = Runtime.getRuntime().exec(commandLine);

		BufferedReader b = new BufferedReader(
                  new InputStreamReader(p.getInputStream()));
		String l;
                while ((l = b.readLine()) != null)
                   {
                        System.out.println(l);
                        //Debug.message("server.exec",l);
		   }

		int ExitVal = p.waitFor();
                System.out.println("terminate with: " + ExitVal);
		return ExitVal;
	    }
	catch (Exception e)
	    {
		System.out.println("Error: " + e.getMessage());
		return ERROR;
	    }
    } // of execProcess

  JPanel contentPane;
  JMenuBar jMenuBar1 = new JMenuBar();
  JMenu jMenuFile = new JMenu();
  JMenuItem jMenuFileExit = new JMenuItem();
  JMenu jMenuHelp = new JMenu();
  JMenuItem jMenuHelpAbout = new JMenuItem();
  JLabel statusBar = new JLabel();
  JPanel jInputPanel = new JPanel();
  TitledBorder titledBorder1;
  JLabel jInputLabel = new JLabel();
  JTabbedPane jDistributionInputTabbed = new JTabbedPane();
  JTabbedPane jFittingInputTabbed = new JTabbedPane();
  JLabel jDistributionInputLabel = new JLabel();
  JLabel jFittingInputLabel = new JLabel();
  JPanel jInputBodyPanel = new JPanel();
  JPanel jFittingBodyPanel = new JPanel();
  JPanel jFittingTailPanel = new JPanel();
  JLabel jGivenByLabel = new JLabel();
  JRadioButton jGivenByPDFButton = new JRadioButton();
  JRadioButton jGivenBySamplesButton = new JRadioButton();
  ButtonGroup GivenByGroup = new ButtonGroup();
  JPanel jPDFPanel = new JPanel();
  TitledBorder titledBorder2;
  JLabel jPDFFamilyLabel = new JLabel();
  JComboBox jPDFFamilyCombo = new JComboBox();
  JLabel jPDFParam1Label = new JLabel();
  JTextField jPDFParam1Field = new JTextField();
  JLabel jPDFParam2Label = new JLabel();
  JTextField jPDFParam2Field = new JTextField();
  JPanel jPDFFilePanel = new JPanel();
  JLabel jPDFFileNameLabel = new JLabel();
  JComboBox jPDFFileTypeCombo = new JComboBox();
  JButton jPDFFileChooserButton = new JButton();
  JTextField jPDFFileNameField = new JTextField();
  JLabel jPDFFileTypeLabel = new JLabel();
  JPanel jInputTailPanel = new JPanel();
  JCheckBox jTailSpecialCheck = new JCheckBox();
  JPanel jPanel1 = new JPanel();
  TitledBorder titledBorder3;
  JLabel jTailShapeLabel = new JLabel();
  JRadioButton jPolyTailButton = new JRadioButton();
  JRadioButton jExpTailButton = new JRadioButton();
  ButtonGroup TailShapeGroup = new ButtonGroup();
  JTextField jTailParamField = new JTextField();
  JTextField jTailStartField = new JTextField();
  JTextField jTailWeightField = new JTextField();
  JLabel jTailParamLabel = new JLabel();
  JLabel jTailStartLabel = new JLabel();
  JLabel jTailWeightLabel = new JLabel();
  JPanel jFittingDomainPanel = new JPanel();
  JLabel jAppliedClassLabel = new JLabel();
  JRadioButton jContClassButton = new JRadioButton();
  JRadioButton jDistClassButton = new JRadioButton();
  ButtonGroup AppliedClassGroup = new ButtonGroup();
  JLabel jMainPhasesLabel = new JLabel();
  JTextField jMainPhasesField = new JTextField();
  JLabel jMeasureLabel = new JLabel();
  JComboBox jMeasureCombo = new JComboBox();
  JLabel jMainUpperLabel = new JLabel();
  JTextField jMainUpperField = new JTextField();
  JLabel jMainUpperModeLabel = new JLabel();
  JRadioButton jBodyQuantileButton = new JRadioButton();
  JRadioButton jBodyLimitButton = new JRadioButton();
  ButtonGroup BodyUpper = new ButtonGroup();
  JRadioButton jTailQuantileButton = new JRadioButton();
  JLabel jTailUpperLabel = new JLabel();
  JRadioButton jTailLimitButton = new JRadioButton();
  JTextField jTailUpperField = new JTextField();
  JLabel jTailLimitsModeLabel = new JLabel();
  ButtonGroup TailLimitGroup = new ButtonGroup();
  JLabel jTailLowerLabel = new JLabel();
  JTextField jTailLowerField = new JTextField();
  JLabel jTailPhasesLabel = new JLabel();
  JTextField jTailPhasesField = new JTextField();
  JLabel jTailAdjacentLabel = new JLabel();
  JTextField jTailAdjacentField = new JTextField();
  JPanel jPanel2 = new JPanel();
  TitledBorder titledBorder4;
  JLabel jResultLabel = new JLabel();
  JLabel jInitialProbLabel = new JLabel();
  JLabel jIntensitiesLabel = new JLabel();
  JLabel jSaveLabel = new JLabel();
  JComboBox jSaveCombo = new JComboBox();
  JButton jSaveButton = new JButton();
  JLabel jCurveCreateLabel = new JLabel();
  JComboBox jCurveCreateCombo = new JComboBox();
  JPanel jCurvePanel = new JPanel();
  JLabel jTheCurveItselfLabel = new JLabel();
  JScrollPane jCurveScrollPane = new JScrollPane();
  JScrollPane jInitialPane = new JScrollPane();
  JScrollPane jIntensityPane = new JScrollPane();
  JTabbedPane jTabbedPane1 = new JTabbedPane();
  JButton jCreateButton = new JButton();
  JButton jPerformFittingButton = new JButton();
  JLabel jStepSizeLabel = new JLabel();
  JTextField jStepSizeField = new JTextField();
  JMenuItem jDemoMenu = new JMenuItem();
  JTextArea jInitialArea = new JTextArea();
  JTextArea jIntensityArea = new JTextArea();
  JButton jSaveCurveButton = new JButton();

  //Construct the frame
  public PhFitFrame() {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      jbInit();
      jStepSizeLabel.setForeground(Color.gray);
      jStepSizeField.setEnabled(false);
      jPDFFilePanel.setVisible(false);
      jPolyTailButton.setEnabled(false);
      jExpTailButton.setEnabled(false);
      jTailStartField.setEnabled(false);
      jTailWeightField.setEnabled(false);
      jTailParamField.setEnabled(false);
      jTailShapeLabel.setForeground(Color.gray);
      jTailParamLabel.setForeground(Color.gray);
      jTailStartLabel.setForeground(Color.gray);
      jTailWeightLabel.setForeground(Color.gray);
      jPolyTailButton.setForeground(Color.gray);
      jExpTailButton.setForeground(Color.gray);
      jExpTailButton.setSelected(false);
      jExpTailButton.setSelected(false);

    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }
  //Component initialization
  private void jbInit() throws Exception  {
    //setIconImage(Toolkit.getDefaultToolkit().createImage(PhFitFrame.class.getResource("[Your Icon]")));
    contentPane = (JPanel) this.getContentPane();
    titledBorder1 = new TitledBorder("");
    titledBorder2 = new TitledBorder("");
    titledBorder3 = new TitledBorder("");
    titledBorder4 = new TitledBorder("");
    contentPane.setLayout(null);
    this.setForeground(Color.black);
    this.setResizable(false);
    this.setSize(new Dimension(686, 528));
    this.setTitle("PhFit");
    this.addWindowListener(new java.awt.event.WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        this_windowClosing(e);
      }
    });
    statusBar.setBorder(BorderFactory.createLoweredBevelBorder());
    statusBar.setText("Do we need Status(Bar)? Do we need Status(Bar)? Do we need Status(Bar)? " +
    "Do we need Status(Bar)? Do we need Status(Bar)?");
    statusBar.setBounds(new Rectangle(1, 455, 681, 17));
    jMenuFile.setText("File");
    jMenuFileExit.setText("Exit");
    jMenuFileExit.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        jMenuFileExit_actionPerformed(e);
      }
    });
    jMenuHelp.setText("Help");
    jMenuHelpAbout.setText("About");
    jMenuHelpAbout.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        jMenuHelpAbout_actionPerformed(e);
      }
    });
    jInputPanel.setBorder(titledBorder1);
    jInputPanel.setBounds(new Rectangle(6, 6, 245, 446));
    jInputPanel.setLayout(null);
    jInputLabel.setFont(new java.awt.Font("Dialog", 1, 12));
    jInputLabel.setText("Input Parameters");
    jInputLabel.setBounds(new Rectangle(9, 8, 114, 17));
    jDistributionInputTabbed.setBounds(new Rectangle(9, 50, 226, 165));
    jFittingInputTabbed.setBounds(new Rectangle(9, 242, 226, 165));
    jDistributionInputLabel.setText("Fitted Distribution:");
    jDistributionInputLabel.setBounds(new Rectangle(11, 31, 139, 17));
    jFittingInputLabel.setText("Fitting Parameters:");
    jFittingInputLabel.setBounds(new Rectangle(11, 223, 139, 17));
    jInputBodyPanel.setLayout(null);
    jFittingBodyPanel.setLayout(null);
    jFittingTailPanel.setLayout(null);
    jGivenByLabel.setText("Given by");
    jGivenByLabel.setBounds(new Rectangle(5, 6, 60, 17));
    jGivenByPDFButton.setHorizontalTextPosition(SwingConstants.RIGHT);
    jGivenByPDFButton.setSelected(true);
    jGivenByPDFButton.setText("distribution function");
    jGivenByPDFButton.setBounds(new Rectangle(69, 2, 152, 25));
    jGivenBySamplesButton.setText("description in file");
    jGivenBySamplesButton.setBounds(new Rectangle(69, 23, 148, 25));
    jGivenBySamplesButton.addChangeListener(new javax.swing.event.ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        jGivenBySamplesButton_stateChanged(e);
      }
    });
    jPDFPanel.setBorder(titledBorder2);
    jPDFPanel.setBounds(new Rectangle(5, 52, 211, 79));
    jPDFPanel.setLayout(null);
    jPDFFamilyLabel.setText("Family:");
    jPDFFamilyLabel.setBounds(new Rectangle(10, 7, 54, 17));
    jPDFFamilyCombo.setBounds(new Rectangle(62, 6, 125, 21));
    jPDFParam1Label.setText("1st Parameter:");
    jPDFParam1Label.setBounds(new Rectangle(8, 31, 91, 17));
    jPDFParam1Field.setBounds(new Rectangle(103, 29, 83, 21));
    jPDFParam2Label.setBounds(new Rectangle(6, 55, 94, 17));
    jPDFParam2Label.setText("2nd Parameter:");
    jPDFParam2Field.setBounds(new Rectangle(103, 53, 83, 21));
    contentPane.setEnabled(true);
    jPDFFilePanel.setLayout(null);
    jPDFFilePanel.setBorder(titledBorder2);
    jPDFFilePanel.setBounds(new Rectangle(5, 52, 211, 79));
    jPDFFileNameLabel.setBounds(new Rectangle(6, 7, 41, 17));
    jPDFFileNameLabel.setText("File:");
    jPDFFileTypeCombo.setBounds(new Rectangle(63, 31, 143, 21));
    jPDFFileChooserButton.setBounds(new Rectangle(187, 6, 19, 20));
    jPDFFileChooserButton.setFont(new java.awt.Font("Dialog", 1, 12));
    jPDFFileChooserButton.setText("...");
    jPDFFileChooserButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jPDFFileChooserButton_actionPerformed(e);
      }
    });
    jPDFFileNameField.setBounds(new Rectangle(35, 6, 148, 21));
    jPDFFileTypeLabel.setText("File Type:");
    jPDFFileTypeLabel.setBounds(new Rectangle(6, 33, 55, 17));
    jInputTailPanel.setLayout(null);
    jTailSpecialCheck.setActionCommand("jSpecialTailCheckBox");
    jTailSpecialCheck.setText("Special Tail");
    jTailSpecialCheck.setBounds(new Rectangle(8, 2, 90, 25));
    jTailSpecialCheck.addChangeListener(new javax.swing.event.ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        jTailSpecialCheck_stateChanged(e);
      }
    });
    jPanel1.setBorder(titledBorder3);
    jPanel1.setBounds(new Rectangle(6, 28, 208, 100));
    jPanel1.setLayout(null);
    jTailShapeLabel.setText("Shape:");
    jTailShapeLabel.setBounds(new Rectangle(3, 8, 43, 17));
    jPolyTailButton.setSelected(true);
    jPolyTailButton.setText("Polynomial (Pareto)");
    jPolyTailButton.setBounds(new Rectangle(47, 5, 158, 25));
    jExpTailButton.setBounds(new Rectangle(47, 25, 157, 25));
    jExpTailButton.setText("Exponential (Weibull)");
    jTailParamField.setBounds(new Rectangle(155, 70, 45, 21));
    jTailStartField.setBounds(new Rectangle(90, 70, 45, 21));
    jTailWeightField.setEnabled(false);
    jTailWeightField.setBounds(new Rectangle(23, 70, 47, 21));
    jTailParamLabel.setText("Parameter:");
    jTailParamLabel.setBounds(new Rectangle(5, 53, 78, 17));
    jTailStartLabel.setText("Start:");
    jTailStartLabel.setBounds(new Rectangle(86, 53, 41, 17));
    jTailWeightLabel.setText("Weight:");
    jTailWeightLabel.setBounds(new Rectangle(145, 53, 55, 17));
    jFittingDomainPanel.setLayout(null);
    jAppliedClassLabel.setText("Applied Phase-Type Class:");
    jAppliedClassLabel.setBounds(new Rectangle(8, 26, 170, 17));
    jContClassButton.setSelected(true);
    jContClassButton.setText("Continuous");
    jContClassButton.setBounds(new Rectangle(107, 46, 103, 25));
    jDistClassButton.setBounds(new Rectangle(107, 67, 103, 25));
    jDistClassButton.addChangeListener(new javax.swing.event.ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        jDistClassButton_stateChanged(e);
      }
    });
    jDistClassButton.setText("Dicrete");
    jMainPhasesLabel.setText("Number of Phases:");
    jMainPhasesLabel.setBounds(new Rectangle(6, 12, 130, 17));
    jMainPhasesField.setBounds(new Rectangle(127, 11, 30, 21));
    jMeasureLabel.setText("Applied Measure:");
    jMeasureLabel.setBounds(new Rectangle(6, 37, 107, 17));
    jMeasureCombo.setBounds(new Rectangle(117, 36, 102, 21));
    jMainUpperLabel.setText("Upper Limit of Fitting:");
    jMainUpperLabel.setBounds(new Rectangle(5, 62, 138, 17));
    jMainUpperField.setBounds(new Rectangle(147, 61, 70, 21));
    jMainUpperModeLabel.setText("Upper Limit given by");
    jMainUpperModeLabel.setBounds(new Rectangle(6, 88, 134, 17));
    jBodyQuantileButton.setSelected(true);
    jBodyQuantileButton.setText("quantile");
    jBodyQuantileButton.setBounds(new Rectangle(143, 85, 82, 25));
    jBodyLimitButton.setText("itself");
    jBodyLimitButton.setBounds(new Rectangle(143, 105, 82, 25));
    jTailQuantileButton.setBounds(new Rectangle(128, 71, 82, 25));
    jTailQuantileButton.setText("quantiles");
    jTailQuantileButton.setSelected(true);
    jTailUpperLabel.setBounds(new Rectangle(2, 51, 172, 17));
    jTailUpperLabel.setText("Upper Limit of Tail Fitting:");
    jTailLimitButton.setBounds(new Rectangle(128, 92, 98, 25));
    jTailLimitButton.setText("themselves");
    jTailUpperField.setBounds(new Rectangle(172, 50, 47, 21));
    jTailLimitsModeLabel.setBounds(new Rectangle(6, 75, 134, 17));
    jTailLimitsModeLabel.setText("Limits are given by");
    jTailLowerLabel.setText("Lower Limit of Tail Fitting:");
    jTailLowerLabel.setBounds(new Rectangle(2, 28, 175, 17));
    jTailLowerField.setBounds(new Rectangle(172, 27, 47, 21));
    jTailPhasesLabel.setBounds(new Rectangle(32, 5, 130, 17));
    jTailPhasesLabel.setText("Number of Phases:");
    jTailPhasesField.setBounds(new Rectangle(150, 3, 30, 21));
    jTailPhasesField.setText("0");
    jTailAdjacentLabel.setText("Distance of adjacent points:");
    jTailAdjacentLabel.setBounds(new Rectangle(3, 115, 177, 17));
    jTailAdjacentField.setBounds(new Rectangle(183, 114, 35, 21));
    jPanel2.setBorder(titledBorder4);
    jPanel2.setBounds(new Rectangle(257, 6, 422, 447));
    jPanel2.setLayout(null);
    jResultLabel.setText("Resulting Distribution");
    jResultLabel.setBounds(new Rectangle(6, 8, 168, 17));
    jResultLabel.setFont(new java.awt.Font("Dialog", 1, 12));
    jInitialProbLabel.setText("Initial prob.:");
    jInitialProbLabel.setBounds(new Rectangle(6, 30, 160, 17));
    jIntensitiesLabel.setBounds(new Rectangle(217, 30, 76, 17));
    jIntensitiesLabel.setText("Intensities:");
    jSaveLabel.setText("Save format:");
    jSaveLabel.setBounds(new Rectangle(14, 88, 88, 17));
    jSaveCombo.setBounds(new Rectangle(97, 87, 168, 21));
    jSaveButton.setBounds(new Rectangle(273, 87, 137, 20));
    jSaveButton.setText("Save");
    jSaveButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jSaveButton_actionPerformed(e);
      }
    });
    jCurveCreateLabel.setText("Curve:");
    jCurveCreateLabel.setBounds(new Rectangle(7, 7, 100, 17));
    jCurveCreateCombo.setBounds(new Rectangle(50, 6, 151, 21));
    jCurvePanel.setBorder(BorderFactory.createLoweredBevelBorder());
    jCurvePanel.setLayout(null);
    jTheCurveItselfLabel.setHorizontalAlignment(SwingConstants.CENTER);
    jTheCurveItselfLabel.setText("Result will be visualized here...");
    jCurveScrollPane.setBounds(new Rectangle(6, 33, 388, 270));
    jInitialPane.setBounds(new Rectangle(92, 31, 121, 51));
    jIntensityPane.setBounds(new Rectangle(293, 32, 118, 50));
    jTabbedPane1.setBounds(new Rectangle(12, 107, 405, 335));
    jCreateButton.setBounds(new Rectangle(204, 5, 97, 23));
    jCreateButton.setText("Create");
    jCreateButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jCreateButton_actionPerformed(e);
      }
    });
    jPerformFittingButton.setBounds(new Rectangle(9, 410, 226, 31));
    jPerformFittingButton.setText("Perform Fitting");
    jPerformFittingButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jPerformFittingButton_actionPerformed(e);
      }
    });
    jStepSizeLabel.setText("Step size:");
    jStepSizeLabel.setBounds(new Rectangle(8, 97, 66, 17));
    jStepSizeField.setBounds(new Rectangle(71, 96, 63, 21));
    jDemoMenu.setText("Demo");
    jDemoMenu.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jDemoMenu_actionPerformed(e);
      }
    });
    jSaveCurveButton.setBounds(new Rectangle(306, 5, 88, 23));
    jSaveCurveButton.setText("Save");
    jSaveCurveButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jSaveCurveButton_actionPerformed(e);
      }
    });
    jCurvePanel.add(jCurveCreateLabel, null);
    jCurvePanel.add(jCurveScrollPane, null);
    jCurvePanel.add(jCurveCreateCombo, null);
    jCurvePanel.add(jCreateButton, null);
    jCurvePanel.add(jSaveCurveButton, null);
    jCurveScrollPane.getViewport().add(jTheCurveItselfLabel, null);
    jPanel2.add(jTabbedPane1, null);
    jPanel2.add(jInitialPane, null);
    jInitialPane.getViewport().add(jInitialArea, null);
    jPanel2.add(jResultLabel, null);
    jPanel2.add(jIntensityPane, null);
    jPanel2.add(jInitialProbLabel, null);
    jPanel2.add(jSaveCombo, null);
    jPanel2.add(jSaveLabel, null);
    jPanel2.add(jSaveButton, null);
    jPanel2.add(jIntensitiesLabel, null);
    jTabbedPane1.add(jCurvePanel, "Visual Properties");
    contentPane.add(statusBar, null);
    jMenuFile.add(jMenuFileExit);
    jMenuHelp.add(jDemoMenu);
    jMenuHelp.add(jMenuHelpAbout);
    jMenuBar1.add(jMenuFile);
    jMenuBar1.add(jMenuHelp);
    this.setJMenuBar(jMenuBar1);
    contentPane.add(jInputPanel, null);
    jInputPanel.add(jDistributionInputLabel, null);
    jInputPanel.add(jInputLabel, null);
    jInputPanel.add(jFittingInputLabel, null);
    jInputPanel.add(jDistributionInputTabbed, null);
    jInputPanel.add(jFittingInputTabbed, null);
    jDistributionInputTabbed.add(jInputBodyPanel, "Body");
    jInputBodyPanel.add(jGivenByLabel, null);
    jInputBodyPanel.add(jGivenByPDFButton, null);
    jInputBodyPanel.add(jGivenBySamplesButton, null);
    jInputBodyPanel.add(jPDFPanel, null);
    GivenByGroup.add(jGivenByPDFButton);
    GivenByGroup.add(jGivenBySamplesButton);
    jPDFPanel.add(jPDFParam1Field, null);
    jPDFPanel.add(jPDFParam2Field, null);
    jPDFPanel.add(jPDFFamilyCombo, null);
    jPDFPanel.add(jPDFParam2Label, null);
    jPDFPanel.add(jPDFParam1Label, null);
    jPDFPanel.add(jPDFFamilyLabel, null);
    jInputBodyPanel.add(jPDFFilePanel, null);
    jPDFFilePanel.add(jPDFFileTypeCombo, null);
    jPDFFilePanel.add(jPDFFileNameLabel, null);
    jPDFFilePanel.add(jPDFFileNameField, null);
    jPDFFilePanel.add(jPDFFileTypeLabel, null);
    jPDFFilePanel.add(jPDFFileChooserButton, null);
    jDistributionInputTabbed.add(jInputTailPanel, "Tail");

    // Filling PDFCombo with families
    for (int i = 0; i < PDFFamilies.length; i++)
      jPDFFamilyCombo.addItem(PDFFamilies[i]);
    // Filling FileTypeCombo with filetypes
    for (int i = 0; i < FileTypes.length; i++)
      jPDFFileTypeCombo.addItem(FileTypes[i]);
    // Filling MeasureCombo with filetypes
    for (int i = 0; i < Measures.length; i++)
      jMeasureCombo.addItem(Measures[i]);
    // Filling SaveCombo with filetypes
    for (int i = 0; i < SaveFormats.length; i++)
      jSaveCombo.addItem(SaveFormats[i]);
    // Filling CurveCreateCombo with filetypes
    for (int i = 0; i < Curves.length; i++)
      jCurveCreateCombo.addItem(Curves[i]);


    jInputTailPanel.add(jTailSpecialCheck, null);
    jInputTailPanel.add(jPanel1, null);
    jPanel1.add(jTailParamField, null);
    jPanel1.add(jTailStartField, null);
    jPanel1.add(jTailWeightField, null);
    jPanel1.add(jTailParamLabel, null);
    jPanel1.add(jTailStartLabel, null);
    jPanel1.add(jTailWeightLabel, null);
    jPanel1.add(jPolyTailButton, null);
    jPanel1.add(jExpTailButton, null);
    jPanel1.add(jTailShapeLabel, null);
    TailShapeGroup.add(jPolyTailButton);
    TailShapeGroup.add(jExpTailButton);
    jFittingInputTabbed.add(jFittingDomainPanel, "Time Domain");
    jFittingDomainPanel.add(jAppliedClassLabel, null);
    jFittingInputTabbed.add(jFittingBodyPanel,  "Body");
    jFittingInputTabbed.add(jFittingTailPanel,  "Tail");
    jFittingDomainPanel.add(jContClassButton, null);
    jFittingDomainPanel.add(jDistClassButton, null);
    AppliedClassGroup.add(jContClassButton);
    AppliedClassGroup.add(jDistClassButton);
    jFittingBodyPanel.add(jMainPhasesLabel, null);
    jFittingBodyPanel.add(jMeasureLabel, null);
    jFittingBodyPanel.add(jMainUpperLabel, null);
    jFittingBodyPanel.add(jMainUpperModeLabel, null);
    jFittingBodyPanel.add(jMeasureCombo, null);
    jFittingBodyPanel.add(jBodyQuantileButton, null);
    jFittingBodyPanel.add(jBodyLimitButton, null);
    jFittingBodyPanel.add(jMainPhasesField, null);
    jFittingBodyPanel.add(jMainUpperField, null);
    BodyUpper.add(jBodyQuantileButton);
    BodyUpper.add(jBodyLimitButton);
    TailLimitGroup.add(jTailQuantileButton);
    TailLimitGroup.add(jTailLimitButton);
    jFittingTailPanel.add(jTailUpperLabel, null);
    jFittingTailPanel.add(jTailLowerLabel, null);
    jFittingTailPanel.add(jTailLimitsModeLabel, null);
    jFittingTailPanel.add(jTailPhasesLabel, null);
    jInputPanel.add(jPerformFittingButton, null);
    contentPane.add(jPanel2, null);
    jFittingTailPanel.add(jTailUpperField, null);
    jFittingTailPanel.add(jTailLowerField, null);
    jFittingTailPanel.add(jTailAdjacentField, null);
    jFittingTailPanel.add(jTailAdjacentLabel, null);
    jFittingTailPanel.add(jTailQuantileButton, null);
    jFittingTailPanel.add(jTailPhasesField, null);
    jFittingTailPanel.add(jTailLimitButton, null);
    jFittingDomainPanel.add(jStepSizeLabel, null);
    jFittingDomainPanel.add(jStepSizeField, null);
    jIntensityPane.getViewport().add(jIntensityArea, null);

  }
  //File | Exit action performed
  public void jMenuFileExit_actionPerformed(ActionEvent e) {
    RemoveAll();
    System.exit(0);
  }
  //Help | About action performed
  public void jMenuHelpAbout_actionPerformed(ActionEvent e) {
    PhFitFrame_AboutBox dlg = new PhFitFrame_AboutBox(this);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.setModal(true);
    dlg.show();
  }
  //Overridden so we can exit when window is closed
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      jMenuFileExit_actionPerformed(null);
    }
  }

  void jGivenBySamplesButton_stateChanged(ChangeEvent e) {
    if(!jGivenBySamplesButton.isSelected())
      {
        jPDFFilePanel.setVisible(false);
        jPDFPanel.setVisible(true);
      }
    else
      {
        jPDFPanel.setVisible(false);
        jPDFFilePanel.setVisible(true);
      }
  }

  void jPDFFileChooserButton_actionPerformed(ActionEvent e) {
    JFileChooser chooser = new JFileChooser();
    int returnVal = chooser.showOpenDialog(this);
    if(returnVal == JFileChooser.APPROVE_OPTION)
      {
        jPDFFileNameField.setText(
          chooser.getSelectedFile().getPath());
      }
  }

  void jTailSpecialCheck_stateChanged(ChangeEvent e) {
    if(jTailSpecialCheck.isSelected())
      {
        jPolyTailButton.setEnabled(true);
        jExpTailButton.setEnabled(true);
        jTailStartField.setEnabled(true);
        jTailWeightField.setEnabled(true);
        jTailParamField.setEnabled(true);
        jTailShapeLabel.setForeground(Color.black);
        jTailParamLabel.setForeground(Color.black);
        jTailStartLabel.setForeground(Color.black);
        jTailWeightLabel.setForeground(Color.black);
        jPolyTailButton.setForeground(Color.black);
        jExpTailButton.setForeground(Color.black);
      }
    else
      {
        jPolyTailButton.setEnabled(false);
        jExpTailButton.setEnabled(false);
        jTailStartField.setEnabled(false);
        jTailWeightField.setEnabled(false);
        jTailParamField.setEnabled(false);
        jTailShapeLabel.setForeground(Color.gray);
        jTailParamLabel.setForeground(Color.gray);
        jTailStartLabel.setForeground(Color.gray);
        jTailWeightLabel.setForeground(Color.gray);
        jPolyTailButton.setForeground(Color.gray);
        jExpTailButton.setForeground(Color.gray);
        jExpTailButton.setSelected(false);
        jExpTailButton.setSelected(false);
      }
  }

  void jCreateButton_actionPerformed(ActionEvent e) {

      double ss=0.0;
      if(!jContClassButton.isSelected())
        ss=Double.valueOf(jStepSizeField.getText()).doubleValue();
      PhFitFrame_CreateBox dlg = new PhFitFrame_CreateBox(
        jContClassButton.isSelected(),jPDFFamilyCombo.getSelectedIndex(),
        jPDFParam1Field.getText(),jPDFParam2Field.getText(),
        imagecounter,jCurveCreateCombo.getSelectedIndex()+2,
        false,0.0,0.0,0,false,false,ss);
      dlg.setVisible(true);

      jTheCurveItselfLabel.setIcon(
        new ImageIcon("TEMPORARYPhFitImage"+imagecounter+".gif"));
      imagecounter=imagecounter+1;
      jTheCurveItselfLabel.setText("");
  }

  void RemoveAll() {
    int i1;
    for(i1=1;i1<imagecounter;i1++)
      {
        File toremove = new File("TEMPORARYPhFitImage"+String.valueOf(i1)+".gif");
        toremove.delete();
      }
      File toremove1 = new File("TEMPORARYPhFitFILE.gnu");
      toremove1.delete();
      File toremove2 = new File("TEMPORARYPhFitFILE.data");
      toremove2.delete();
      File toremove3 = new File("TEMPORARYPhFitFILE.initials");
      toremove3.delete();
      File toremove4 = new File("TEMPORARYPhFitFILE.intensities");
      toremove4.delete();
      File toremove5 = new File("TEMPORARYPhFitFILE.result");
      toremove5.delete();
      File toremove6 = new File("TEMPORARYPhFitFILE.textual");
      toremove6.delete();
      File toremove7 = new File("TEMPORARYPhFitFILE.matrix");
      toremove7.delete();
    }

  void this_windowClosing(WindowEvent e) {
    RemoveAll();
    }

  String FittingContPDF() {
    String co;
    co="contfit 1 ";
    co=co+String.valueOf(jPDFFamilyCombo.getSelectedIndex()+1)+" ";
    co=co+jPDFParam1Field.getText()+" ";
    co=co+jPDFParam2Field.getText()+" ";
    co=co+jMainPhasesField.getText()+" ";
    if(jBodyQuantileButton.isSelected())
      {
        co=co+"-"+jMainUpperField.getText()+" ";
      }
    else
      {
        co=co+jMainUpperField.getText()+" ";
      }
    co=co+String.valueOf(jMeasureCombo.getSelectedIndex()+1)+" ";
    if(jTailPhasesField.getText().compareTo("")==0 ||
      jTailPhasesField.getText().compareTo("0")==0)
      {
        co=co+"0 0 0 0";
      }
    else
      {
        co=co+jTailPhasesField.getText()+" ";
        if(jTailQuantileButton.isSelected())
          {
            co=co+"-"+jTailLowerField.getText()+" ";
            co=co+"-"+jTailUpperField.getText()+" ";
          }
        else
          {
            co=co+jTailLowerField.getText()+" ";
            co=co+jTailUpperField.getText()+" ";
          }
        co=co+jTailAdjacentField.getText();
      }
    return co;
  }

  String FittingDiscPDF() {
    String co;
    co="discfit 1 ";
    co=co+String.valueOf(jPDFFamilyCombo.getSelectedIndex()+1)+" ";
    co=co+jPDFParam1Field.getText()+" ";
    co=co+jPDFParam2Field.getText()+" ";
    co=co+jMainPhasesField.getText()+" ";
    if(jBodyQuantileButton.isSelected())
      {
        co=co+"-"+jMainUpperField.getText()+" ";
      }
    else
      {
        co=co+jMainUpperField.getText()+" ";
      }
    co=co+String.valueOf(jMeasureCombo.getSelectedIndex()+1)+" ";
    if(jTailPhasesField.getText().compareTo("")==0 ||
      jTailPhasesField.getText().compareTo("0")==0)
      {
        co=co+"0 0 0 0";
      }
    else
      {
        co=co+jTailPhasesField.getText()+" ";
        if(jTailQuantileButton.isSelected())
          {
            co=co+"-"+jTailLowerField.getText()+" ";
            co=co+"-"+jTailUpperField.getText()+" ";
          }
        else
          {
            co=co+jTailLowerField.getText()+" ";
            co=co+jTailUpperField.getText()+" ";
          }
        co=co+jTailAdjacentField.getText();
      }
    co=co+" "+jStepSizeField.getText();
    return co;
  }

  void LoadResults()
    {
      try {
      FileInputStream initials =
        new FileInputStream("TEMPORARYPhFitFILE.initials");
      BufferedReader initialsBR =
        new BufferedReader(new InputStreamReader(initials));
      jInitialArea.setText("");
      while(initialsBR.ready())
        jInitialArea.append(initialsBR.readLine()+"\n");
      jInitialArea.setCaretPosition(0);
      initials.close();
      } catch(IOException f) {}
      try {
      FileInputStream intensities =
        new FileInputStream("TEMPORARYPhFitFILE.intensities");
      BufferedReader intensitiesBR =
        new BufferedReader(new InputStreamReader(intensities));
      jIntensityArea.setText("");
      while(intensitiesBR.ready())
        jIntensityArea.append(intensitiesBR.readLine()+"\n");
      jIntensityArea.setCaretPosition(0);
      intensities.close();
      } catch(IOException f) {}
    }

  void jPerformFittingButton_actionPerformed(ActionEvent e) {

    if(jContClassButton.isSelected() && jGivenByPDFButton.isSelected())
      {
        // Some check should be implemented here;
        String command=FittingContPDF();
        PhFitFrame_Messages dlg = new PhFitFrame_Messages(command,false);
        dlg.setVisible(true);
        if(dlg.Success())
          {
            LoadResults();
          }
      }
    if(!jContClassButton.isSelected() && jGivenByPDFButton.isSelected())
      {
        // Some check should be implemented here;
        String command=FittingDiscPDF();
        PhFitFrame_Messages dlg = new PhFitFrame_Messages(command,false);
        dlg.setVisible(true);
        if(dlg.Success())
          {
            LoadResults();
          }
      }
  }

  void jDistClassButton_stateChanged(ChangeEvent e) {
    if(jDistClassButton.isSelected())
      {
        jStepSizeLabel.setForeground(Color.black);
        jStepSizeField.setEnabled(true);
      }
    else
      {
        jStepSizeLabel.setForeground(Color.gray);
        jStepSizeField.setEnabled(false);
      }
  }


  class performdemo extends Thread {

    boolean givenbypdf;
    int distindex;
    double param1,param2;
    boolean specialtail;
    boolean PHclass;
    double stepsize;
    int mainphases;
    int measure;
    double mainupper;
    boolean quantile;
    int tailphases;
    boolean tquantile;
    double taillower,tailupper,tailadjacent;
    double[] curves;

    public performdemo(boolean gbpdf,int di, double p1, double p2,
      boolean st, boolean PHc, double ss, int mp, int meas, double mu, boolean q,
      int tp, boolean tq, double tl, double tu, double ta, double[] c)
      {
      givenbypdf=gbpdf;
      distindex=di;
      param1=p1;param2=p2;
      specialtail=st;
      PHclass=PHc;
      stepsize=ss;
      mainphases=mp;
      measure=meas;
      mainupper=mu;
      quantile=q;
      tailphases=tp;
      tquantile=tq;
      taillower=tl;tailupper=tu;tailadjacent=ta;
      curves=new double[c.length];
      System.arraycopy(c,0,curves,0,c.length);
      }

    void FlashLabel(JComponent l) {
      int base=Constants.blinkspeed;
      try {
      for(int i1=0;i1<Constants.blinknumber;i1++)
        {
          l.setForeground(Color.gray);
          this.sleep(base);
          l.setForeground(Color.black);
          this.sleep(base);
        }
      } catch (InterruptedException e){}
    }

    public void run() {
      boolean lx,ly;

      // setting distribution input
      jDistributionInputTabbed.setSelectedIndex(0);
      FlashLabel(jGivenByLabel);
      jGivenBySamplesButton.setSelected(!givenbypdf);
      jGivenByPDFButton.setSelected(givenbypdf);
      FlashLabel(jPDFFamilyLabel);
      jPDFFamilyCombo.setSelectedIndex(distindex);
      FlashLabel(jPDFParam1Label);
      jPDFParam1Field.setText(String.valueOf(param1));
      FlashLabel(jPDFParam2Label);
      jPDFParam2Field.setText(String.valueOf(param2));
      try { this.sleep(200); } catch (InterruptedException e){}
      jDistributionInputTabbed.setSelectedIndex(1);
      FlashLabel(jTailSpecialCheck);
      jTailSpecialCheck.setSelected(specialtail);
      try { this.sleep(200); } catch (InterruptedException e){}

      // setting fitting parameters
      jFittingInputTabbed.setSelectedIndex(0);
      FlashLabel(jAppliedClassLabel);
      jContClassButton.setSelected(PHclass);
      jDistClassButton.setSelected(!PHclass);
      if(jDistClassButton.isSelected())
        {
        FlashLabel(jStepSizeLabel);
        jStepSizeField.setText(String.valueOf(stepsize));
        }
      try { this.sleep(200); } catch (InterruptedException e){}
      jFittingInputTabbed.setSelectedIndex(1);
      FlashLabel(jMainPhasesLabel);
      jMainPhasesField.setText(String.valueOf(mainphases));
      FlashLabel(jMeasureLabel);
      jMeasureCombo.setSelectedIndex(measure);
      FlashLabel(jMainUpperLabel);
      jMainUpperField.setText(String.valueOf(mainupper));
      FlashLabel(jMainUpperModeLabel);
      jBodyQuantileButton.setSelected(quantile);
      jBodyLimitButton.setSelected(!quantile);
      try { this.sleep(200); } catch (InterruptedException e){}
      jFittingInputTabbed.setSelectedIndex(2);
      FlashLabel(jTailPhasesLabel);
      jTailPhasesField.setText(String.valueOf(tailphases));
      FlashLabel(jTailLowerLabel);
      jTailLowerField.setText(String.valueOf(taillower));
      FlashLabel(jTailUpperLabel);
      jTailUpperField.setText(String.valueOf(tailupper));
      FlashLabel(jTailLimitsModeLabel);
      jTailQuantileButton.setSelected(tquantile);
      jTailLimitButton.setSelected(!tquantile);
      FlashLabel(jTailAdjacentLabel);
      jTailAdjacentField.setText(String.valueOf(tailadjacent));

      // perform fitting
      FlashLabel(jPerformFittingButton);
      String command="";
      if(PHclass) command=FittingContPDF();
      if(!PHclass) command=FittingDiscPDF();
      PhFitFrame_Messages dlgperform = new PhFitFrame_Messages(command,true);
      dlgperform.setVisible(true);
      LoadResults();

      // create curves
      for(int i1=0;i1<curves.length/6;i1++)
      {
        lx=true;
        if(curves[i1*6+4]==0.0) lx=false;
        ly=true;
        if(curves[i1*6+4]==0.0) ly=false;
        FlashLabel(jCurveCreateLabel);
        jCurveCreateCombo.setSelectedIndex((int) curves[i1*6]);
        FlashLabel(jCreateButton);
        PhFitFrame_CreateBox dlgcreate1 = new PhFitFrame_CreateBox(
          PHclass,jPDFFamilyCombo.getSelectedIndex(),
          jPDFParam1Field.getText(),jPDFParam2Field.getText(),
          imagecounter,jCurveCreateCombo.getSelectedIndex()+2,
          true,curves[i1*6+1],curves[i1*6+2],(int)curves[i1*6+3],lx,ly,stepsize);
        dlgcreate1.setVisible(true);
        jTheCurveItselfLabel.setIcon(
          new ImageIcon("TEMPORARYPhFitImage"+imagecounter+".gif"));
        imagecounter=imagecounter+1;
        jTheCurveItselfLabel.setText("");
        try {
        this.sleep(2000);
        } catch (InterruptedException e){}
      }
    }
  }

  void jDemoMenu_actionPerformed(ActionEvent e) {
    PhFitFrame_Demo dlg = new PhFitFrame_Demo(this);
    dlg.setVisible(true);

    int wantdemo=dlg.WantDemo_Demo();
    int demotype=dlg.DemoType_Demo();
    int demo=dlg.Demo_Demo();
    if(wantdemo==1)
      {
        // continuous PDF demos
        if(demotype==0)
          {
            switch(demo)
            {
            // weibull
            case 0:
              double[] c0={0.0,0.0,4.0,50.0,0.0,0.0,
                          1.0,0.0,3.0,50.0,0.0,0.0,
                          2.0,0.0,5.0,50.0,0.0,0.0,
                          3.0,0.0,3.0,50.0,0.0,0.0,
                          4.0,0.0,3.0,50.0,0.0,0.0,
                          5.0,0.0,4.0,50.0,0.0,0.0,
                          6.0,0.0,3.0,50.0,0.0,0.0,
                          7.0,0.0,4.0,50.0,0.0,0.0
                          };
              performdemo p0 = new performdemo(true,0,1.0,1.5,false,true,0.0,4,
                0,0.001,true,0,false,0.0,0.0,0.0,c0);
              p0.start();
              break;
            case 1:
              double[] c1={0.0,0.0,4.0,50.0,0.0,0.0,
                          1.0,0.0,3.0,50.0,0.0,0.0,
                          2.0,0.0,5.0,50.0,0.0,0.0,
                          3.0,0.0,3.0,50.0,0.0,0.0,
                          4.0,0.0,3.0,50.0,0.0,0.0,
                          5.0,0.0,4.0,50.0,0.0,0.0,
                          6.0,0.0,3.0,50.0,0.0,0.0,
                          7.0,0.0,4.0,50.0,0.0,0.0
                          };
              performdemo p1 = new performdemo(true,1,1.0,0.2,false,true,0.0,4,
                0,0.001,true,0,false,0.0,0.0,0.0,c1);
              p1.start();
              break;
            case 2:
              double[] c2={0.0,0.0,4.0,50.0,0.0,0.0,
                          1.0,0.0,3.0,50.0,0.0,0.0,
                          2.0,0.0,5.0,50.0,0.0,0.0,
                          3.0,0.0,3.0,50.0,0.0,0.0,
                          4.0,0.0,3.0,50.0,0.0,0.0,
                          5.0,0.0,4.0,50.0,0.0,0.0,
                          6.0,0.0,3.0,50.0,0.0,0.0,
                          7.0,0.0,4.0,50.0,0.0,0.0
                          };
              performdemo p2 = new performdemo(true,2,1.0,2.0,false,true,0.0,4,
                0,3,false,0,false,0.0,0.0,0.0,c2);
              p2.start();
              break;
            case 3:
              double[] c3={0.0,0.0,4.0,50.0,0.0,0.0,
                          0.0,10.0,100000.0,100.0,1.0,1.0,
                          2.0,10.0,100000.0,100.0,1.0,1.0,};
              performdemo p3 = new performdemo(true,3,1.5,4,false,true,0.0,4,
                0,5,false,6,false,8.0,100000.0,0.5,c3);
              p3.start();
              break;
            case 4:
              double[] c4={0.0,0.0,5.0,100.0,0.0,0.0,
                          0.0,10.0,100000.0,100.0,1.0,1.0,
                          2.0,10.0,100000.0,100.0,1.0,1.0,};
              performdemo p4 = new performdemo(true,4,1.5,2.0,false,true,0.0,4,
                1,5.0,false,6,false,10.0,100000.0,0.5,c4);
              p4.start();
              break;
            }
          }
        // discrete PDF demos
        if(demotype==1)
          {
            switch(demo)
            {
            // weibull
            case 0:
              double[] cd0={0.0,0.0,4.0,50.0,0.0,0.0,
                          1.0,0.0,3.0,50.0,0.0,0.0,
                          2.0,0.0,5.0,50.0,0.0,0.0,
                          3.0,0.0,3.0,50.0,0.0,0.0,
                          4.0,0.0,3.0,50.0,0.0,0.0,
                          5.0,0.0,4.0,50.0,0.0,0.0,
                          6.0,0.0,3.0,50.0,0.0,0.0,
                          7.0,0.0,4.0,50.0,0.0,0.0
                          };
              performdemo pd0 = new performdemo(true,0,1.0,1.5,false,false,0.1,4,
                0,0.001,true,0,false,0.0,0.0,0.0,cd0);
              pd0.start();
              break;
            case 4:
              double[] cd4={0.0,0.0,4.0,100.0,0.0,0.0,
                          0.0,10.0,100000.0,100.0,1.0,1.0,
                          2.0,10.0,100000.0,100.0,1.0,1.0,};
              performdemo pd4 = new performdemo(true,4,1.5,2.0,false,false,0.1,8,
                1,5.0,false,6,false,10.0,100000.0,0.5,cd4);
              pd4.start();
            }
          }

      }
  }

  void FileCopy(String source, String dest) {
    try {
    FileInputStream textuals =
      new FileInputStream(source);
    FileOutputStream textualsout =
      new FileOutputStream(dest);
    BufferedReader textualBR =
      new BufferedReader(new InputStreamReader(textuals));
    PrintWriter tout = new PrintWriter(textualsout);
    while(textualBR.ready())
      tout.print(textualBR.readLine()+"\n");
    textualBR.close();
    tout.close();
    textualsout.close();
    textuals.close();
    } catch (Exception a) {}
  }

  void jSaveButton_actionPerformed(ActionEvent e) {
    String fn;
    JFileChooser chooser = new JFileChooser();
    int returnVal = chooser.showOpenDialog(this);
    if(returnVal == JFileChooser.APPROVE_OPTION)
      {
        fn=chooser.getSelectedFile().getPath();
      if(jSaveCombo.getSelectedIndex()==0)
        {
          FileCopy("TEMPORARYPhFitFILE.textual",fn);
        }
      if(jSaveCombo.getSelectedIndex()==1)
        {
          FileCopy("TEMPORARYPhFitFILE.matrix",fn);
        }
     }
  }

  void jSaveCurveButton_actionPerformed(ActionEvent e) {
    String fn;
    JFileChooser chooser = new JFileChooser();
    int returnVal = chooser.showOpenDialog(this);
    if(returnVal == JFileChooser.APPROVE_OPTION)
      {
        fn=chooser.getSelectedFile().getPath();
        FileCopy("TEMPORARYPhFitFILE.data",fn+".data");
        FileCopy("TEMPORARYPhFitFILE.gnu",fn+".gnu");
      }
  }
}